package unisa.gps.etour.gui.operatoreagenzia;

import java.awt.*;
import java.awt.event.*;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;

import javax.imageio.ImageIO;
import javax.swing.*;
import javax.swing.filechooser.FileFilter;
import javax.swing.border.*;

import unisa.gps.etour.gui.operatoreagenzia.Home;

/**
 * Questa classe realizza il pannello per la dialog per l'inserimento di un
 * nuovo banner o per la sostituzione di un banner selezionato.
 * 
 * @version 1.0
 * @author Mario Gallo 
 * 
 * &#169; 2007 eTour Project - Copyright by SE@SA Lab DMI -
 *         University of Salerno
 */
public class BannerDialog extends JPanel
{
	private final static Dimension bannerSize = new Dimension(140, 40);
	private JLabel anteprima = null;
	private JButton btnCarica = null;
	private JPanel questo;

	/**
	 * Questo è il costruttore di default.
	 */
	public BannerDialog()
	{
		super(null);
		questo = this;
		setPreferredSize(new Dimension(420, 160));
		anteprima = new JLabel();
		anteprima.setBounds(new Rectangle(40, 20, 250, 60));
		anteprima.setBorder(BorderFactory.createTitledBorder(BorderFactory
				.createLineBorder(new Color(51, 102, 255), 3),
				"Anteprima Banner", TitledBorder.DEFAULT_JUSTIFICATION,
				TitledBorder.DEFAULT_POSITION,
				new Font("Dialog", Font.BOLD, 12), new Color(0, 102, 204)));
		btnCarica = new JButton();
		btnCarica.setBounds(new Rectangle(320, 30, 50, 40));
		btnCarica.setIcon(new ImageIcon(getClass().getResource(
				Home.URL_IMAGES + "ApriFile.png")));
		btnCarica.addActionListener(new ActionListener()
		{

			public void actionPerformed(ActionEvent arg0)
			{
				File f = apriFileDialog();

				if (f == null)
				{
					return;
				}

				BufferedImage img = null;
				try
				{
					img = ImageIO.read(f);
					if (img.getWidth() > bannerSize.width
							|| img.getHeight() > bannerSize.height)
					{
						img = img.getSubimage(0, 0, bannerSize.width,
								bannerSize.height);
					}
				}
				catch (IOException e)
				{
					e.printStackTrace();
				}
				ImageIcon n = new ImageIcon(img);
				anteprima.setIcon(n);
				anteprima.repaint();
			}
		});

		JLabel txtAttenzione = new JLabel("Attenzione!");
		txtAttenzione.setIcon(new ImageIcon(getClass().getResource(
				Home.URL_IMAGES + "warning16.png")));
		txtAttenzione.setBounds(10, 85, 100, 30);
		JLabel txtTesto = new JLabel(
				"<html><ul><li>L'immagine per il banner non può superare i<b><font color=\"red\">"
						+ bannerSize.width
						+ "x"
						+ bannerSize.height
						+ "</font></b> pixels."
						+ "<li>Le immagini di grandezza maggiore verranno ridimensionate.</ul></html>");
		txtTesto.setBounds(0, 90, 420, 80);
		add(txtTesto, null);
		add(txtAttenzione, null);
		add(anteprima, null);
		add(btnCarica, null);

	}

	/**
	 * Questo metodo inizializza l'immagine contenuta nell'anteprima.
	 * 
	 * @param pBanner ImageIcon - un'immagine di un banner.
	 */
	public void setSelectedBanner(ImageIcon pBanner)
	{
		anteprima.setIcon(pBanner);
	}

	/**
	 * Questo metodo restituisce l'immagine contenuta nell'anteprima.
	 * 
	 * @return ImageIcon - l'immagine del banner.
	 */
	public ImageIcon getSelectedBanner()
	{
		return (ImageIcon) anteprima.getIcon();
	}

	/**
	 * Questo metodo apre la dialog per la scelta di un file dal disco.
	 * 
	 * @return
	 * <ul>
	 * <li>File - il file selezionato.
	 * <li>null - se non è stato selezionato nessun file.
	 * </ul>
	 */
	private File apriFileDialog()
	{
		JFileChooser apriFile = new JFileChooser();
		apriFile.setDialogTitle("Seleziona una nuova immagine");
		apriFile.setAcceptAllFileFilterUsed(false);
		apriFile.setMultiSelectionEnabled(false);

		// File Filter per la finestra di apertura del file.
		apriFile.setFileFilter(new FileFilter()
		{
			public boolean accept(File arg0)
			{
				if (arg0.isDirectory())
					return true;
				String nome = arg0.getName().toLowerCase();
				if (nome.endsWith("jpg") || nome.endsWith("gif")
						|| nome.endsWith("png"))
					return true;
				else
					return false;
			}

			public String getDescription()
			{
				return "Immagini (*.PNG, *.GIF, *.JPG)";
			}

		});
		int returnVal = apriFile.showOpenDialog(questo);
		if (returnVal == JFileChooser.APPROVE_OPTION)
		{
			return apriFile.getSelectedFile();
		}
		else
			return null;

	}

}
