package unisa.gps.etour.gui.operatoreagenzia;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;
import unisa.gps.etour.bean.BeanBeneCulturale;
import unisa.gps.etour.bean.BeanTag;
import unisa.gps.etour.control.GestioneBeniCulturali.IGestioneBeniCulturaliAgenzia;
import unisa.gps.etour.control.GestioneBeniCulturali.IGestioneBeniCulturaliComune;
import unisa.gps.etour.control.GestioneTag.IGestioneTagComune;
import unisa.gps.etour.gui.DeskManager;
import unisa.gps.etour.gui.HelpManager;
import unisa.gps.etour.gui.operatoreagenzia.tables.MediaVotiRenderer;
import unisa.gps.etour.gui.operatoreagenzia.tables.Punto3DRenderer;
import unisa.gps.etour.gui.operatoreagenzia.tables.ScrollableTable;
import unisa.gps.etour.gui.operatoreagenzia.tables.SitoTableModel;
import unisa.gps.etour.util.Punto3D;
import java.io.FileNotFoundException;
import java.rmi.RemoteException;
import java.rmi.registry.LocateRegistry;
import java.rmi.registry.Registry;
import java.util.ArrayList;

/**
 * Classe che realizza l'interfaccia per la gestione di beni culturali lato
 * Operatore Agenzia.
 * 
 * @author Mario Gallo
 * @version 0.8 
 * &#169; 2007 eTour Project - Copyright by SE@SA Lab DMI -
 *          University of Salerno
 */
public class BeniCulturali extends JInternalFrame
{
	private JDesktopPane jDesktopPane;
	private JPanel jContentPane = null;
	private JToolBar BCToolbar = null;
	private JButton btnNuovoBC = null;
	private JButton btnSchedaBC = null;
	private JButton btnEliminaBC = null;
	private JButton btnModificaBC = null;
	private JPanel rightPanel = null;
	private JPanel searchPanel = null;
	private JPanel helpPanel = null;
	private JScrollPane jScrollPane = null;
	private JTable tableBC = null;
	private TagPanel pannelloTag = null;
	private JTextPane textGuida = null;
	private JTextField nomeBC = null;
	private JButton btnRicerca = null;
	private JButton btnAzzera = null;
	private DeskManager desktopManager;
	private ArrayList<SchedaBC> figli;
	private SitoTableModel tableModel;
	private HelpManager bcHelp;
	private IGestioneBeniCulturaliAgenzia gestioneBC;
	private IGestioneTagComune tag;
	protected IGestioneBeniCulturaliComune ricercaBC;

	/**
	 * Questo &egrave; il costruttore di default.
	 */
	public BeniCulturali()
	{
		super("Beni Culturali");
		setPreferredSize(Home.CHILD_SIZE);
		frameIcon = new ImageIcon(getClass().getResource(
				Home.URL_IMAGES + "BC.png"));
		setDefaultCloseOperation(JInternalFrame.DO_NOTHING_ON_CLOSE);
		closable = true;
		resizable = true;
		iconable = true;
		maximizable = true;

		// Setting up dell'help manager per beni culturali.
		textGuida = new JTextPane();
		try
		{
			bcHelp = new HelpManager(Home.URL_HELP + "BeniCulturali.txt",
					textGuida);
		}
		catch (FileNotFoundException e)
		{
			textGuida
					.setText("<html><b>Guida in linea non disponibile!</b></html>");
		}
		setContentPane(getJContentPane());
		figli = new ArrayList<SchedaBC>();
		addInternalFrameListener(new InternalFrameAdapter()
		{
			/*
			 * All'inserimento del frame nel desktop pane recupera desktop pane
			 * e desktop manager e inizializza gli oggetti remoti per la
			 * gestione dei beni culturali.
			 */

			public void internalFrameOpened(InternalFrameEvent pEvent)
			{
				JInternalFrame frame = pEvent.getInternalFrame();
				jDesktopPane = frame.getDesktopPane();
				desktopManager = (DeskManager) jDesktopPane.getDesktopManager();

				// Setting up degli oggetti remoti per la gestione dei beni
				// culturali.
				try
				{
					Registry reg = LocateRegistry.getRegistry(Home.HOST);
					gestioneBC = (IGestioneBeniCulturaliAgenzia) reg
							.lookup("GestioneBeniCulturaliAgenzia");
					tag = (IGestioneTagComune) reg.lookup("GestioneTagComune");
					ricercaBC = (IGestioneBeniCulturaliComune) reg
							.lookup("GestioneBeniCulturaliComune");
					// Caricamento dei dati.
					caricaTabella(false);
					caricaTags();
				}
				/*
				 * Due eccezioni: RemoteException e NotBoundException. Il
				 * risultato è lo stesso. La gestione non è operabile e la
				 * finestra dopo il messaggio di errore verrà chiusa.
				 */
				catch (Exception ex)
				{
					JLabel errore = new JLabel(
							"<html><h2>Impossibile comunicare con il server di eTour.</h2>"
									+ "<h3><u>La finestra di gestione richiesta verrà chiusa.</u></h3>"
									+ "<p><b>Possibili Cause:</b>"
									+ "<ul><li>Nessuna connessione alla rete.</li>"
									+ "<li>Server inattivo.</li>"
									+ "<li>Server intasato.</li></ul>"
									+ "<p>Si prega di riprovare in un secondo momento.</p>"
									+ "<p>Se l'errore dovesse persistere, si prega di contattare l'assistenza tecnica.</p>"
									+ "<p>Ci scusiamo per il disagio.</html>");
					ImageIcon err = new ImageIcon(getClass().getResource(
							Home.URL_IMAGES + "error48.png"));
					JOptionPane.showMessageDialog(jDesktopPane, errore,
							"Errore!", JOptionPane.ERROR_MESSAGE, err);
					frame.dispose();
				}
			}

			/*
			 * Alla chiusura del frame viene visualizzato il dialogo di
			 * conferma.
			 */
			public void internalFrameClosing(InternalFrameEvent pEvent)
			{
				// Creazione del dialogo di conferma.
				JPanel root = new JPanel(new BorderLayout());
				JLabel message = new JLabel(
						"Sei sicuro di voler chiudere la finestra di gestione dei beni culturali?");
				message.setFont(new Font("Dialog", Font.BOLD, 14));
				JLabel avviso = new JLabel(
						"N.B. Verranno chiuse tutte le finestre aperte da questa gestione.",
						SwingConstants.CENTER);
				avviso.setIcon(new ImageIcon(getClass().getResource(
						Home.URL_IMAGES + "warning16.png")));
				root.add(message, BorderLayout.NORTH);
				root.add(avviso, BorderLayout.SOUTH);
				String[] options = { "Chiudi", "Annulla" };
				int choice = JOptionPane.showInternalOptionDialog(jContentPane,
						root, "Conferma chiusura Beni Culturali",
						JOptionPane.OK_CANCEL_OPTION,
						JOptionPane.QUESTION_MESSAGE, frameIcon, options,
						options[1]);
				/*
				 * Se viene confermata la chiusura della gestione, tutte le
				 * classi "figlie" vengono chiuse.
				 */
				if (choice == JOptionPane.OK_OPTION)
				{
					for (int i = 0; i < figli.size(); i++)
					{
						figli.get(i).dispose();
					}
					pEvent.getInternalFrame().dispose();
				}
			}

		});
	}

	/**
	 * Aggiorna il modello corrente della table dei beni culturali con il bean
	 * del bene culturale fornito in input.
	 * 
	 * @param pBc BeanBeneCulturale - il bean con il quale aggiornare il
	 *            modello.
	 */
	protected void updateTableModel(BeanBeneCulturale pBc)
	{
		tableModel.updateBC(pBc);
	}

	/**
	 * Chiude la scheda bene culturale selezionata.
	 * 
	 * @param pScheda SchedaBC - la scheda bene culturale da chiudere.
	 */
	protected void closeScheda(SchedaBC pScheda)
	{
		figli.remove(pScheda);
		pScheda.dispose();
	}

	/**
	 * Questo metodo inizializza il content pane del frame.
	 * 
	 * @return javax.swing.JPanel - il content pane.
	 */
	private JPanel getJContentPane()
	{
		if (jContentPane == null)
		{
			jContentPane = new JPanel();
			jContentPane.setLayout(new BorderLayout());
			jContentPane.add(getBCToolbar(), BorderLayout.NORTH);
			jContentPane.add(getPannelloDestro(), BorderLayout.EAST);
			jContentPane.add(getJScrollPane(), BorderLayout.CENTER);
		}
		return jContentPane;
	}

	/**
	 * Questo metodo inizializza la toolbar per le funzionalit&agrave; relative alla
	 * gestione dei beni culturali.
	 * 
	 * @return javax.swing.JToolBar - la toolbar per la gestione dei beni
	 *         culturali.
	 */
	private JToolBar getBCToolbar()
	{
		if (BCToolbar == null)
		{
			BCToolbar = new JToolBar();
			BCToolbar.setPreferredSize(new Dimension(1, 50));
			BCToolbar.setFloatable(false);
			BCToolbar.setOrientation(JToolBar.HORIZONTAL);
			BCToolbar.setLayout(null);
			BCToolbar.add(getBtnNuovoBC());
			BCToolbar.addSeparator();
			BCToolbar.add(getBtnModificaBC());
			BCToolbar.addSeparator();
			BCToolbar.add(getBtnSchedaBC());
			BCToolbar.addSeparator();
			BCToolbar.add(getBtnEliminaBC());
		}
		return BCToolbar;
	}

	/**
	 * Questo metodo inizializza il pulsante per l'inserimento di un nuovo bene
	 * culturale.
	 * 
	 * @return javax.swing.JButton - il pulsante per l'inserimento.
	 */
	private JButton getBtnNuovoBC()
	{
		if (btnNuovoBC == null)
		{
			btnNuovoBC = new JButton();
			btnNuovoBC.setText("<html>Nuovo<br>Bene&nbsp;Culturale</html>");
			btnNuovoBC.setBounds(5, 5, 140, 40);
			btnNuovoBC.setIcon(new ImageIcon(getClass().getResource(
					Home.URL_IMAGES + "nuovoBC.png")));
			btnNuovoBC.setName("btnNuovoBC");
			btnNuovoBC.addMouseListener(bcHelp);
			btnNuovoBC
					.setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
			btnNuovoBC.addActionListener(new ActionListener()
			{
				public void actionPerformed(ActionEvent pEvent)
				{
					// Apre una scheda per l'inserimento di un nuovo bene
					// culturale.
					openScheda(null, false);
				}
			});
		}
		return btnNuovoBC;
	}

	/**
	 * Questo metodo inizializza il pulsante per accedere alla scheda di un bene
	 * culturale selezionato.
	 * 
	 * @return javax.swing.JButton - il pulsante per la scheda.
	 */
	private JButton getBtnSchedaBC()
	{
		if (btnSchedaBC == null)
		{
			btnSchedaBC = new JButton();
			btnSchedaBC.setBounds(305, 5, 140, 40);
			btnSchedaBC.setText("<html>Scheda<br>Bene&nbsp;Culturale</html>");
			btnSchedaBC.setVerticalTextPosition(SwingConstants.TOP);
			btnSchedaBC.setIcon(new ImageIcon(getClass().getResource(
					Home.URL_IMAGES + "scheda.png")));
			btnSchedaBC.setEnabled(false);
			btnSchedaBC.setCursor(Cursor
					.getPredefinedCursor(Cursor.HAND_CURSOR));
			btnSchedaBC.setName("btnSchedaBC");
			btnSchedaBC.addMouseListener(bcHelp);
			btnSchedaBC.addActionListener(new ActionListener()
			{
				public void actionPerformed(ActionEvent pEvent)
				{
					int selectedRow = tableBC.getSelectedRow();
					BeanBeneCulturale toDo = null;
					try
					{
						toDo = gestioneBC.ottieniBeneCulturale(tableModel
								.getID(selectedRow));
						// Apre la scheda del bene culturale con la modifica
						// disattivata.
						openScheda(toDo, false);
					}
					catch (Exception ex)
					{
						JLabel errore = new JLabel(
								"<html><h2>Impossibile comunicare con il server di eTour.</h2>"
										+ "<h3><u>La scheda richiesta non può essere caricata.</u></h3>"
										+ "<p>Si prega di riprovare in un secondo momento.</p>"
										+ "<p>Se l'errore dovesse persistere, si prega di contattare l'assistenza tecnica.</p>"
										+ "<p>Ci scusiamo per il disagio.</html>");
						ImageIcon err = new ImageIcon(getClass().getResource(
								Home.URL_IMAGES + "error48.png"));
						JOptionPane.showMessageDialog(jDesktopPane, errore,
								"Errore!", JOptionPane.ERROR_MESSAGE, err);
					}
				}
			});
		}
		return btnSchedaBC;
	}

	/**
	 * Questo metodo inizializza il pulsante per accedere alla modifica di un
	 * bene culturale.
	 * 
	 * @return javax.swing.JButton - il pulsante per la scheda.
	 */
	private JButton getBtnModificaBC()
	{
		if (btnModificaBC == null)
		{
			btnModificaBC = new JButton();
			btnModificaBC.setBounds(155, 5, 140, 40);
			btnModificaBC
					.setText("<html>Modifica&nbsp;Dati<br>Bene&nbsp;Culturale</html>");
			btnModificaBC.setEnabled(false);
			btnModificaBC.setCursor(Cursor
					.getPredefinedCursor(Cursor.HAND_CURSOR));
			btnModificaBC.setIcon(new ImageIcon(getClass().getResource(
					Home.URL_IMAGES + "ModificaBC32.png")));
			btnModificaBC.setName("btnModificaBC");
			btnModificaBC.addMouseListener(bcHelp);
			btnModificaBC.addActionListener(new ActionListener()
			{
				public void actionPerformed(ActionEvent pEvent)
				{
					int selectedRow = tableBC.getSelectedRow();
					BeanBeneCulturale toDo = null;
					try
					{
						toDo = gestioneBC.ottieniBeneCulturale(tableModel
								.getID(selectedRow));
						// Apre la scheda del bene culturale con la modifica
						// attivata.
						openScheda(toDo, true);
					}
					catch (Exception ex)
					{
						JLabel errore = new JLabel(
								"<html><h2>Impossibile comunicare con il server di eTour.</h2>"
										+ "<h3><u>La scheda richiesta non può essere caricata.</u></h3>"
										+ "<p>Si prega di riprovare in un secondo momento.</p>"
										+ "<p>Se l'errore dovesse persistere, si prega di contattare l'assistenza tecnica.</p>"
										+ "<p>Ci scusiamo per il disagio.</html>");
						ImageIcon err = new ImageIcon(getClass().getResource(
								Home.URL_IMAGES + "error48.png"));
						JOptionPane.showMessageDialog(jDesktopPane, errore,
								"Errore!", JOptionPane.ERROR_MESSAGE, err);
					}

				}
			});
		}
		return btnModificaBC;
	}

	/**
	 * Questo metodo inizializza il pulsante per l'eliminazione di un bene
	 * culturale selezionato.
	 * 
	 * @return javax.swing.JButton - il pulsante per l'eliminazione.
	 */
	private JButton getBtnEliminaBC()
	{
		if (btnEliminaBC == null)
		{
			btnEliminaBC = new JButton();
			btnEliminaBC.setBounds(455, 5, 140, 40);
			btnEliminaBC.setText("<html>Elimina<br>Bene Culturale</html>");
			btnEliminaBC.setVerticalTextPosition(SwingConstants.TOP);
			btnEliminaBC.setCursor(Cursor
					.getPredefinedCursor(Cursor.HAND_CURSOR));
			btnEliminaBC.setIcon(new ImageIcon(getClass().getResource(
					Home.URL_IMAGES + "EliminaBC32.png")));
			btnEliminaBC.setCursor(Cursor
					.getPredefinedCursor(Cursor.HAND_CURSOR));
			btnEliminaBC.setEnabled(false);
			btnEliminaBC.addActionListener(new ActionListener()
			{
				public void actionPerformed(ActionEvent pEvent)
				{
					int selectedRow = tableBC.getSelectedRow();
					String nome = (String) tableModel
							.getValueAt(selectedRow, 0);

					// Creazione del dialogo di conferma eliminazione.
					JPanel root = new JPanel(new BorderLayout());
					JLabel message = new JLabel(
							"Sei sicuro di voler eliminare il bene culturale "
									+ nome + "?");
					message.setFont(new Font("Dialog", Font.BOLD, 14));
					JLabel avviso = new JLabel(
							"I dati eliminati non potranno essere più recuperati.",
							SwingConstants.CENTER);
					avviso.setIcon(new ImageIcon(getClass().getResource(
							Home.URL_IMAGES + "warning16.png")));
					root.add(message, BorderLayout.NORTH);
					root.add(avviso, BorderLayout.SOUTH);
					String[] options = { "Elimina", "Annulla" };
					int scelta = JOptionPane.showInternalOptionDialog(
							jContentPane, root, "Conferma Eliminazione",
							JOptionPane.YES_NO_OPTION,
							JOptionPane.QUESTION_MESSAGE,
							new ImageIcon(getClass().getResource(
									Home.URL_IMAGES + "EliminaBC48.png")),
							options, options[1]);
					// Se l'eliminazione viene confermata, elimina il bene
					// culturale selezionato.
					if (scelta == JOptionPane.YES_OPTION)
					{
						try
						{
							gestioneBC.cancellaBeneCulturale(tableModel
									.getID(selectedRow));
							tableModel.removeSito(selectedRow);
							JLabel confirm = new JLabel("Il bene culturale "
									+ nome + " è stato eliminato.");
							confirm.setFont(new Font("Dialog", Font.BOLD, 14));
							JOptionPane.showInternalMessageDialog(jContentPane,
									confirm, "Beni culturali",
									JOptionPane.OK_OPTION, new ImageIcon(
											getClass().getResource(
													Home.URL_IMAGES
															+ "ok32.png")));
						}
						catch (Exception ex)
						{
							JLabel errore = new JLabel(
									"<html><h2>Impossibile comunicare con il server di eTour.</h2>"
											+ "<h3><u>L'operazione di eliminazione richiesta non può essere completata.</u></h3>"
											+ "<p>Si prega di riprovare in un secondo momento.</p>"
											+ "<p>Se l'errore dovesse persistere, si prega di contattare l'assistenza tecnica.</p>"
											+ "<p>Ci scusiamo per il disagio.</html>");
							ImageIcon err = new ImageIcon(getClass()
									.getResource(
											Home.URL_IMAGES + "error48.png"));
							JOptionPane.showMessageDialog(jDesktopPane, errore,
									"Errore!", JOptionPane.ERROR_MESSAGE, err);
						}
					}
				}
			});
			btnEliminaBC.setName("btnEliminaBC");
			btnEliminaBC.addMouseListener(bcHelp);

		}
		return btnEliminaBC;

	}

	/**
	 * Questo metodo inizializza il lato destro dell'interfaccia.
	 * 
	 * @return javax.swing.JPanel - il pannello destro dell'intefaccia.
	 */
	private JPanel getPannelloDestro()
	{
		if (rightPanel == null)
		{
			rightPanel = new JPanel();
			rightPanel.setLayout(new BorderLayout());
			rightPanel.add(getHelpPanel(), BorderLayout.SOUTH);
			rightPanel.add(getSearchPanel(), BorderLayout.CENTER);
		}
		return rightPanel;
	}

	/**
	 * Questo metodo inizializza il pannello per la ricerca di un bene
	 * culturale.
	 * 
	 * @return javax.swing.JPanel - il pannello di ricerca.
	 */
	private JPanel getSearchPanel()
	{
		if (searchPanel == null)
		{
			GridBagConstraints g = new GridBagConstraints();
			searchPanel = new JPanel(new GridBagLayout());
			searchPanel.setBorder(BorderFactory.createTitledBorder(
					BorderFactory.createLineBorder(new Color(51, 102, 255), 3),
					"Ricerca Bene Culturale",
					TitledBorder.DEFAULT_JUSTIFICATION,
					TitledBorder.DEFAULT_POSITION, new Font("Dialog",
							Font.BOLD, 12), new Color(0, 102, 204)));
			g.anchor = GridBagConstraints.CENTER;
			// Top - Left - Bottom - Right
			g.insets = new Insets(5, 5, 5, 5);
			g.gridwidth = 2;
			g.gridx = 0;
			g.gridy = 0;
			searchPanel.add(new JLabel("Nome Bene Culturale:"), g);
			g.gridy = 1;
			nomeBC = new JTextField();
			nomeBC.setColumns(12);
			nomeBC.setName("nomeBC");
			nomeBC.addMouseListener(bcHelp);
			searchPanel.add(nomeBC, g);
			g.gridy = 2;
			searchPanel.add(new JLabel("Seleziona tag di ricerca:"), g);
			g.fill = GridBagConstraints.VERTICAL;
			g.gridy = 3;
			g.weightx = 1.0;
			g.weighty = 1.0;
			g.insets = new Insets(5, 5, 10, 5);
			pannelloTag = new TagPanel();
			pannelloTag.setPreferredSize(new Dimension(180, 40));
			pannelloTag.setName("pannelloTag");
			pannelloTag.addMouseListener(bcHelp);
			searchPanel.add(pannelloTag, g);
			g.insets = new Insets(5, 5, 5, 5);
			g.weightx = 0;
			g.weighty = 0;
			g.gridwidth = 1;
			g.gridy = 4;
			g.fill = GridBagConstraints.NONE;
			searchPanel.add(getBtnRicerca(), g);
			g.gridx = 1;
			searchPanel.add(getBtnAzzera(), g);

		}
		return searchPanel;
	}

	/**
	 * Questo metodo inizializza il pannello contenente la guida in linea.
	 * 
	 * @return javax.swing.JPanel - il pannello della guida.
	 */
	private JPanel getHelpPanel()
	{
		if (helpPanel == null)
		{
			helpPanel = new JPanel();
			helpPanel.setLayout(new BorderLayout());
			helpPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory
					.createLineBorder(new Color(51, 102, 255), 3),
					"Guida in Linea", TitledBorder.DEFAULT_JUSTIFICATION,
					TitledBorder.DEFAULT_POSITION, new Font("Dialog",
							Font.BOLD, 12), new Color(0, 102, 204)));
			textGuida.setPreferredSize(new Dimension(100, 80));
			textGuida.setContentType("text/html");
			textGuida
					.setText("<html>Muovi il puntatore del mouse su un controllo "
							+ "d'interesse per visualizzare la guida sensibile al contesto.</html>");
			textGuida.setEditable(false);
			textGuida.setOpaque(false);
			helpPanel.add(textGuida, BorderLayout.CENTER);
		}
		return helpPanel;
	}

	/**
	 * Questo metodo inizializza lo scroll pane e la table dei beni culturali.
	 * 
	 * @return javax.swing.JScrollPane - lo scrollPane.
	 */
	private JScrollPane getJScrollPane()
	{
		if (jScrollPane == null)
		{
			if (tableBC == null)
			{
				tableModel = new SitoTableModel();
				tableBC = new ScrollableTable(tableModel);
				tableBC.setRowHeight(32);
				tableBC.setDefaultRenderer(Double.class,
						new MediaVotiRenderer());
				tableBC
						.setDefaultRenderer(Punto3D.class,
								new Punto3DRenderer());
				tableBC.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
				tableBC.setSelectionForeground(Color.RED);
				tableBC.setSelectionBackground(Color.white);
				tableBC.setShowVerticalLines(false);
				tableBC.setColumnSelectionAllowed(false);
				tableBC.addMouseListener(bcHelp);
				tableBC.setName("tableBC");
				/*
				 * SelectionListener - se una riga � selezionata, i pulsanti
				 * scheda, modifica ed elimina sono attivi. Altrimenti, sono
				 * disabilitati.
				 */
				ListSelectionModel selectionModel = tableBC.getSelectionModel();
				selectionModel
						.addListSelectionListener(new ListSelectionListener()
						{
							public void valueChanged(ListSelectionEvent pEvent)
							{
								if (tableBC.getSelectedRow() != -1)
								{
									btnEliminaBC.setEnabled(true);
									btnSchedaBC.setEnabled(true);
									btnModificaBC.setEnabled(true);
								}
								else
								{
									btnEliminaBC.setEnabled(false);
									btnSchedaBC.setEnabled(false);
									btnModificaBC.setEnabled(false);
								}
							}
						});
				/*
				 * KeyListener <ENTER> - Scheda del bene culturale selezionato.
				 * <BACKSPACE> - Elimina il bene culturale selezionato. <SPACE> -
				 * Modifica il bene culturale selezionato.
				 */
				tableBC.addKeyListener(new KeyAdapter()
				{
					public void keyPressed(KeyEvent pEvent)
					{
						int keyCode = pEvent.getKeyCode();
						if (keyCode == KeyEvent.VK_ENTER)
						{
							btnSchedaBC.doClick();
						}
						else if (keyCode == KeyEvent.VK_BACK_SPACE)
						{
							btnEliminaBC.doClick();
						}
						else if (keyCode == KeyEvent.VK_SPACE)
						{
							btnModificaBC.doClick();
						}
					}
				});
			}

			jScrollPane = new JScrollPane(tableBC);
			jScrollPane
					.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);
			jScrollPane
					.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
		}
		return jScrollPane;
	}

	/**
	 * Questo metodo inizializza il pulsante per la ricerca di un bene
	 * culturale.
	 * 
	 * @return javax.swing.JButton - il pulsante per la ricerca.
	 */
	private JButton getBtnRicerca()
	{
		if (btnRicerca == null)
		{
			btnRicerca = new JButton();
			btnRicerca.setText("Ricerca");
			btnRicerca.setPreferredSize(new Dimension(98, 26));
			btnRicerca.setIcon(new ImageIcon(getClass().getResource(
					Home.URL_IMAGES + "Ricerca16.png")));
			btnRicerca
					.setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
			btnRicerca.setName("btnRicerca");
			btnRicerca.addMouseListener(bcHelp);
		}
		return btnRicerca;
	}

	/**
	 * Questo metodo inizializza il pulsante per l'azzeramento della form di
	 * ricerca bene culturale.
	 * 
	 * @return javax.swing.JButton - il pulsante per il reset della form.
	 */
	private JButton getBtnAzzera()
	{
		if (btnAzzera == null)
		{
			btnAzzera = new JButton();
			btnAzzera.setText("Azzera");
			btnAzzera.setHorizontalTextPosition(SwingConstants.LEADING);
			btnAzzera.setPreferredSize(new Dimension(98, 26));
			btnAzzera.setIcon(new ImageIcon(getClass().getResource(
					Home.URL_IMAGES + "Azzera16.png")));
			btnAzzera.setName("btnAzzera");
			btnAzzera.setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
			btnAzzera.addMouseListener(bcHelp);
			btnAzzera.addActionListener(new ActionListener()
			{
				public void actionPerformed(ActionEvent arg0)
				{
					nomeBC.setText("");
					pannelloTag.azzera();
				}

			});
		}
		return btnAzzera;
	}

	/**
	 * Questo metodo apre una scheda del bene culturale selezionato o apre la
	 * finestra per l'inserimento di un nuovo bene culturale.
	 * 
	 * @param pBc BeanBeneCulturale - il bean del bene culturale di cui si
	 *            desidera visualizzare la scheda.
	 * @param pModifica boolean -
	 *            <ul>
	 *            <li><i>true</i> - indica che si sta effettuando una modifica
	 *            dei dati del bene culturale.
	 *            <li><i>false</i> indica che si sta visualizzando la scheda
	 *            del bene culturale.
	 */
	private void openScheda(BeanBeneCulturale pBc, boolean pModifica)
	{
		SchedaBC nuova;
		if (pBc == null)
		{
			nuova = new SchedaBC(this);

		}
		else
		{
			for (int i = 0; i < figli.size(); i++)
			{
				SchedaBC corrente = figli.get(i);
				if (pBc.getId() == corrente.getId())
				{
					desktopManager.activateFrame(corrente);
					return;
				}
			}
			nuova = new SchedaBC(this, pBc, pModifica);
		}
		jDesktopPane.add(nuova, Integer.MAX_VALUE);
		desktopManager.centerFrame(nuova);
		nuova.setVisible(true);
		figli.add(nuova);
	}

	/**
	 * Questo metodo importa i beni culturali scaricati dal server nella
	 * tabella.
	 * 
	 * @param pRicerca boolean
	 *            <ul>
	 *            <li><i>true</i> - include i parametri di ricerca.
	 *            <li><i>false</i> altrimenti.
	 *            </ul>
	 */
	private void caricaTabella(boolean pRicerca)
	{
		ArrayList<BeanBeneCulturale> beniCulturali = null;
		try
		{
			if (pRicerca)
			{

			}
			else
			{
				beniCulturali = gestioneBC.ottieniBeniCulturali();
			}
		}
		// In caso di errore visualizza un messaggio di errore.
		catch (RemoteException e)
		{
			JLabel errore = new JLabel(
					"<html><h2>Impossibile comunicare con il server di eTour.</h2>"
							+ "<h3><u>La lista dei beni culturali non è stata caricata.</u></h3>"
							+ "<p>Si prega di riprovare in un secondo momento.</p>"
							+ "<p>Se l'errore dovesse persistere, si prega di contattare l'assistenza tecnica.</p>"
							+ "<p>Ci scusiamo per il disagio.</html>");
			ImageIcon err = new ImageIcon(getClass().getResource(
					Home.URL_IMAGES + "error48.png"));
			JOptionPane.showInternalMessageDialog(this, errore, "Errore!",
					JOptionPane.ERROR_MESSAGE, err);
		}
		finally
		{
			tableModel = new SitoTableModel(beniCulturali);
			tableBC.setModel(tableModel);
			organizzaRiga();
		}
	}

	/**
	 * Questo metodo carica i tag disponibili nel sistema e li importa nel
	 * pannello dei tag.
	 */
	private void caricaTags()
	{
		ArrayList<BeanTag> beanTags = null;
		try
		{
			beanTags = tag.ottieniTags();
			for (BeanTag b : beanTags)
			{
				pannelloTag.insertTag(b);
			}
			pannelloTag.repaint();
		}
		// In caso di errore il pannello tag rimane vuoto.
		catch (RemoteException e)
		{
		}
	}

	/**
	 * Questo metodo imposta le dimensioni delle colonne per i dati dei beni
	 * culturali.
	 */
	private void organizzaRiga()
	{
		// Nome
		tableBC.getColumnModel().getColumn(0).setPreferredWidth(120);
		// Indirizzo
		tableBC.getColumnModel().getColumn(1).setPreferredWidth(120);
		// Telefono
		tableBC.getColumnModel().getColumn(2).setPreferredWidth(80);
		// Località
		tableBC.getColumnModel().getColumn(3).setPreferredWidth(80);
		// Città
		tableBC.getColumnModel().getColumn(4).setPreferredWidth(80);
		// CAP
		tableBC.getColumnModel().getColumn(5).setPreferredWidth(50);
		// Prov
		tableBC.getColumnModel().getColumn(6).setPreferredWidth(30);
		// RATING
		tableBC.getColumnModel().getColumn(7).setPreferredWidth(80);
		// POSGEO
		tableBC.getColumnModel().getColumn(8).setPreferredWidth(120);
	}
}
