package unisa.gps.etour.control.fuzzy;

import java.io.Serializable;
import java.util.Hashtable;

public class Categoria implements Serializable
{

	/**
	 * Classe che descrive le caratteristiche di una Categoria
	 * contiene una Hashtable che rappresenta il dizionario relativo 
	 * alla Categoria che contiene quindi per ogni termine associato 
	 * alla Categoria i valori di frequenza, distanza e rilevanza.
	 * Offre metodi di accesso, modifica e metodi ausiliari.
	 */
	private static final long serialVersionUID = -8652232946927756089L;
	private String nome; //nome della categoria
	private Hashtable<String, float[]> termini; //elenco di termini con relativa frequenza, rilavanza e distanza

	/*
	 * Costruttore:
	 * Riceve come parametro il nome della Categoria da creare
	 */
	public Categoria(String pNome)
	{
		nome = pNome;
		termini = new Hashtable<String,float[]>();
	}

	/*
	 * Restituisce in output la Hashtable contenente i termini 
	 * con i rispettivi valori di frequenza, rilevanza e distanza
	 */
	public Hashtable<String, float[]> getTermini()
	{
		return termini;
	}

	/*
	 * Restituisce in output il nome della Categoria
	 */
	public String getName()
	{
		return nome;
	}

	/*
	 * Prende come parametro la stringa rappresentante il termine
	 * di cui si vuole prelevare i valori di frequenza, distanza e rilevanza
	 */
	public float[] getVal(String pTermine) throws NullPointerException
	{
		if (esisteTermine(pTermine))
			return termini.get(pTermine);

		return null;
	}

	/*
	 * Agiunge un termine al dizinario della categoria
	 */
	public void addTermine(String pTermine)
	{
		termini.put(pTermine, new float[3]);
	}

	/*
	 * Agiunge un termine al dizinario della categoria 
	 * settando anche i valori di frequenza, distanza e rilavanza
	 */
	public boolean addTermine(String pTermine, float[] pVal)
	{
		if((pVal==null)||(pTermine.equals("")))
			return false;
		termini.put(pTermine, pVal);
		
		return true;
	}

	/*
	 * Setta i valori relativi al termine pTermine
	 */
	public boolean setValTermine(String pTermine, float[] pVal)
			throws NullPointerException
	{
		if (esisteTermine(pTermine))
		{
			termini.put(pTermine, pVal);
			return true;
		}

		return false;
	}
	
	
	public void setTermini(Hashtable<String,float[]> pTermini)
	{
		termini = pTermini;
	}

	/*
	 * Restituisce True se il termine è presente nel 
	 * dizionario della Categoria False altrimenti
	 */
	public boolean esisteTermine(String pTermine)
	{
		try
		{
			float [] ret = termini.get(pTermine);
			if(ret!=null)
				return true;
		}
		catch (NullPointerException e)
		{
			return false;
		}
		
		return false;
	}

}
