﻿/**
 * Classe che contiene metodi statici che effettuano
 * i controlli di consistenza sui dati dei bean banner
 * e news. 
 *
 * @author Palladino Fabio
 * @version 0.1
 * 
 * &#169; 2007 eTour Project - Copyright by SE@SA Lab DMI - University of Salerno
 */
package com.trapan.spg.control.GestioneAdvertisement;

import java.util.Date;

import com.trapan.spg.bean.BeanBanner;
import com.trapan.spg.bean.BeanNews;

public class ControlloDatiAdvertisement
{
	/**
	 * Effettua il controllo formale e di consistenza sui 
	 * dati del banner contenuto nel bean passato per parametro.
	 * @param pBanner bean contenente i dati del banner.
	 * @return
	 */
	public static boolean controlloBanner(BeanBanner pBanner){
		boolean toReturn = false;
		
		if (pBanner != null){
			toReturn = (pBanner.getId()>0 && pBanner.getPercorsoFile()!="" && 
					pBanner.getIdPuntoDiRistoro()>0);
		}
				
		return toReturn;
	}
	/**
	 * Metodo che effettua i controlli di  consistenza e 
	 * correttezza dei dati contenuti nel bean passato
	 * per parametro, in modo particolare controlla che le date
	 * di pubblicazione e di scadenza della news siano coerenti,
	 *  ovvero che la seconda sia successiva alla prima.
	 *
	 * @param pNews bean contenente i dati della news
	 * @return Restituisce true se il bean contiene dati consistenti 
	 */
	public static boolean controlloNews(BeanNews pNews){
		boolean toReturn = false;
		
		/* Controllo sulla validità generale del parametro del metodo*/
		if (pNews != null){
			
			Date dataPubb = pNews.getDataPubblicazione();		// Data di pubblicazione
			Date dataScad = pNews.getDataScadenza();			// Data scadenza
			String news = pNews.getNews();						// Testo della News 		
			
			/* Controllo sulla nullità dei campi */
			if (dataPubb != null && dataScad != null && news != null){
				/* Controllo sulla coerenza delle date */
				toReturn = dataPubb.before(dataScad);
				
				/* Controlla che il testo non sia vuoto */
				toReturn = toReturn && (news != "");
				
				/* Controlla che l'id sia maggiore di 0 */
				toReturn = toReturn && (pNews.getId() > 0);
				
				/* Controlla il valore della priorità */
			}
					
		}
			
		return toReturn;
		
	}
}
