package unisa.gps.etour.repository;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;

/**
 * Classe che implementa l'interfaccia Statistiche Localita
 * 
 * @author Martone Giuseppe
 * @version 0.1 &#169; 2007 eTour Project - Copyright by SE@SA Lab DMI -
 *          University of Salerno
 */
public class DBStatisticheLocalita implements IDBStatisticheLocalita
{
	// Costruttore vuoto
	public DBStatisticheLocalita()
	{

	}

	public ArrayList<String> ottieniListaLocalita() throws SQLException
	{
		// Variabile per la connessione
		Connection conn = null;
		// Variabile per le query
		Statement stat = null;
		// Variabile per i risultati delle query
		ResultSet result = null;
		try
		{
			// Ottengo la connessione
			conn = DBConnessionePool.ottieniConnessione();
			// Creazione dello Statement
			stat = conn.createStatement();
			// Query per l'estrazione delle localita tra i PR e i BC
			String query = " (SELECT DISTINCT Localita "
					+ "FROM beneculturale) UNION "
					+ "(SELECT DISTINCT Localita FROM puntodiristoro)";
			// Viene eseguita la query
			result = stat.executeQuery(query);
			// Si estraggono i risultati dal result set e si inseriscono nella
			// lista
			// da restituire
			// Lista che conterrà i risultati ottenuti
			ArrayList<String> lista = new ArrayList<String>();
			while (result.next())
			{
				// Aggiungo nella lista le località ottenute
				lista.add(result.getString("Localita"));
			}
			return lista;
		}
		// Viene eseguito sempre e si occupa di chiudere il Result, lo Statement
		// e la Connessione
		finally
		{
			if (result != null)
			{
				result.close();
			}
			if (stat != null)
			{
				stat.close();
			}
			if (conn != null)
			{
				DBConnessionePool.rilasciaConnessione(conn);
			}
		}
	}

	public double ottieniMedieVotiLocalita(String plocalita)
			throws SQLException
	{
		// Variabile per la connessione
		Connection conn = null;
		// Variabile per le query
		Statement stat = null;
		// Variabile per i risultati delle query
		ResultSet result = null;
		try
		{
			// Ottengo la connessione
			conn = DBConnessionePool.ottieniConnessione();
			// Creazione dello Statement
			stat = conn.createStatement();
			// Query per l'estrazione della media dei voti dei punti di ristoro
			// in base ad una localita passata
			String query = "SELECT  avg(MediaVoti) as MediaVoti FROM puntodiristoro WHERE Localita ='"
					+ plocalita + "'";
			// Viene eseguita la query
			result = stat.executeQuery(query);
			// Si estraggono i risultati dal result set
			double punto = 0.0;
			if (result.next())
			{
				punto = result.getDouble("MediaVoti");
			}
			// Query per l'estrazione della media dei voti dei beni culturali
			// in base ad una localita passata
			query = "SELECT  avg(MediaVoti) AS MediaVoti FROM beneculturale WHERE Localita ='"
					+ plocalita + "'";
			// Viene eseguita la query
			result = stat.executeQuery(query);
			// Si estraggono i risultati dal result set
			double bene = 0.0;
			if (result.next())
			{
				bene = result.getDouble("MediaVoti");
			}
			// Si restituisce la media tra i punti di ristoro e i beni culturali
			return (punto + bene) / 2;

		}
		// Viene eseguito sempre e si occupa di chiudere il Result, lo Statement
		// e la Connessione
		finally
		{
			if (result != null)
			{
				result.close();
			}
			if (stat != null)
			{
				stat.close();
			}
			if (conn != null)
			{
				DBConnessionePool.rilasciaConnessione(conn);
			}

		}
	}
}
