﻿package unisa.gps.etour.control.fuzzy;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.List;

public class ElencoCategorie implements Serializable
{

	/**
	 * Mantiene traccia dei dati di ogni categoria
	 */
	private static final long serialVersionUID = 1L;
	private Hashtable<String, Categoria> categorie;// tabella hash che conserva
	// per ogni categoria una
	// classe Categoria
	private Hashtable<String, float[]> totTermini;// tabella hash che conserva
	// i termini di tutte le
	// categorie
	float maxDist;// contiene il massimo delle distanze

	/**
	 * Il costruttore inizializza le due tabelle hash che contengono le
	 * categorie e i termini di tutte le categorie
	 */
	public ElencoCategorie()
	{
		categorie = new Hashtable<String, Categoria>();
		totTermini = new Hashtable<String, float[]>();
	}

	/**
	 * Metodo di accesso all'attributo maxDist
	 * 
	 * @return la distanza massima di tutti i termini con tutte le categorie
	 */
	public float getMaxDist()
	{
		return maxDist;
	}

	/**
	 * Metodo di accesso alla tabella delle categorie
	 * 
	 * @return categorie
	 */
	public Hashtable<String, Categoria> getAllCategorie()
	{
		return categorie;
	}

	/**
	 * Metodo di accesso alla tabella dei termini totali
	 * 
	 * @return totTermini
	 */
	public Hashtable<String, float[]> getTotTermini()
	{
		return totTermini;
	}

	/**
	 * Metodo per accedere ad una categoria presente nella tabella delle
	 * categorie
	 * 
	 * @param pNomeCategoria
	 * @return oggetto Categoria rappresentante la categoria di nome
	 *         pNomeCategoria
	 */
	public Categoria getCategoria(String pNomeCategoria)
	{

		if (esisteCategoria(pNomeCategoria))// se esiste la categoria voluta
			return categorie.get(pNomeCategoria);// restituisce l'elemento
		// assciato a pNomeCategoria

		return null;// altrimenti null
	}

	/**
	 * Metodi di accesso ai valori di un determinato termine presente nella
	 * tabella delle categorie
	 * 
	 * @param pTermine
	 * @return Restituisce i valori associati al termine pTermine
	 */
	public float[] getTermine(String pTermine)
	{
		if (esisteTermine(pTermine))// se il termine è presente nella tebella
			// dei termini totali
			return (float[]) totTermini.get(pTermine);// restituisce il vaolre
		// associato

		return null;// null altrimenti
	}

	/**
	 * Metodo che permette di aggiungere una categoria alla tabella delle
	 * categorie
	 * 
	 * @param pNomeCategoria nome categoria da aggiungere
	 * @param pCategoria oggetto associato alla categoria
	 * @return vero se l'operazione è stata effettuata con successo falso
	 *         altrimenti
	 */
	public boolean addCategoria(String pNomeCategoria, Categoria pCategoria)
	{
		if (!esisteCategoria(pNomeCategoria))// se la categoria esiste
			return false;// restituisci falso

		categorie.put(pNomeCategoria, pCategoria);// altrimenti carica la
		// categoria nella tabella

		return true;// restituisci vero
	}

	/**
	 * Modifica una categoria della tabella delle categorie
	 * 
	 * @param pNomeCategoria nome della categoria da modificare
	 * @param pCategoria oggetto da associare alla categoria
	 * @return vero se l'operazione è stata effettuata con successo falso
	 *         altrimenti
	 */
	public boolean setCategoria(String pNomeCategoria, Categoria pCategoria)
	{
		if (esisteCategoria(pNomeCategoria))// se la categoria non esiste
			return false;// restituisci falso

		categorie.put(pNomeCategoria, pCategoria);// aggiorna la tabella delle
		// catogorie

		return true;// restituisci vero
	}

	/**
	 * Metodo che permette di settare il valore di un termine nella tabelle dei
	 * termini totali
	 * 
	 * @param pTermine nome del termine
	 * @param pVal valora da associare al termine
	 */
	public void setTermine(String pTermine, float[] pVal)
	{

		totTermini.put(pTermine, pVal);
	}

	/**
	 * Metodo che permette di setta re il valore della distanza massima di un
	 * termine da una categoria
	 * 
	 * @param pMaxDist
	 */
	public void setMaxDist(float pMaxDist)
	{
		maxDist = pMaxDist;
	}

	/**
	 * Metodo che permette di ricavare una collezione iterabile dei nomi di
	 * tutte le categorie presenti nella tabela delle categorie
	 * 
	 * @return Collezione iterabile di stringhe
	 */
	public Iterable<String> Categorie()
	{
		List<String> toReturn = new ArrayList<String>();// crea una nuova lista
		for (Enumeration<String> val = categorie.keys(); val.hasMoreElements();)// itera
		// N
		// volte
		// dove
		// N è
		// il
		// numero
		// di
		// categorie
		// presenti
		// nella
		// tabella
		{
			toReturn.add(val.nextElement());// aggiunge alla lista il nome di
			// una categoria
		}

		return toReturn;
	}

	/**
	 * Metodo ausiliare che permette di verificare l'esistenza di una categoria
	 * nella tabella delle categorie
	 * 
	 * @param pKey nome della categoria
	 * @return vero se la categoria esiste falso altrimenti
	 */
	public boolean esisteCategoria(String pKey)
	{
		try
		{
			categorie.get(pKey);// tenta di estrarra la categoria di nome pKey
								// dalla tabella delle categorie
			return true;// se l'operazione non genera eccezioni la categoria
						// esiste e restituisce true
		}
		catch (NullPointerException e)
		{
			return false;// false altrimenti
		}
	}

	/**
	 * Metodo ausiliare che permette di verificare l'esistenza di una termine
	 * nella tabella dei termini totali
	 * 
	 * @param pKey termine
	 * @return vero se il termine esiste falso altrimenti
	 */
	public boolean esisteTermine(String pKey)
	{
		//vedi esisteCategoria
		try
		{
			if (totTermini.get(pKey) != null)
				return true;
		}
		catch (NullPointerException e)
		{
			return false;
		}

		return false;
	}

}
