/**
 * Classe che implementa l'interfaccia generale
 * della gestione dell'advertisement.
 *
 * @author Palladino Fabio
 * @version 0.1
 * 
 * � 2007 eTour Project - Copyright by SE@SA Lab DMI � University of Salerno
 */
package unisa.gps.etour.control.GestioneAdvertisement;

import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.HashMap;

import javax.imageio.ImageIO;
import javax.swing.ImageIcon;

import unisa.gps.etour.bean.BeanBanner;
import unisa.gps.etour.bean.BeanConvenzione;
import unisa.gps.etour.repository.DBBanner;
//import unisa.gps.etour.control.GestioneAdvertisement.test.stubs.*;
import unisa.gps.etour.repository.DBConvenzione;
import unisa.gps.etour.repository.IDBBanner;
import unisa.gps.etour.repository.IDBConvenzione;
import unisa.gps.etour.util.ControlloDati;
import unisa.gps.etour.util.CostantiGlobali;
import unisa.gps.etour.util.MessaggiErrore;

public class GestioneAdvertisement extends UnicastRemoteObject implements
		IGestioneAdvertisement
{
	private static final long serialVersionUID = 1L;
	/** Oggetto che gestisce le operazioni sui banner*/
	private IDBBanner dbBanner;
	/**
	 * Costruttore, istanzia un oggetto di tipo DBBanner che 
	 * contiene i metodi che effettuano le operazioni sui dati
	 * dell'entità banner. 
	 * 
	 * @throws RemoteException
	 */
	public GestioneAdvertisement() throws RemoteException
	{
		super();
		dbBanner = new DBBanner();		
	}
	/**
	 * Elimina un banner e l'immagine associata dal sistema.
	 * 
	 * @param	pBannerID id del banner da eliminare
	 * @throws RemoteException
	 */
	public boolean cancellaBanner(int pIdBanner) throws RemoteException
	{
		/* bean contenente i dati del banner */
		BeanBanner banner;
		
		try 
		{
			/* Carica il banner e ne controlla i dati */
			banner = dbBanner.ottieniBannerDaID(pIdBanner);
			if(!ControlloDati.checkBeanBanner(banner)){
				throw new RemoteException(MessaggiErrore.ERRORE_DATI);
			}
			/* cancella l'immagine associata al banner */
			File immagine = new File(banner.getPercorsoFile());
			immagine.delete();
			/* Cancella l'intero bean del banner*/
			return (dbBanner.cancellaBanner(pIdBanner));
		} 
		catch (SQLException e) {
			throw new RemoteException(MessaggiErrore.ERRORE_CONNESSIONE_DBMS);
		}
	}
	/**
	 * Metodo che modifica l'immagine associata al banner: 
	 * cancella la vecchia immagine e memorizza la nuova, 
	 * sfruttando lo stesso percorso della vecchia immagine.
	 * 
	 *  @param pBannerID id del banner
	 *  @param pImmagine oggetto ImageIcon contenente la nuova immagine del banner
	 *  @return restituisce true se l'operazione ha successo
	 */
	public boolean modificaBanner(int pBannerID,ImageIcon pImmagine) throws RemoteException
	{
			
		try
		{
			/* effettua un controllo sull'immagine del banner */
			if(!ControlloDati.checkImmagine(pImmagine)){
				throw new RemoteException(MessaggiErrore.ERRORE_DATI);
			}
			/* carica l'immagine dell'icona */
			BufferedImage buffImg = (BufferedImage) pImmagine.getImage();
			/* carica i dati del banner */
			BeanBanner banner = dbBanner.ottieniBannerDaID(pBannerID);
			/* Controllo sui dati del banner caricato*/
			if(!ControlloDati.checkBeanBanner(banner)){
				throw new RemoteException(MessaggiErrore.ERRORE_DATI);
			}
			/* riscrive l'immagine nel file e ritorna il risultato dell'operazione */
			File imgFile = new File(banner.getPercorsoFile());
			return (ImageIO.write(buffImg,"jpg",imgFile));
		}
		catch(SQLException e){
			throw new RemoteException(MessaggiErrore.ERRORE_CONNESSIONE_DBMS);
		}
		catch(IOException e){
			if(e instanceof RemoteException)
			{
				throw new RemoteException(MessaggiErrore.ERRORE_DATI);
			}else
			{
				throw new RemoteException(MessaggiErrore.ERRORE_FILE);
			}			
		}
	}
	/**
	 * Restituisce una lista di banner relativi a un punto di ristoro.
	 * Utilizza il metodo {@link DBBanner.ottieniBanner}
	 * 
	 * @param pPuntoDiRistoroID ID del punto di ristoro
	 * @throws RemoteException
	 */
	public HashMap<BeanBanner,ImageIcon> ottieniBannersDaID(int pIdPuntoDiRistoro)
			throws RemoteException{
		/* crea una HashMap che conterrà i banner e le immagini associate */
		HashMap<BeanBanner,ImageIcon> toReturn = new HashMap<BeanBanner, ImageIcon>();
		
		try
		{
			/* Ottiene la lista dei banner di un punto di ristoro */
			ArrayList<BeanBanner> listaBanner = dbBanner.ottieniBanner(pIdPuntoDiRistoro);
			/* per ogni banner valido della lista */ 
			for(BeanBanner banner : listaBanner){
				/* se il banner è valido lo carica nella HashMap insieme al file immagine */
				if(ControlloDati.checkBeanBanner(banner)){
					/* crea il file dell'immagine */
					File fileImmagine = new File(banner.getPercorsoFile());
					/* crea un oggetto ImageIcon dal file immagine */
					ImageIcon icon = new ImageIcon(ImageIO.read(fileImmagine));
					/* aggiunge il banner e l'immagine all'HashMap */
					toReturn.put(banner, icon);
				}
			}
			return toReturn;			
		}
		catch (SQLException e){
			throw new RemoteException(MessaggiErrore.ERRORE_CONNESSIONE_DBMS);
		}
		catch (IOException e){
			throw new RemoteException(MessaggiErrore.ERRORE_FILE);
		}
		catch (Exception e){
			throw new RemoteException(MessaggiErrore.ERRORE_SCONOSCIUTO);
		}
	}	
	/**
	 * Crea e inserisce un nuovo banner, controllando che il num. massimo
	 * dei banner visualizzabili per un punto di ristoro non sia 
	 * stato già raggiunto, in questo caso lancia un'eccezione al metodo chiamante.
	 *  
	 * @param pIDPuntoDiRistoro Id del punto di ristoro
	 * @param pImmagineBanner oggetto ImageIcon contenente l'immagine del banner
	 * @return	Restituisce true se l'inserimento è andato a buon fine.
	 * @exception RemoteException
	 */
	public boolean inserisciBanner(int pIdPuntoDiRistoro, ImageIcon pImmagineBanner) 
		throws RemoteException
	{
		
		try
		{
			/* effettua un controllo sull'immagine */
			if(!ControlloDati.checkImmagine(pImmagineBanner)){
				throw new RemoteException(MessaggiErrore.ERRORE_DATI);
			}
			/* crea un oggetto che gestisce le convenzioni */
			IDBConvenzione dbConvenzione = new DBConvenzione();
			/* carica i dati della convenzione di un punto di ristoro */
			BeanConvenzione convenzione = dbConvenzione.ottieniConvezioneAttiva(pIdPuntoDiRistoro);
			/* estrae il numero masssimo di banner visualizzabili dalla convenzione */
			int maxBanner = convenzione.getMaxBanner();
			/* carica la lista dei banner di un punto di ristoro e ne memorizza il numero*/
			int numBanner = (dbBanner.ottieniBanner(pIdPuntoDiRistoro)).size();
			/* controlla che il numero massimo di banner non sia stato raggiunto */
			if(!(numBanner < maxBanner))
			{
				throw new Exception(MessaggiErrore.ERRORE_NUM_BANNER);
			}
			/* crea un banner vuoto */
			BeanBanner banner = new BeanBanner();
			/* crea un nome di file univoco */
			String path = CostantiGlobali.SERVER_IMAGE_PATH + pIdPuntoDiRistoro;
			int i = 0;
			File fileImg = new File(path + "_" + i + ".jpg");
			while(fileImg.exists()){
				i++;
				fileImg = new File(path + "_" + i + ".jpg");
			}
			/* memorizza l'immagine nel file system */
			BufferedImage im = (BufferedImage) pImmagineBanner.getImage();
			if(!ImageIO.write(im, "jpg", fileImg)){
				/* scrittura fallita */
				throw new IOException();
			}
			/* carica i dati nel banner */
			banner.setIdPuntoDiRistoro(pIdPuntoDiRistoro);
			banner.setPercorsoFile(fileImg.getPath());
			/* inserisce il banner nel database */
			return (dbBanner.inserisciBanner(banner));
		}
		catch(SQLException e)
		{
			throw new RemoteException(MessaggiErrore.ERRORE_CONNESSIONE_DBMS);
		}
		catch(IOException e)
		{
			throw new RemoteException(MessaggiErrore.ERRORE_FILE);
		}
		catch(Exception e)
		{
			if(e.getMessage().equals(MessaggiErrore.ERRORE_NUM_BANNER))
			{
				/* è stato raggiunto il num massimo di banner inseribili */
				throw new RemoteException(MessaggiErrore.ERRORE_NUM_BANNER);
			}else
			{
				throw new RemoteException(MessaggiErrore.ERRORE_SCONOSCIUTO);
			}
			
		}
		
	}

}

	
