package unisa.gps.etour.control.GestioneBeniCulturali;

import java.rmi.RemoteException;
import java.sql.SQLException;
import java.util.ArrayList;
import unisa.gps.etour.bean.BeanBeneCulturale;
import unisa.gps.etour.bean.BeanTag;
import unisa.gps.etour.bean.BeanVisitaBC;
import unisa.gps.etour.util.MessaggiErrore;

/**
 * Classe di gestione di beni culturali peculiare per Agenzia
 *
 * @author De Simone Michelangelo
 * @version 0.1
 * 
 *  &#169; 2007 eTour Project - Copyright by SE@SA Lab DMI - University of Salerno
 */
public class GestioneBeniCulturaliAgenzia extends GestioneBeniCulturaliComune
		implements IGestioneBeniCulturaliAgenzia
{
	/*
	 * Costruttore di classe, richicama ed inizializza la classe di gestione comune
	 */
	public GestioneBeniCulturaliAgenzia() throws RemoteException
	{
		super();
	}
	
	/*
	 * Implementa il metodo per l'eliminazione dal sistema di un bene culturale.
	 * 
	 * @see unisa.gps.etour.control.GestioneBeniCulturali.IGestioneBeniCulturaliAgenzia#cancellaBeneCulturale(int)
	 */
	public boolean cancellaBeneCulturale(int pBeneCulturaleID)
			throws RemoteException
	{
		if (!ControlloBeniCulturali.controllaIdBeneCulturale(pBeneCulturaleID))
			throw new RemoteException(MessaggiErrore.ERRORE_DATI);
		
		try
		{
			return (dbbc.cancellaBeneCulturale(pBeneCulturaleID));
		}
		catch (SQLException e)
		{
			throw new RemoteException(MessaggiErrore.ERRORE_DBMS);
		}
		catch (Exception e)
		{
			throw new  RemoteException(MessaggiErrore.ERRORE_SCONOSCIUTO);
		}
	}
	
	/*
	 * Implementa il metodo per l'inserimento di un nuovo bene culturale.
	 * 
	 * @see unisa.gps.etour.control.GestioneBeniCulturali.IGestioneBeniCulturaliAgenzia#inserisciBeneCulturale(unisa.gps.etour.bean.BeanBeneCulturale)
	 */
	public boolean inserisciBeneCulturale(BeanBeneCulturale pBeneCulturale)
			throws RemoteException
	{
		if (!ControlloBeniCulturali.controllaDatiBeneCulturale(pBeneCulturale))
			throw new RemoteException(MessaggiErrore.ERRORE_FORMATO_BEAN);
		
		try
		{
			return (dbbc.inserisciBeneCulturale(pBeneCulturale));
		}
		catch (SQLException e)
		{
			throw new RemoteException(MessaggiErrore.ERRORE_DBMS);
		}
		catch (Exception e)
		{
			throw new RemoteException(MessaggiErrore.ERRORE_SCONOSCIUTO);
		}
	}
	
	/*
	 * Implementa il metodo per l'ottenimento di tutti i beni culturali attualmente presenti nel sistema.
	 * 
	 * @see unisa.gps.etour.control.GestioneBeniCulturali.IGestioneBeniCulturaliAgenzia#ottieniBeniCulturali()
	 */
	public ArrayList<BeanBeneCulturale> ottieniBeniCulturali() throws RemoteException
	{
		try
		{
			return (dbbc.ottieniListaBC());
		}
		catch (SQLException e)
		{
			throw new RemoteException(MessaggiErrore.ERRORE_DBMS);
		}
		catch (Exception e)
		{
			throw new RemoteException(MessaggiErrore.ERRORE_SCONOSCIUTO);
		}
	}

	/*
	 * Implementa il metodo per la modifica di un bene culturale presente nel sistema.
	 * 
	 * @see unisa.gps.etour.control.GestioneBeniCulturali.IGestioneBeniCulturaliAgenzia#modificaBeneCulturale(unisa.gps.etour.bean.BeanBeneCulturale)
	 */
	public boolean modificaBeneCulturale(BeanBeneCulturale pBeneCulturale)
			throws RemoteException
	{
		if (!ControlloBeniCulturali.controllaDatiBeneCulturale(pBeneCulturale))
			throw new RemoteException(MessaggiErrore.ERRORE_FORMATO_BEAN);

		try
		{
			return (dbbc.modificaBeneCulturale(pBeneCulturale));
		}
		catch (SQLException e)
		{
			throw new RemoteException(MessaggiErrore.ERRORE_DBMS);
		}
		catch (Exception e)
		{
			throw new RemoteException(MessaggiErrore.ERRORE_SCONOSCIUTO);
		}
	}

	/*
	 * Implementa il metodo per l'aggiunta di un tag ad un bene culturale.
	 * 
	 * @see unisa.gps.etour.control.GestioneBeniCulturali.IGestioneBeniCulturaliAgenzia#aggiungiTagBeneCulturale(int, int)
	 */
	public boolean aggiungiTagBeneCulturale(int pBeneCulturaleID, int pTagID) throws RemoteException
	{
		if (!ControlloBeniCulturali.controllaIdBeneCulturale(pBeneCulturaleID) || !(pTagID > 0))
			throw new RemoteException(MessaggiErrore.ERRORE_DATI);

		/*
		 * Questo segmento di codice controlla che effettivamente il bene culturale speficiato
		 * abbia il tag definito.
		 */
		
		/*
		 * Ottengo tutti i tag per il bene culturale passato per parametro
		 */ 
		ArrayList<BeanTag> tempTag = null;

		boolean contieneTag = false;

		try
		{
			tempTag = dbtag.ottieniTagBeneCulturale(pBeneCulturaleID);
		}
		catch (SQLException e)
		{
			throw new RemoteException(MessaggiErrore.ERRORE_DBMS);
		}

		/*
		 * Qui si itera alla ricerca del tag che speficiato, se lo si trova si imposta una sentinella
		 * in modo da non aggiungere un tag due volte allo stesso bene culturale.
		 */
		for (BeanTag t: tempTag)
			if (t.getId() == pTagID)
				contieneTag = true;
		
		if (!contieneTag)
			try
			{
				return (dbtag.aggiungeTagBeneCulturale(pBeneCulturaleID, pTagID));
			}
			catch (SQLException e)
			{
				throw new RemoteException(MessaggiErrore.ERRORE_DBMS);
			}
			catch (Exception e)
			{
				throw new RemoteException(MessaggiErrore.ERRORE_SCONOSCIUTO);
			}
		
		return false;
	}

	/*
	 * Implementa il metodo per la rimozione di un tag da un bene culturale.
	 * 
	 * @see unisa.gps.etour.control.GestioneBeniCulturali.IGestioneBeniCulturaliAgenzia#rimuoviTagBeneCulturale(int, int)
	 */
	public boolean rimuoviTagBeneCulturale(int pBeneCulturaleID, int pTagID) throws RemoteException
	{
		if (!ControlloBeniCulturali.controllaIdBeneCulturale(pBeneCulturaleID) || !(pTagID > 0))
			throw new RemoteException(MessaggiErrore.ERRORE_DATI);

		/*
		 * Questo segmento di codice controlla che effettivamente il bene culturale speficiato
		 * abbia il tag specificato.
		 */
		
		/*
		 * Ottengo tutti i tag per il bene culturale passato per parametro
		 */ 
		ArrayList<BeanTag> tempTag = null;

		try
		{
			tempTag = dbtag.ottieniTagBeneCulturale(pBeneCulturaleID);
		}
		catch (SQLException e)
		{
			throw new RemoteException(MessaggiErrore.ERRORE_DBMS);
		}

		/*
		 * Qui si itera alla ricerca del tag che speficiato, se lo si trova si effettua l'operazione
		 * di rimozione del tag e si restituisce il controllo
		 */
		for (BeanTag t: tempTag)
		{
			if (t.getId() == pTagID)
			{
				try
				{
					return (dbtag.cancellaTagBeneCulturale(pBeneCulturaleID, pTagID));
				}
				catch (SQLException e)
				{
					throw new RemoteException(MessaggiErrore.ERRORE_DBMS);
				}
				catch (Exception e)
				{
					throw new RemoteException(MessaggiErrore.ERRORE_SCONOSCIUTO);
				}
			}
		}
		
		return false;
	}
}
