﻿package unisa.gps.etour.control.GestionePuntiDiRistoro;

import java.rmi.RemoteException;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import unisa.gps.etour.bean.BeanConvenzione;
import unisa.gps.etour.bean.BeanPuntoDiRistoro;
import unisa.gps.etour.bean.BeanTag;
import unisa.gps.etour.bean.BeanTurista;
import unisa.gps.etour.bean.BeanVisitaPR;
/*
 * CAUSA TEST import
 * unisa.gps.etour.control.GestionePuntiDiRistoro.test.stub.DBConvenzione;
 * import unisa.gps.etour.control.GestionePuntiDiRistoro.test.stub.DBTurista;
 */
import unisa.gps.etour.repository.DBConvenzione;
import unisa.gps.etour.repository.DBTurista;
import unisa.gps.etour.repository.IDBConvenzione;
import unisa.gps.etour.util.MessaggiErrore;

/**
 * Classe contentente i metodi per la gestione dei Punti di Ristoro da parte
 * dell'Operatore Agenzia
 * 
 * @author Morelli Giuseppe
 * @version 0.1 &#169; 2007 eTour Project - Copyright by SE@SA Lab DMI -
 *          University of Salerno
 */
public class GestionePuntiDiRistoroAgenzia extends GestionePuntiDiRistoroComune
		implements IGestionePuntiDiRistoroAgenzia
{

	private static final long serialVersionUID = 1L;

	// Costruttore
	public GestionePuntiDiRistoroAgenzia() throws RemoteException
	{
		// Richiamo il costruttore della classe ereditata in modo da istanziare
		// le connessioni al database
		super();
		dbTurista = new DBTurista();
	}

	// Metodo che permette all'Operatore Agenzia di cancellare un certo Punto di
	// Ristoro
	// passando come parametro l'ID del Punto di Ristoro stesso
	public boolean cancellaPuntoDiRistoro(int pPuntoDiRistoroID)
			throws RemoteException
	{
		// Controllo la validità dell'identificativo
		if (pPuntoDiRistoroID < 0)
			throw new RemoteException(MessaggiErrore.ERRORE_DATI);
		try
		{
			// eseguo il metodo che cancella il Punto di Ristoro dal Database
			// ed in caso di operazione a buon fine ritorno true
			if (puntoRistoro.cancellaPuntoDiRistoro(pPuntoDiRistoroID))
				return true;
		}
		// Eccezione nelle operazioni sul database
		catch (SQLException e)
		{
			System.out.println("Errore nel metodo cancellaPuntoDiRistoro: "
					+ e.toString());
			throw new RemoteException(MessaggiErrore.ERRORE_DBMS);
		}
		// Eccezioni impreviste determinate da altri fattori
		catch (Exception ee)
		{
			System.out.println("Errore nel metodo cancellaPuntoDiRistoro: "
					+ ee.toString());
			throw new RemoteException(MessaggiErrore.ERRORE_SCONOSCIUTO);
		}
		// Nel caso le operazioni non siano andate a buon fine ritorno false
		return false;
	}

	// Metodo che permette all'Operatore Agenzia di inserire nel database
	// il nuovo Punto di Ristoro con le informazioni contenute nel bean
	public boolean inserisciPuntoDiRistoro(BeanPuntoDiRistoro pPuntoDiRistoro)
			throws RemoteException
	{
		// Controllo la validità del bean passato come parametro e nel caso
		// scateno l'eccezione remota
		if ((pPuntoDiRistoro == null)
				|| (!(pPuntoDiRistoro instanceof BeanPuntoDiRistoro)))
			throw new RemoteException(MessaggiErrore.ERRORE_FORMATO_BEAN);
		try
		{
			// Richiamo il metodo della classe che opera sul database per
			// inserire il nuovo Punto di Ristoro
			if (puntoRistoro.inserisciPuntoDiRistoro(pPuntoDiRistoro))
				// Nel caso in cui le operazioni siano andate a buon fine
				// restituisco true
				return true;
		}
		// Eccezione nelle operazioni sul Database
		catch (SQLException e)
		{
			System.out.println("Errore nel metodo inserisciPuntoDiRistoro: "
					+ e.toString());
			throw new RemoteException(MessaggiErrore.ERRORE_DBMS);
		}
		// Eccezioni impreviste determinate da altri fattori
		catch (Exception ee)
		{
			System.out.println("Errore nel metodo inserisciPuntoDiRistoro: "
					+ ee.toString());
			throw new RemoteException(MessaggiErrore.ERRORE_SCONOSCIUTO);
		}
		// Nel caso in cui l'operazione non sia riuscita ritorno false
		return false;
	}

	// Metodo che permette di ottenere un ArrayList con tutti i Bean dei punti
	// di Ristoro
	public ArrayList<BeanPuntoDiRistoro> ottieniPuntiDiRistoro()
			throws RemoteException
	{
		// ArrayList da ritornare alla fine del metodo
		ArrayList<BeanPuntoDiRistoro> toReturn = null;
		try
		{
			// Ottengo la lista dei Punti di Ristoro tramite la classe di
			// connessione al database
			// e salvo la lista stessa nell'ArrayList
			toReturn = puntoRistoro.ottieniListaPR();
		}
		// Eccezione nelle operazioni sul database
		catch (SQLException e)
		{
			System.out.println("Errore nel metodo ottieniPuntiDiRistoro: "
					+ e.toString());
			throw new RemoteException(MessaggiErrore.ERRORE_DBMS);
		}
		// Eccezioni impreviste determinate da altri fattori
		catch (Exception ee)
		{
			System.out.println("Errore nel metodo ottieniPuntiDiRistoro: "
					+ ee.toString());
			throw new RemoteException(MessaggiErrore.ERRORE_SCONOSCIUTO);
		}
		// Controllo l'ArrayList da ritornare in modo da non passare valori null
		// al chiamante
		if (toReturn == null)
			throw new RemoteException(MessaggiErrore.ERRORE_FORMATO_BEAN);
		// Ritorno l'ArrayList con tutti i Punti di Ristoro
		return toReturn;
	}

	// Metodo che permette di ottenere tutti i punti di Ristoro che abbiano
	// una convenzione attiva o meno a seconda del parametro passato
	public ArrayList<BeanPuntoDiRistoro> ottieniPuntiDiRistoro(
			boolean statoConvenzione) throws RemoteException
	{
		// Array che mi permette di memorizzare tutti i Punti di Ristoro e dal
		// quale rimuoverò
		// a seconda del parametro passato i Punti di Ristoro attivi o meno
		ArrayList<BeanPuntoDiRistoro> toReturn = null;
		// Array che mi permette di memorizzare tutti i Punti di Ristoro attivi
		// tramite la connessione al database
		ArrayList<BeanPuntoDiRistoro> attivi = null;
		// Istanza per la connessione al database
		IDBConvenzione conv = new DBConvenzione();
		try
		{
			// Ricavo dalla connessione tutti i Punti di Ristoro Attivi
			attivi = conv.ottieniListaConvenzioneAttivaPR();
		}
		// Eccezione nelle operazioni sul database
		catch (SQLException e)
		{
			System.out
					.println("Errore nel metodo ottieniPuntiDiRistoro(boolean): "
							+ e.toString());
			throw new RemoteException(MessaggiErrore.ERRORE_DBMS);
		}
		// Eccezioni impreviste determinate da altri fattori
		catch (Exception ee)
		{
			System.out
					.println("Errore nel metodo ottieniPuntiDiRistoro(boolean): "
							+ ee.toString());
			throw new RemoteException(MessaggiErrore.ERRORE_SCONOSCIUTO);
		}
		// Se si vogliono ottenere i Punti Di Ristoro attivi, allora ritorno
		// direttamente quelli passati dalla connessione al database
		if (statoConvenzione)
		{
			// Controllo il contenuto dell'ArrayList in modo da non ritornare
			// valori nulli al chiamante
			if (attivi == null)
				throw new RemoteException(MessaggiErrore.ERRORE_FORMATO_BEAN);
			return attivi;
		}
		else
		{
			try
			{
				// Ricavo dalla connessione tutti i Punti di Ristoro per poi
				// effettuare i confronti
				toReturn = puntoRistoro.ottieniListaPR();
			}
			// Eccezione nelle operazioni sul database
			catch (SQLException e)
			{
				System.out
						.println("Errore nel metodo ottieniPuntiDiRistoro(boolean): "
								+ e.toString());
				throw new RemoteException(MessaggiErrore.ERRORE_DBMS);
			}
			// Eccezioni impreviste determinate da altri fattori
			catch (Exception ee)
			{
				System.out
						.println("Errore nel metodo ottieniPuntiDiRistoro(boolean): "
								+ ee.toString());
				throw new RemoteException(MessaggiErrore.ERRORE_SCONOSCIUTO);
			}
			// Dimensione dell'ArrayList contenente tutti i Punti di Ristoro che
			// potrebbe
			// cambiare dimensione se viene rimosso qualche elemento
			int dim = toReturn.size();
			// Variabile che mi permette di capire se rimuovere o meno un Punto
			// di Ristoro
			// dall'Array che poi devo ritornare
			boolean presente = false;
			// Primo ciclo per scorrere tutti gli elementi dell'ArrayList di
			// Tutti i Punti di Ristoro
			for (int i = 0; i < dim; i++)
			{
				// Secondo ciclo per scorrere tutti gli elementi dell'ArrayList
				// coni soli Punti di Ristoro attivi
				for (int j = 0; j < attivi.size(); j++)
				{
					// Se il Punto di Ristoro in questione ha l'ID uguale a uno
					// di quelli attivi, allora imposto presente a true
					if (attivi.get(j).getId() == toReturn.get(i).getId())
						presente = true;
				}
				// Se il Punto di Ristoro in questione ha una convenzione attiva
				// lo rimuovo da quelli da restituire
				if (presente)
					toReturn.remove(i);
			}
		}
		// Ritorno l'ArrayList ottenuto
		return toReturn;
	}

	// Metodo che permette di modificare un Punto di Ristoro passato come
	// parametro
	public boolean modificaPuntoDiRistoro(
			BeanPuntoDiRistoro pPuntoDiRistoroAggiornato)
			throws RemoteException
	{
		// Controllo la validità del bean passato come parametro e nel caso
		// scateno un'eccezione remota
		if (pPuntoDiRistoroAggiornato == null
				|| (!(pPuntoDiRistoroAggiornato instanceof BeanPuntoDiRistoro)))
			throw new RemoteException(MessaggiErrore.ERRORE_FORMATO_BEAN);
		try
		{
			// Richiamo il metodo con la connessione al database per modificare
			// il Punto di Ristoro
			if (puntoRistoro.modificaPuntoDiRistoro(pPuntoDiRistoroAggiornato))
				// Ritorno un valore positivo se l'operazione è andata a buon
				// fine
				return true;
		}
		// Eccezione nelle operazioni sul database
		catch (SQLException e)
		{
			System.out.println("Errore nel metodo modificaPuntoDiRistoro: "
					+ e.toString());
			throw new RemoteException(MessaggiErrore.ERRORE_DBMS);
		}
		// Eccezioni impreviste determinate da altri fattori
		catch (Exception ee)
		{
			System.out.println("Errore nel metodo modificaPuntoDiRistoro: "
					+ ee.toString());
			throw new RemoteException(MessaggiErrore.ERRORE_SCONOSCIUTO);
		}
		// Ritorno false se l'operazione non è andata a buon fine
		return false;
	}

	// Metodo che permette di ottenere il Bean di un particolare Punto di
	// Ristoro il cui
	// identificativo è passato come parametro
	public BeanPuntoDiRistoro ottieniPuntoDiRistoro(int pPuntoDiRistoroID)
			throws RemoteException
	{
		// Controllo la validità dell'identificativo
		if (pPuntoDiRistoroID < 0)
			throw new RemoteException(MessaggiErrore.ERRORE_DATI);
		// Bean da ritornare al chiamante
		BeanPuntoDiRistoro toReturn = null;
		try
		{
			// Ricavo il Punto di Ristoro in questione tramite la connessione al
			// database
			toReturn = puntoRistoro.ottieniPuntoDiRistoro(pPuntoDiRistoroID);
		}
		// Eccezione nelle opearazioni sul database
		catch (SQLException e)
		{
			throw new RemoteException(MessaggiErrore.ERRORE_DBMS);
		}
		// Eccezioni impreviste causate da altri fattori
		catch (Exception ee)
		{
			System.out.println("Errore nel metodo ottieniPuntoDiRistoro: "
					+ ee.toString());
			throw new RemoteException(MessaggiErrore.ERRORE_SCONOSCIUTO);
		}
		// Controllo il bean da restituire in modo da non ritornare valori nulli
		// al chiamante
		if (toReturn == null)
			throw new RemoteException(MessaggiErrore.ERRORE_FORMATO_BEAN);
		// Ritorno il bean del Punto di Ristoro
		return toReturn;
	}

	// Metodo che permette di attivare una particolare convenzione di un Punto
	// di Ristoro passato come parametro
	public boolean attivaConvenzione(int pPuntoDiRistoroID,
			BeanConvenzione pConv) throws RemoteException
	{
		// Controllo la validità dei parametri passati
		if ((pPuntoDiRistoroID < 0) || (pConv == null)
				|| (!(pConv instanceof BeanConvenzione)))
			throw new RemoteException(MessaggiErrore.ERRORE_DATI);
		// Controllo ulteriore sui dati
		if (pConv.getIdPuntoDiRistoro() != pPuntoDiRistoroID)
			throw new RemoteException(MessaggiErrore.ERRORE_DATI);
		IDBConvenzione conv = null;
		try
		{
			// Istanzio la classe per la connessione al database
			conv = new DBConvenzione();
			// Se la convenzione non è ancora attiva, allora provvedo ad
			// attivarla in locale e poi passare il bean modificato al database
			if (conv.ottieniConvezioneAttiva(pPuntoDiRistoroID) == null)
			{
				pConv.setAttiva(true);
				conv.modificaConvenzione(pConv);
				return true;
			}
		}
		// Eccezione nelle operazioni sul database
		catch (SQLException e)
		{
			throw new RemoteException(MessaggiErrore.ERRORE_DBMS);
		}
		// Eccezioni impreviste causate da altri fattori
		catch (Exception ee)
		{
			System.out.println("Errore nel metodo attivaConvenzione: "
					+ ee.toString());
			throw new RemoteException(MessaggiErrore.ERRORE_SCONOSCIUTO);
		}
		// Se l'operazione non è andata a buon fine ritorno false
		return false;
	}

	// Metodo che restituisce una HashMap contenente, per il Punto di Ristoro
	// passato come parametro, i feedback ad esso associati
	public HashMap<BeanVisitaPR, String> ottieniFeedbackPuntoDiRistoro(
			int pPuntoDiRistoroID) throws RemoteException
	{
		// Controllo l'identificativo passato come parametro
		if (pPuntoDiRistoroID < 0)
			throw new RemoteException(MessaggiErrore.ERRORE_DATI);
		// Istanzio la mappa e l'ArrayList che utilizzerò nell'esecuzione del
		// metodo
		HashMap<BeanVisitaPR, String> mappaRitorno = null;
		ArrayList<BeanVisitaPR> bvisita = null;
		try
		{
			// Qui prendo la lista di tutte le visite per il PR passato come
			// parametro
			bvisita = feed.ottieniListaVisitaPR(pPuntoDiRistoroID);
			// Istanzio la mappa delle stesse dimensioni della lista di
			// BeanVisitaPR
			mappaRitorno = new HashMap<BeanVisitaPR, String>(bvisita.size());
			// Qui inizio a iterare per aggiungere a ogni visita il relativo
			// username
			for (Iterator<BeanVisitaPR> iteratoreVisitaPR = bvisita.iterator(); iteratoreVisitaPR
					.hasNext();)
			{
				// Recuperto il BeanVisitaPR
				BeanVisitaPR bVisitaTemp = iteratoreVisitaPR.next();
				// Recupero il Turista che ha lasciato il commento che sto
				// esaminando
				BeanTurista bTuristaTemp = dbTurista.ottieniTurista(bVisitaTemp
						.getIdTurista());
				// Prendo l'username del Turista
				String usernameTuristaTemp = bTuristaTemp.getUsername();
				// Inserisco la coppia nella mappa
				mappaRitorno.put(bVisitaTemp, usernameTuristaTemp);
			}
		}
		// Eccezione nelle operazioni sul Database
		catch (SQLException e)
		{
			System.out.println("Errore nel metodo ottieniFeedbackPR: "
					+ e.toString());
			throw new RemoteException(MessaggiErrore.ERRORE_DBMS);
		}
		// Eccezioni impreviste causate da altri fattori
		catch (Exception ee)
		{
			System.out.println("Errore nel metodo ottieniFeedbackPR: "
					+ ee.toString());
			throw new RemoteException(MessaggiErrore.ERRORE_SCONOSCIUTO);
		}
		// Controllo il parametro di ritorno in modo da non passare valori nulli
		// al database
		if (mappaRitorno == null)
			throw new RemoteException(MessaggiErrore.ERRORE_FORMATO_BEAN);
		return mappaRitorno;
	}

	// Metodo che inserisce un tag tra quelli di un Punto Di Ristoro
	public boolean cancellaTagPuntoDiRistoro(int pPuntoDiRistoroId, int pTagId)
			throws RemoteException
	{
		// Controlla la validità dei dati passati
		if ((pPuntoDiRistoroId < 0) || (pTagId < 0))
			throw new RemoteException(MessaggiErrore.ERRORE_DATI);
		// ArrayList nel quale memorizzo tutti i tag
		ArrayList<BeanTag> tags;
		// Variabile booleana per controllare se il Punto di Ristoro
		// possiede il tag che si vuole cancellare
		boolean presente = false;
		try
		{
			// Utilizzo il metodo tramite la classe di connessione al database
			tags = tag.ottieniTagPuntoDiRistoro(pPuntoDiRistoroId);
		}
		// Eccezione nell'esecuzione delle operazioni su database
		catch (SQLException e)
		{
			System.out.println("Errore nel metodo cancellaTagPuntoDiRistoro: "
					+ e.toString());
			throw new RemoteException(MessaggiErrore.ERRORE_DBMS);
		}
		// Eccezione imprevista dovuta ad altri fattori
		catch (Exception ee)
		{
			System.out.println("Errore nel metodo cancellaTagPuntoDiRistoro: "
					+ ee.toString());
			throw new RemoteException(MessaggiErrore.ERRORE_SCONOSCIUTO);
		}
		// Controllo ciclicamente se il tag è presente attualmente
		// tra quelli del Punto di Ristoro
		for (BeanTag t : tags)
			if (t.getId() == pTagId)
				presente = true;
		// Se il tag è presente tra quelli del Punto di Ristoro, allora
		// provvedo ad eseguire l'operazione di cancellazione
		if (presente)
		{
			try
			{
				return tag.cancellaTagPuntoDiRistoro(pPuntoDiRistoroId, pTagId);
			}
			// Eccezione nell'esecuzione dell'operazione sul database
			catch (SQLException e)
			{
				System.out
						.println("Errore nel metodo cancellaTagPuntoDiRistoro: "
								+ e.toString());
				throw new RemoteException(MessaggiErrore.ERRORE_DBMS);
			}
			// Eccezione imprevista dovuta ad altri fattori
			catch (Exception ee)
			{
				System.out
						.println("Errore nel metodo cancellaTagPuntoDiRistoro: "
								+ ee.toString());
				throw new RemoteException(MessaggiErrore.ERRORE_SCONOSCIUTO);
			}
		}
		// Nel caso in cui qualcosa non sia andato a buon fine
		// ritorno il valore false
		return false;
	}

	// Metodo che cancella un tag da quelli di un Punto Di Ristoro
	// Le operazioni sono identiche a quelle sopra, eccezion fatta per
	// il controllo sulla presenza del tag tra quelli del Punto di Ristoro
	// che deve dare esito negativo, e la chiamata che qui è al metodo di
	// inserimento
	public boolean inserisciTagPuntoDiRistoro(int pPuntoDiRistoroId, int pTagId)
			throws RemoteException
	{
		if ((pPuntoDiRistoroId < 0) || (pTagId < 0))
			throw new RemoteException(MessaggiErrore.ERRORE_DATI);

		ArrayList<BeanTag> tags;
		boolean presente = false;
		try
		{
			tags = tag.ottieniTagPuntoDiRistoro(pPuntoDiRistoroId);
		}
		catch (SQLException e)
		{
			System.out.println("Errore nel metodo inserisciTagPuntoDiRistoro: "
					+ e.toString());
			throw new RemoteException(MessaggiErrore.ERRORE_DBMS);
		}
		catch (Exception ee)
		{
			System.out.println("Errore nel metodo inserisciTagPuntoDiRistoro: "
					+ ee.toString());
			throw new RemoteException(MessaggiErrore.ERRORE_SCONOSCIUTO);
		}
		for (BeanTag t : tags)
			if (t.getId() == pTagId)
				presente = true;
		// Controllo che il Punto di Ristoro non abbia già il Tag specificato
		if (!presente)
		{
			try
			{
				// Chiamata al metodo di aggiunta tramite la classe di
				// connessione al database
				return tag.aggiungeTagPuntoDiRistoro(pPuntoDiRistoroId, pTagId);
			}
			catch (SQLException e)
			{
				System.out
						.println("Errore nel metodo inserisciTagPuntoDiRistoro: "
								+ e.toString());
				throw new RemoteException(MessaggiErrore.ERRORE_DBMS);
			}
			catch (Exception ee)
			{
				System.out
						.println("Errore nel metodo inserisciTagPuntoDiRistoro: "
								+ ee.toString());
				throw new RemoteException(MessaggiErrore.ERRORE_SCONOSCIUTO);
			}
		}
		// Ritorno false se qualche operazione non è andata a buon fine
		return false;
	}
}
