package unisa.gps.etour.control.GestioneTag;

import java.rmi.RemoteException;
import java.sql.SQLException;
import unisa.gps.etour.bean.BeanTag;
import unisa.gps.etour.util.MessaggiErrore;

/**
 * Classe che implementa i metodi per le funzionalit� dell'Operatore Agenzia
 * estendendo la classe di Gestione Tag Comune
 * 
 * @author Morelli Giuseppe
 * @version 0.1 � 2007 eTour Project - Copyright by SE@SA Lab DMI � University
 *          of Salerno
 */
public class GestioneTagOperatoreAgenzia extends GestioneTagComune implements
		IGestioneTagOperatoreAgenzia
{

	private static final long serialVersionUID = 1L;

	public GestioneTagOperatoreAgenzia() throws RemoteException
	{
		// Richiama il costruttore della superclasse per la comunicazione col
		// Database
		super();
	}

	// Metodo che cancella dal Database il tag il cui identificativo è passato
	// come parametro
	public boolean cancellaTag(int pTagID) throws RemoteException
	{
		// Controllo la valisità dei dati passati
		if ((pTagID <= 0))
			throw new RemoteException(MessaggiErrore.ERRORE_DATI);
		try
		{
			// Effettuo l'operazione sul database
			tag.cancellaTag(pTagID);
			return true;
		}
		// Eccezione nell'esecuzione delle operazioni sul Database
		catch (SQLException e)
		{
			System.out
					.println("Errore nel metodo cancellaTag: " + e.toString());
			throw new RemoteException(MessaggiErrore.ERRORE_DBMS);
		}
		// Eccezioni impreviste causate da altri fattori
		catch (Exception ee)
		{
			System.out.println("Errore nel metodo cancellaTag: "
					+ ee.toString());
			throw new RemoteException(MessaggiErrore.ERRORE_SCONOSCIUTO);
		}
	}

	// Metodo che permette l'inserimento di un nuovo Tag passato come parametro
	public boolean inserisciTag(BeanTag pTagNuovo) throws RemoteException
	{
		// Controllo la validità del Bean e dei dati contenuti al suo interno
		if (pTagNuovo == null)
			throw new RemoteException(MessaggiErrore.ERRORE_FORMATO_BEAN);
		if ((pTagNuovo = checkTag(pTagNuovo)) == null)
			throw new RemoteException(MessaggiErrore.ERRORE_FORMATO_BEAN);
		try
		{
			// Esecuzione dell'operazione sul Database
			tag.inserisciTag(pTagNuovo);
			return true;
		}
		// Eccezione nell'esecuzione dell'Operazione sul Database
		catch (SQLException e)
		{
			System.out.println("Errore nel metodo inserisciTag: "
					+ e.toString());
			throw new RemoteException(MessaggiErrore.ERRORE_DBMS);
		}
		// Eccezioni impreviste causate da altri fattori
		catch (Exception ee)
		{
			System.out.println("Errore nel metodo inserisciTag: "
					+ ee.toString());
			throw new RemoteException(MessaggiErrore.ERRORE_SCONOSCIUTO);
		}
	}

	// Metodo che permette la modifica di un Tag che è quello passato come
	// parametro
	public boolean modificaTag(BeanTag pTagModificato) throws RemoteException
	{
		// Controllo la validità dei dati
		if ((pTagModificato = checkTag(pTagModificato)) == null)
			throw new RemoteException(MessaggiErrore.ERRORE_FORMATO_BEAN);
		try
		{
			// Esecuzione dell'operazione sul Database
			tag.modificaTag(pTagModificato);
			return true;
		}
		// Eccezione nell'esecuzione dell'operazione sul Database
		catch (SQLException e)
		{
			System.out
					.println("Errore nel metodo modificaTag: " + e.toString());
			throw new RemoteException(MessaggiErrore.ERRORE_DBMS);
		}
		// Eccezioni impreviste causate da altri fattori
		catch (Exception ee)
		{
			System.out.println("Errore nel metodo modificaTag: "
					+ ee.toString());
			throw new RemoteException(MessaggiErrore.ERRORE_SCONOSCIUTO);
		}
	}

	// Metodo che permette di ottenere il Tag il cui identificativo è passato
	// come parametro
	public BeanTag ottieniTag(int pTagID) throws RemoteException
	{
		// Controllo la validità dei dati
		if (pTagID <= 0)
			throw new RemoteException(MessaggiErrore.ERRORE_DATI);
		// Bean da ritornare
		BeanTag toReturn;
		try
		{
			// Esecuzione dell'operazione sul Database
			toReturn = tag.ottieniTag(pTagID);
		}
		// Eccezione nell'esecuzione dell'Operazione sul Database
		catch (SQLException e)
		{
			System.out.println("Errore nel metodo ottieniTag: " + e.toString());
			throw new RemoteException(MessaggiErrore.ERRORE_DBMS);
		}
		// Eccezioni impreviste causate da altri fattori
		catch (Exception ee)
		{
			System.out
					.println("Errore nel metodo ottieniTag: " + ee.toString());
			throw new RemoteException(MessaggiErrore.ERRORE_SCONOSCIUTO);
		}
		if (toReturn == null)
			throw new RemoteException(MessaggiErrore.ERRORE_FORMATO_BEAN);
		return toReturn;
	}

	// Metodo che controlla tutti gli attributi di un BeanTag
	private BeanTag checkTag(BeanTag toControl)
	{
		// Controllo dell'ID
		if (toControl.getId() <= 0)
			return null;
		// Controllo la descrizione
		if (toControl.getDescrizione().equals(""))
			toControl.setDescrizione("***");
		// Controllo il nome
		if (toControl.getNome().equals(""))
			return null;
		// Controllo che il nome non contenga uno spazio
		if (toControl.getNome().contains(" "))
			return null;
		return toControl;
	}
}
