﻿package unisa.gps.etour.control.GestioneUtentiRegistrati;

import java.rmi.RemoteException;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Iterator;
import unisa.gps.etour.bean.BeanTurista;
import unisa.gps.etour.bean.BeanVisitaBC;
import unisa.gps.etour.bean.BeanVisitaPR;
import unisa.gps.etour.repository.DBVisitaBC;
import unisa.gps.etour.repository.DBVisitaPR;
import unisa.gps.etour.repository.IDBVisitaBC;
import unisa.gps.etour.repository.IDBVisitaPR;
import unisa.gps.etour.util.MessaggiErrore;

/**
 * Classe che implementa l'interfaccia per la gestione dei Turisti dal lato
 * dell'agenzia ed estende UnicastRemoteObject per la comunicazione in remoto
 * Offre metodi di base per la gestione e metodi aggiuntivi per la restituzione
 * di Turisti con particolari caratteristiche
 * 
 * @author Morelli Giuseppe
 * @version 0.1 � 2007 eTour Project - Copyright by SE@SA Lab DMI � University
 *          of Salerno
 */
public class GestioneTuristiAgenzia extends GestioneTuristaComune implements
		IGestioneTuristiAgenzia
{

	private IDBVisitaBC feedbackBC;
	private IDBVisitaPR feedbackPR;

	public GestioneTuristiAgenzia() throws RemoteException
	{
		super();
		try
		{
			// Istanzio gli oggetti per le connessioni al database
			feedbackBC = new DBVisitaBC();
			feedbackPR = new DBVisitaPR();
		}
		// Eccezione relativa alla connessione al database
		catch (Exception e)
		{
			throw new RemoteException(MessaggiErrore.ERRORE_SCONOSCIUTO);
		}
	}

	// Metodo che permette di ottenere indistintamente tutti i Turisti presenti
	// nel database
	public ArrayList<BeanTurista> ottieniTuristi() throws RemoteException
	{
		// oggetto che conterrà il valore di ritorno
		ArrayList<BeanTurista> toReturn;
		// Recupero i dati
		try
		{
			// Richiama il metodo con stringa vuota per ottenere tutti i Turisti
			toReturn = turista.ottieniTuristi("");
			if (toReturn == null)
				throw new RemoteException(MessaggiErrore.ERRORE_DBMS);
		}
		// Eccezione nelle operazioni sul Database
		catch (SQLException e)
		{
			// se dal data layer viene lanciata una eccezione SQLException
			// si lancia l'eccezione remota RemoteException
			System.out.println("Errore nel metodo ottieniTuristi: "
					+ e.toString());
			throw new RemoteException(MessaggiErrore.ERRORE_DBMS);
		}
		// Eccezione imprevista causata da altri fattori
		catch (Exception ee)
		{
			System.out.println("Errore nel metodo ottieniTuristi: "
					+ ee.toString());
			throw new RemoteException(MessaggiErrore.ERRORE_SCONOSCIUTO);
		}
		// restituisce il risultato ottenuro dal data layer
		if (toReturn == null)
			throw new RemoteException(MessaggiErrore.ERRORE_FORMATO_BEAN);
		return toReturn;
	}

	// Metodo che permette di ottenere tutti i Turisti che abbiano un account
	// attivo o meno
	public ArrayList<BeanTurista> ottieniTuristi(boolean statoAccount)
			throws RemoteException
	{
		// ArrayList contenente i risultati
		ArrayList<BeanTurista> toReturn;
		// Recupero dei dati
		try
		{
			// si prelevano tutti i Turisti con lo stato passato come parametro
			toReturn = turista.ottieniTuristi(statoAccount);
		}
		// Eccezione nell'esecuzione dell'operazione sul database
		catch (SQLException e)
		{
			// se il data layer invia una eccezione si lancia l'eccezione remota
			System.out.println("Errore nel metodo ottieniTuristi(boolean): "
					+ e.toString());
			throw new RemoteException(MessaggiErrore.ERRORE_DBMS);
		}
		// Eccezioni impreviste causate da altri fattori
		catch (Exception ee)
		{
			System.out.println("Errore nel metodo ottieniTuristi(boolean): "
					+ ee.toString());
			throw new RemoteException(MessaggiErrore.ERRORE_SCONOSCIUTO);
		}
		// Controllo i dati da ritornare, in modo da non restituire valori nulli
		if (toReturn == null)
			throw new RemoteException(MessaggiErrore.ERRORE_FORMATO_BEAN);
		return toReturn;
	}

	// Metodo che permette l'attivazione di un Turista non ancora attivato
	public boolean attivaTurista(int pIdTurista) throws RemoteException
	{
		// Controllo la validità dei dati di input
		if (pIdTurista < 0)
			throw new RemoteException(MessaggiErrore.ERRORE_DATI);
		// bean contenente i dati del turista da attivare
		BeanTurista toChange;
		// Recupero e modifica dei dati richiesti
		try
		{
			// Ottengo i dati dal Database
			toChange = turista.ottieniTurista(pIdTurista);
			// Controllo che il Turista non sia già attivato
			if (toChange.isAttiva())
				throw new RemoteException(MessaggiErrore.ERRORE_DATI);
			// si setta il valore che indica lo stato di attivazione a true
			toChange.setAttiva(true);
			// si tenta di modificare i dati del database
			if (turista.modificaTurista(toChange))
				// se l'operazione di modifica va a buon fine si restituisce
				// true
				return true;
		}
		// Eccezione nell'esecuzione delle operazioni sul Database
		catch (SQLException e)
		{
			// se il data layer invia una eccezione si lancia l'eccezione remota
			System.out.println("Errore nel metodo attivaTurista: "
					+ e.toString());
			throw new RemoteException(MessaggiErrore.ERRORE_DBMS);
		}
		// Eccezioni impreviste causate da altri fattori
		catch (Exception ee)
		{
			System.out.println("Errore nel metodo attivaTurista: "
					+ ee.toString());
			throw new RemoteException(MessaggiErrore.ERRORE_SCONOSCIUTO);
		}
		// Se non sono state lanciate eccezioni ma le modifiche
		// non sono andate a buon fine si restituisce false
		return false;
	}

	// Metodo che permette di disattivare un Turista
	public boolean disattivaTurista(int pIdTurista) throws RemoteException
	{
		// Controllo la validità dei dati
		if (pIdTurista < 0)
			throw new RemoteException(MessaggiErrore.ERRORE_DATI);
		BeanTurista toChange;// bean contenente i dati del turista da
		// disattivare
		// Recupero e modifica dei dati richiesti
		try
		{
			// Ottengo i dati dal database
			toChange = turista.ottieniTurista(pIdTurista);
			// Controllo che il turista sia attivo
			if (!toChange.isAttiva())
				throw new RemoteException(MessaggiErrore.ERRORE_DATI);
			// si setta il valore che indica lo stato di attivazione a false
			toChange.setAttiva(false);
			// si tenta di modificare i dati del database
			if (turista.modificaTurista(toChange))
				// se l'operazione di modifica va a buon fine si restituisce
				// true
				return true;
		}
		// Eccezione nell'esecuzione delle operazioni sul database
		catch (SQLException e)
		{
			// se il data layer invia una eccezione si lancia l'eccezione remota
			System.out.println("Errore nel metodo disattivaTurista: "
					+ e.toString());
			throw new RemoteException(MessaggiErrore.ERRORE_DBMS);
		}
		// Eccezioni impreviste causate da altri fattori
		catch (Exception ee)
		{
			System.out.println("Errore nel metodo disattivaTurista: "
					+ ee.toString());
			throw new RemoteException(MessaggiErrore.ERRORE_SCONOSCIUTO);
		}
		// Se non sono state lanciate eccezioni ma le modifiche
		// non sono andate a buon fine si restituisce false
		return false;
	}

	// Metodo che permette di cancellare un Turista dal Database
	public boolean cancellaTurista(int pIdTurista) throws RemoteException
	{
		// Controllo la validità dei dati
		if (pIdTurista < 0)
			throw new RemoteException(MessaggiErrore.ERRORE_DATI);
		try
		{
			// se la cancellazione va a buon fine si restituisce true
			if (turista.cancellaTurista(pIdTurista))
				return true;
		}
		catch (SQLException e)
		{
			// se il data layer invia una eccezione si lancia l'eccezione remota
			System.out.println("Errore nel metodo cancellaTurista: "
					+ e.toString());
			throw new RemoteException(MessaggiErrore.ERRORE_DBMS);
		}
		// Eccezione imprevista causata da altri fattori
		catch (Exception ee)
		{
			System.out.println("Errore nel metodo cancellaTurista: "
					+ ee.toString());
			throw new RemoteException(MessaggiErrore.ERRORE_SCONOSCIUTO);
		}
		// altrimenti se non sono state lanciate eccezioni ma la cancellazione
		// non sono andate a buon fine si restituisce false
		return false;
	}

	// Metodo che restituisce un ArrayList contenente i feedback rilasciati da
	// un certo Turista per i Beni Culturali
	public ArrayList<BeanVisitaBC> ottieniFeedbackBC(int pIdTurista)
			throws RemoteException
	{
		// Controllo la validità dei dati
		if (pIdTurista < 0)
			throw new RemoteException(MessaggiErrore.ERRORE_DATI);
		// ArrayList di ritorno
		ArrayList<BeanVisitaBC> toReturn;
		try
		{
			// Recupero i dati dal Database
			toReturn = feedbackBC.ottieniListaVisitaBCTurista(pIdTurista);
		}
		catch (SQLException e)
		{
			// se il data layer invia una eccezione si lancia l'eccezione remota
			System.out.println("Errore nel metodo ottieniFeedbackBC: "
					+ e.toString());
			throw new RemoteException(MessaggiErrore.ERRORE_DBMS);
		}
		// Eccezione imprevista causata da altri fattori
		catch (Exception ee)
		{
			System.out.println("Errore nel metodo ottieniFeedbackBC: "
					+ ee.toString());
			throw new RemoteException(MessaggiErrore.ERRORE_SCONOSCIUTO);
		}
		// Controllo i dati di ritorno in modo da non restituire valori nulli
		if (toReturn == null)
			throw new RemoteException(MessaggiErrore.ERRORE_FORMATO_BEAN);
		return toReturn;
	}

	// Metodo che restituisce un ArrayList contenente i feedback rilasciati da
	// un certo Turista per i Beni Culturali
	public ArrayList<BeanVisitaPR> ottieniFeedbackPR(int pIdTurista)
			throws RemoteException
	{
		// Controllo la validità dei dati
		if (pIdTurista < 0)
			throw new RemoteException(MessaggiErrore.ERRORE_DATI);
		ArrayList<BeanVisitaPR> toReturn;// variabile di ritorno
		try
		{
			// si inserisce in toReturn il risultato
			// restituito dal data layer
			toReturn = feedbackPR.ottieniListaVisitaPRTurista(pIdTurista);
		}
		catch (SQLException e)
		{
			// se il data layer invia una eccezione si lancia l'eccezione remota
			System.out.println("Errore nel metodo ottieniFeedbackPR: "
					+ e.toString());
			throw new RemoteException(MessaggiErrore.ERRORE_DBMS);
		}
		// Eccezioni impreviste causate da altri fattori
		catch (Exception ee)
		{
			System.out.println("Errore nel metodo ottieniFeedbackPR: "
					+ ee.toString());
			throw new RemoteException(MessaggiErrore.ERRORE_SCONOSCIUTO);
		}
		// Controllo l'Array di ritorno, in modo da non restituire valori nulli
		if (toReturn == null)
			throw new RemoteException(MessaggiErrore.ERRORE_FORMATO_BEAN);
		return toReturn;
	}
}
