package unisa.gps.etour.gui;

import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.*;
import java.util.HashMap;
import java.util.Scanner;
import java.util.StringTokenizer;
import java.net.URISyntaxException;
import javax.swing.JComponent;
import javax.swing.text.JTextComponent;

/**
 * Questa classe realizza il meccanismo della guida sensibile al 
 * contesto.
 * 
 * @version 0.2
 * @author Mario Gallo
 * 
 * &#169; 2007 eTour Project - Copyright by SE@SA Lab DMI -
 *         University of Salerno
 */
public class HelpManager extends MouseAdapter
{
	private HashMap<String,String> dizionario;
	private JTextComponent destinazione;
	private String previousMessage;
	
	/**
	 * <p>Il costruttore crea una nuova istanza della classe per il
	 * managing della guida in linea a partire dal file contentente 
	 * la guida vera e propria e il componente nel quale visualizzarla.</p>
	 * 
	 * @param pPercorso String - il percorso del file della guida.
	 * @param pComponent JTextComponent - il componente nel quale visualizzare
	 * la guida.
	 * @throws FileNotFoundException - se il file della guida indicato non viene
	 * trovato.
	 * 
	 */
	public HelpManager(String pPercorso, JTextComponent pComponent) throws FileNotFoundException
	{
		dizionario = new HashMap<String,String>();
		destinazione = pComponent;
		File aFile = null;
		try
		{
			aFile = new File(getClass().getResource(pPercorso).toURI());
		}
		catch (URISyntaxException ex)
		{
			ex.printStackTrace();
		}
		Scanner scanning = new Scanner(new BufferedReader(new FileReader(aFile)));
		StringTokenizer tokenizer;
		while (scanning.hasNext())
		{
			tokenizer = new StringTokenizer(scanning.nextLine(),"##");
			dizionario.put(tokenizer.nextToken(), tokenizer.nextToken());
		}
		
		scanning.close();
	}

	/**
	 * Questo metodo gestisce l'evento che occorre quando il puntatore
	 * del mouse entra nell'area di un componente.
	 * 
	 * @param pMouseEnteredEvent MouseEvent - l'evento generato dal mouse.
	 * 
	 */
	public void mouseEntered(MouseEvent pMouseEnteredEvent)
	{
		previousMessage = destinazione.getText();
		JComponent componente = (JComponent) pMouseEnteredEvent.getComponent();
		String help = dizionario.get(componente.getName());
		destinazione.setText(help);	
		destinazione.repaint();
	}

	/**
	 * 
	 * Questo metodo gestisce l'evento che occorre quando il puntatore
	 * del mouse esce dall'area di un componente.
	 * 
	 * @param pMouseExitedEvent MouseEvent - l'evento generato dal mouse.
	 * 
	 */
	public void mouseExited(MouseEvent pMouseExitedEvent)
	{
		destinazione.setText(previousMessage);
		destinazione.repaint();

	}
}
