package unisa.gps.etour.repository;

import java.sql.SQLException;
import java.util.ArrayList;

import unisa.gps.etour.bean.BeanConvenzione;
import unisa.gps.etour.bean.BeanPuntoDiRistoro;

/**
 * Interfaccia per la gestione delle convezioni sul database
 * 
 * @author Martone Giuseppe
 * @version 0.1 � 2007 eTour Project - Copyright by SE@SA Lab DMI � University
 *          of Salerno
 */
public interface IDBConvenzione
{

	/**
	 * Aggiunge una convenzione nel database
	 * 
	 * @param pConvenzione Convenzione da aggiungere
	 * @throws SQLException
	 */
	public boolean inserisciConvenzione(BeanConvenzione pConvenzione)
			throws SQLException;

	/**
	 * Modifica una convenzione nel database
	 * 
	 * @param pConvenzione Convenzione dati della convenzione da aggiornare
	 * @return True se c'e' stata una modifica false altrimenti
	 * @throws SQLException
	 */
	public boolean modificaConvenzione(BeanConvenzione pConvenzione)
			throws SQLException;

	/**
	 * Elimina una convenzione dal database
	 * 
	 * @param pIdConvenzione Identificativo della convenzione da eliminare
	 * @return True se � stata cancellata false altrimenti
	 * @throws SQLException
	 */
	public boolean cancellaConvenzione(int pIdConvenzione) throws SQLException;

	/**
	 * Restituisce lo storico delle convenzioni di un punto di ristoro
	 * 
	 * @param idPuntoDiRistoro Identificativo del punto di ristoro
	 * @return Lista delle convenzioni del Punto di ristoro dato come argomento
	 * @throws SQLException
	 */
	public ArrayList<BeanConvenzione> ottieniStoricoConvenzione(
			int idPuntoDiRistoro) throws SQLException;

	/**
	 * Restituisce la convenzione attiva di un punto di ristoro
	 * 
	 * @param pIdPuntoDiRistoro Identificativo del punto di ristoro
	 * @return Convenzione Attiva
	 * @throws SQLException
	 */
	public BeanConvenzione ottieniConvezioneAttiva(int pIdPuntoDiRistoro)
			throws SQLException;

	/**
	 * Restituisce la lista di tutti i PR che hanno una convenzione attiva
	 * 
	 * @return Lista di tutti i PR con convenzione attiva
	 * @throws SQLException
	 */
	public ArrayList<BeanPuntoDiRistoro> ottieniListaConvenzioneAttivaPR()
			throws SQLException;

}
