package unisa.gps.etour.repository;

import java.sql.SQLException;
import java.util.ArrayList;

import unisa.gps.etour.bean.BeanPreferenzaDiRicerca;

/**
 * Interfaccia per la gestione delle preferenze di ricerca nel database
 * 
 * @author Martone Giuseppe
 * @version 0.1 � 2007 eTour Project - Copyright by SE@SA Lab DMI � University
 *          of Salerno
 */
public interface IDBPreferenzeDiRicerca
{
	/**
	 * Aggiunge Una preferenza Di Ricerca
	 * 
	 * @param pPreferenza Preferenza di ricerca
	 * @throws SQLException
	 */
	public boolean inserisciPreferenzaDiRicerca(
			BeanPreferenzaDiRicerca pPreferenza) throws SQLException;

	/**
	 * Elimina una preferenza di ricerca
	 * 
	 * @param pPreferenza Preferenza da eliminare
	 * @throws SQLException
	 * @return True se e' stata cancellata false altrimenti
	 */
	public boolean cancellaPreferenzaDiRicerca(int pIdPreferenza)
			throws SQLException;

	/**
	 * Restituisce la lista delle preferenze di ricerca di un turista
	 * 
	 * @param pIdTurista Id del turista
	 * @throws SQLException
	 * @return Lista delle preferenze di ricerca
	 */
	public ArrayList<BeanPreferenzaDiRicerca> ottieniPreferenzeDiRicercaDelTurista(
			int pIdTurista) throws SQLException;

	/**
	 * Restituisce la lista delle preferenze di ricerca di un bene culturale
	 * 
	 * @param pIdBeneCulturale Identificativo del bene culturale
	 * @throws SQLException
	 * @return Lista delle preferenze di ricerca.
	 */
	public ArrayList<BeanPreferenzaDiRicerca> ottieniPreferenzeDiRicercaDelBC(
			int pIdBeneCulturale) throws SQLException;

	/**
	 * Restituisce la lista delle preferenze di ricerca di un punto di ristoro
	 * 
	 * @param pIdPuntoDiRistoro identificativo di un punto di ristoro
	 * @throws SQLException
	 * @return Lista delle preferenze di ricerca.
	 */
	public ArrayList<BeanPreferenzaDiRicerca> ottieniPreferenzeDiRicercaDelPR(
			int pIdPuntoDiRistoro) throws SQLException;

	/**
	 * Aggiunge una preferenza ad un Bene culturale
	 * 
	 * @param pIdBeneCulturale Identificativo del bene culturale
	 * @param pIdPreferenzaDiRicerca Identificativo della PreferenzaDiRicerca
	 * @throws SQLException
	 * @param pPreferenza Preferenza di ricerca
	 */
	public boolean inserisciPreferenzaDiRicercaDelBC(int pIdBeneCulturale,
			int pIdPreferenzaDiRicerca) throws SQLException;

	/**
	 * Aggiunge una preferenza di ricerca ad un turista
	 * 
	 * @param pIdTurista Id del turista
	 * @param pIdPreferenzaDiRicerca Identificativo delle PreferenzeDiRicerca
	 * @throws SQLException
	 * @param pPreferenza Preferenza di ricerca
	 */
	public boolean inserisciPreferenzaDiRicercaDelTurista(int pIdTurista,
			int pIdPreferenzaDiRicerca) throws SQLException;

	/**
	 * Aggiunge una preferenza di ricerca ad un punto di ristoro
	 * 
	 * @param pIdPuntoDiRistoro Identificativo del punto di ristoro
	 * @param pIdPreferenzaDiRicerca Identificativo della PreferenzaDiRicerca
	 * @throws SQLException
	 * @param pPreferenza Preferenza di ricerca
	 */
	public boolean inserisciPreferenzaDiRicercaDelPR(int pIdPuntoDiRistoro,
			int pIdPreferenzaDiRicerca) throws SQLException;

	/**
	 * Elimina una preferenza di ricerca di un Turista
	 * 
	 * @param pIdTurista Id del turista
	 * @param pIdPreferenza Preferenza di ricerca
	 * @throws SQLException
	 * @return True se e' stata cancellata false altrimenti
	 */
	public boolean cancellaPreferenzaDiRicercaTurista(int pIdTurista,
			int pIdPreferenza) throws SQLException;

	/**
	 * Elimina una preferenza di ricerca di un bene culturale
	 * 
	 * @param pIdPreferenzaDiRicerca Preferenza di ricerca
	 * @param pIdBeneCulturale Identificativo del bene culturale
	 * @throws SQLException
	 * @return True se e' stata cancellata false altrimenti
	 */
	public boolean cancellaPreferenzaDiRicercaBC(int pIdBeneCulturale,
			int pIdPreferenzaDiRicerca) throws SQLException;

	/**
	 * Elimina una preferenza di ricerca di un punto di ristoro
	 * 
	 * @param pIdPreferenza Preferenza di ricerca
	 * @param pIdPuntoDiistoro Identificativo del punto di ristoro
	 * @throws SQLException
	 * @return True se e' stata cancellata false altrimenti
	 */
	public boolean cancellaPreferenzaDiRicercaPR(int pIdPuntoDiistoro,
			int pIdPreferenza) throws SQLException;

	/**
	 * Restituisce la lista di tutte le preferenze di ricerca presenti nel DB
	 * 
	 * @throws SQLException
	 * @return Lista delle preferenze di ricerca presenti nel DB
	 */
	public ArrayList<BeanPreferenzaDiRicerca> ottieniPreferenzeDiRicerca()
			throws SQLException;

}
