package unisa.gps.etour.repository;

import java.sql.SQLException;
import java.util.ArrayList;

import unisa.gps.etour.bean.BeanTag;

/**
 * Interfaccia per la gestione dei Tag nel database
 * 
 * @author Miranda Mauro
 * @version 0.1 � 2007 eTour Project - Copyright by SE@SA Lab DMI � University
 *          of Salerno
 */
public interface IDBTag
{
	/**
	 * Aggiunge un tag
	 * 
	 * @param pTag Tag da aggiungere
	 * @throws SQLException
	 */
	public boolean inserisciTag(BeanTag pTag) throws SQLException;

	/**
	 * Modifica i dati di un tag
	 * 
	 * @param pTag Tag da modificare
	 * @throws SQLException
	 * @return True se e' stato modificato false altrimenti
	 */
	public boolean modificaTag(BeanTag pTag) throws SQLException;

	/**
	 * Elimina un tag dal database
	 * 
	 * @param pIdTag Identificativo Tag da eliminare
	 * @throws SQLException
	 * @return True se e' stato cancellato false altrimenti
	 */
	public boolean cancellaTag(int pIdTag) throws SQLException;

	/**
	 * Restituisce la lista dei tag presenti nel database
	 * 
	 * @throws SQLException
	 * @return Lista contenente i tag
	 */
	public ArrayList<BeanTag> ottieniListaTag() throws SQLException;

	/**
	 * Restituisce un singolo tag
	 * 
	 * @param pId Identificativo del tag
	 * @throws SQLException
	 * @return Tag
	 */
	public BeanTag ottieniTag(int pId) throws SQLException;

	/**
	 * Aggiunge un Tag ad un bene culturale
	 * 
	 * @param pIdBeneCulturale Identificativo del Bene Culturale
	 * @param pIdTag Identificativo del tag
	 * @throws SQLException
	 */
	public boolean aggiungeTagBeneCulturale(int pIdBeneCulturale, int pIdTag)
			throws SQLException;

	/**
	 * Aggiunge un Tag ad un punto di ristoro
	 * 
	 * @param pIdPuntoDiRistoro Identificativo del punto di ristoro
	 * @param pIdTag Identificativo del tag
	 * @throws SQLException
	 */
	public boolean aggiungeTagPuntoDiRistoro(int pIdPuntoDiRistoro, int pIdTag)
			throws SQLException;

	/**
	 * Restituisce la lista dei tag di un bene culturale
	 * 
	 * @param pIdBeneCulturale Identificativo del Bene Culturale
	 * @throws SQLException
	 * @return Lista di tag
	 */
	public ArrayList<BeanTag> ottieniTagBeneCulturale(int pIdBeneCulturale)
			throws SQLException;

	/**
	 * Restituisce la lista dei tag di un punto di ristoro
	 * 
	 * @param pIdPuntoDiRistoro Identificativo del punto di ristoro
	 * @throws SQLException
	 * @return Lista di tag
	 */
	public ArrayList<BeanTag> ottieniTagPuntoDiRistoro(int pIdPuntoDiRistoro)
			throws SQLException;

	/**
	 * Elimina un Tag ad un bene culturale
	 * 
	 * @param pIdBeneCulturale Identificativo del Bene Culturale
	 * @param pIdTag Identificativo del tag
	 * @throws SQLException
	 * @return True se e' stato cancellato false altrimenti
	 */
	public boolean cancellaTagBeneCulturale(int pIdBeneCulturale, int pIdTag)
			throws SQLException;

	/**
	 * Elimina un tag ad un punto di ristoro
	 * 
	 * @param pIdPuntoDiRistoro Identificativo
	 * @param pIdTag Identificativo del tag
	 * @throws SQLException
	 * @return True se e' stato cancellato false altrimenti
	 */
	public boolean cancellaTagPuntoDiRistoro(int pIdPuntoDiRistoro, int pIdTag)
			throws SQLException;

}
