package unisa.gps.etour.control.GestioneBeniCulturali;

import java.rmi.Remote;
import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.HashMap;
import unisa.gps.etour.bean.BeanBeneCulturale;
import unisa.gps.etour.bean.BeanTag;
import unisa.gps.etour.bean.BeanVisitaBC;

/**
 * Interfaccia per le operazioni comuni ad Utenti e Operatore Agenzia sui
 * BeniCulturali
 * 
 * @author De Simone Michelangelo
 * @version 0.1
 * 
 *  &#169; 2007 eTour Project - Copyright by SE@SA Lab DMI - University of Salerno
 */
public interface IGestioneBeniCulturaliComune extends Remote
{
	/**
	 * 
	 * Metodo per la restituzione di un particolare Bene Culturale
	 * 
	 * @param pBeneCulturaleID L'identificativo del bene culturale da restituire
	 * @return BeanBeneCulturale Contiene i dati del Bene Culturale richiesto
	 * @throws RemoteException Eccezione nel flusso
	 */
	public BeanBeneCulturale ottieniBeneCulturale(int pBeneCulturaleID) throws RemoteException;
	
	/** 
	 * Restituisce la lista dei tag di un bene culturale
	 *
	 * @param pBeneCulturaleID Identificativo del Bene Culturale
	 * @return ArrayList<BeanTag> I tag del bene culturale specificato
	 * @throws RemoteException Eccezione nel flusso
	 */
	public ArrayList<BeanTag> ottieniTagBeneCulturale(int pBeneCulturaleID)	throws RemoteException;
	
	/**
	 * 
	 * Restituisce una lista di feedback per il bene culturale specificato
	 *
	 * @param pBeneCulturaleID Identificativo del Bene Culturale
	 * @return HashMap<BeanVisitaBC, String> I feedback del Bene Culturale
	 * @throws RemoteException Eccezione nel flusso
	 */
	public HashMap<BeanVisitaBC, String> ottieniFeedbackBeneCulturale(int pBeneCulturaleID) throws RemoteException;

	/**
	 * 
	 * Restituisce, per il bene culturale specificato, un array in cui ogni posizione contiene il numero di
	 * feedback corrispondenti al valore dell'indice dell'array piu' uno.
	 * Il calcolo e' effettuato nel periodo tra 30 giorni ed oggi.
	 *
	 * @param pBeneCulturaleID Identificativo del Bene Culturale
	 * @return ArrayList<Integer> Le statistiche degli ultimi trenta giorni
	 * @throws RemoteException Eccezione nel flusso
	 */
	public ArrayList<Integer> ottieniStatisticheBeneCulturale(int pBeneCulturaleID) throws RemoteException;

	/**
	 * Metodo per l'aggiornamento (o modifica) di un feedback per un certo bene
	 * culturale. Il metodo ha l'onere di
	 * 
	 * @param pBeneCulturaleID L'identificativo del bene culturale del quale modificare il feedback
	 * @param pBeanVisitaBC Il nuovo feedback per il bene culturale indicato
	 * @return boolean L'esito dell'operazione; true se &egrave; andata a buon fine, false altrimenti
	 * @throws RemoteException Eccezione nel flusso
	 */
	public boolean modificaFeedbackBeneCulturale(int pBeneCulturaleID, BeanVisitaBC pBeanVisitaBC) throws RemoteException;
}
