package unisa.gps.etour.control.GestionePuntiDiRistoro;

import java.rmi.Remote;
import java.rmi.RemoteException;
import java.util.ArrayList;
import unisa.gps.etour.bean.BeanPuntoDiRistoro;
import unisa.gps.etour.bean.BeanTag;
import unisa.gps.etour.bean.BeanVisitaPR;

/**
 * Interfaccia per le operazioni comuni riguardanti i Punti di Ristoro
 * 
 * @author Morelli Giuseppe
 * @version 0.1 � 2007 eTour Project - Copyright by SE@SA Lab DMI � University
 *          of Salerno
 */
public interface IGestionePuntiDiRistoroComune extends Remote
{

	/**
	 * Metodo per la restituzione di un particolare Punto di Ristoro
	 * 
	 * @param pPuntoDiRistoroID per identificare il Punto di Ristoro da
	 *            restituire
	 * @return Bean contenente i dati del Punto di Ristoro interessato
	 * @throws RemoteException Eccezione Remota
	 */
	public BeanPuntoDiRistoro ottieniPuntoDiRistoro(int pPuntoDiRistoroID)
			throws RemoteException;

	/**
	 * Metodo che restituisce i Tag associati ad un certo punto di ristoro
	 * 
	 * @param pPuntoDiRistoroID Identificativo del Punto di Ristoro
	 * @return Struttura contenente tutti i BeanTag associati al punto di
	 *         ristoro passato come parametro
	 * @throws RemoteException Eccezione Remota
	 */
	public ArrayList<BeanTag> ottieniTagPuntoDiRistoro(int pPuntoDiRistoroID)
			throws RemoteException;

	/**
	 * Metodo che restituisce gli ultimi 10 commenti rilasciati per un certo
	 * punto di ristoro
	 * 
	 * @param pPuntoDiRistoroID Identificativo per il Punto di Ristoro in
	 *            questione
	 * @return Array di stringhe contenente 10 elementi
	 * @throws RemoteException Eccezione Remota
	 */
	public String[] ottieniUltimiCommenti(int pPuntoDiRistoroID)
			throws RemoteException;

	/**
	 * Restituisce, per il Punto di Ristoro specificato, un array in cui ogni
	 * posizione contiene il numero di feedback corrispondenti al valore
	 * dell'indice dell'array piu' uno. Il calcolo e' effettuato nel periodo tra
	 * 30 giorni ed oggi.
	 * 
	 * @param pPuntoDiRistoroID Identificativo univoco del Punto di Ristoro
	 * @return ArrayList contenente i contatori come spiegato sopra
	 * @throws RemoteException Eccezione Remota
	 */
	public ArrayList<Integer> ottieniStatistichePuntoDiRistoro(
			int pPuntoDiRistoroID) throws RemoteException;

	/**
	 * Metodo che permette di modificare il commento rilasciato per un certo
	 * Punto Di Ristoro
	 * 
	 * @param pPuntoDiRistoroId Identificativo univoco del Punto Di Ristoro
	 * @param nuovaVisita Bean contenente il nuovo commento
	 * @return Booleano con valore true se l'operazione è andata a buon fine,
	 *         false altrimenti
	 * @throws RemoteException Eccezione Remota
	 */
	public boolean modificaFeedbackPuntoDiRistoro(int pPuntoDiRistoroId,
			BeanVisitaPR nuovaVisita) throws RemoteException;
}
