package unisa.gps.etour.control.GestioneUtentiRegistrati;

import java.rmi.RemoteException;
import java.util.ArrayList;

import unisa.gps.etour.bean.BeanBeneCulturale;
import unisa.gps.etour.bean.BeanPreferenzaDiRicerca;
import unisa.gps.etour.bean.BeanPreferenzeGeneriche;
import unisa.gps.etour.bean.BeanPuntoDiRistoro;
import unisa.gps.etour.bean.BeanTurista;
import unisa.gps.etour.bean.BeanVisitaBC;
import unisa.gps.etour.bean.BeanVisitaPR;

/**
 * Interfaccia relativa alla gestione dei Dati del Turista
 * 
 * @author Penna Giuseppe, Leone Federico
 * @version 0.1 2007 eTour Project - Copyright by SE@SA Lab DMI University of
 *          Salerno
 */
public interface IGestioneTuristaClient extends IGestioneTuristaComune
{

	/**
	 * Metodo per l'inserimento di un Turista
	 * 
	 * @param pTurista Contenitore di tutti i dati relativi al Turista da
	 *            inserire
	 * @return Booleano: True se l'inserimento ha successo, False altrimenti
	 * @throws RemoteException Eccezione Remota
	 */
	public boolean inserisciTurista(BeanTurista pTurista)
			throws RemoteException;

	/**
	 * Metodo per l'inserimento delle Preferenze Generiche del Turista
	 * 
	 * @param pIdTurista Identificativo del Turista al quale associare le
	 *            Preferenze Generiche
	 * @param pPreferenzeGeneriche Le Preferenze Generiche da inserire
	 * @return Booleano: True se l'inserimento ha successo, False altrimenti
	 * @throws RemoteException Eccezione Remota
	 */
	public boolean inserisciPreferenzeGeneriche(
			BeanPreferenzeGeneriche pPreferenzeGeneriche)
			throws RemoteException;

	/**
	 * Metodo per l'estrazione delle Preferenze Generiche dato il Turista
	 * 
	 * @param pIdTurista Identificativo del Turista del quale si vogliono
	 *            ottenere le Preferenze Generiche
	 * @return Le Preferenze Generiche relative al Turista dato
	 * @throws RemoteException Eccezione Remota
	 */
	public BeanPreferenzeGeneriche ottieniPreferenzeGeneriche(int pIdTurista)
			throws RemoteException;

	/**
	 * Metodo per la modifica delle Preferenze Generiche dato il Turista
	 * 
	 * @param pIdTurista Identificativo del Turista per il quale si vogliono
	 *            modificare le Preferenze Generiche
	 * @param pPreferenzeGenericheNuove Le Prefereferze Generiche da inserire
	 * @param pPreferenzeGenericheVecchie Le Preferenze Generiche da sostituire
	 * @return Booleano: True se la madifica ha successo, False altrimenti
	 * @throws RemoteException
	 */
	public boolean modificaPreferenzeGeneriche(
			BeanPreferenzeGeneriche pPreferenzeGenericheNuove)
			throws RemoteException;

	/**
	 * Metodo per la cancellazione delle Preferenze Generiche associate al
	 * Turista
	 * 
	 * @param pIdTurista Identificativo del Turista per il quale si vogliono
	 *            cancellare le Preferenze Generiche
	 * @return Le Preferenze Generiche cancellate
	 * @throws RemoteException Eccezione Remota
	 */
	public BeanPreferenzeGeneriche cancellaPreferenzeGeneriche(int pIdTurista)
			throws RemoteException;

	/**
	 * Metodo per l'inserimento di una Preferenza Di Ricerca
	 * 
	 * @param pIdTurista Identificativo del Turista per il quale si intende
	 *            inserire una Preferenza Di Ricerca
	 * @param pPreferenzaDiRicerca Preferenza Di Ricerca da inserire
	 * @return Boolean: True se l'inserimento ha successo, False altrimenti
	 * @throws RemoteException
	 */
	public boolean inserisciPreferenzaDiRicerca(int pIdTurista,
			BeanPreferenzaDiRicerca pPreferenzaDiRicerca)
			throws RemoteException;

	/**
	 * Metodo per l'estrazione dell'insieme di Preferenze Di Ricerca dato il
	 * Turista
	 * 
	 * @param pIdTurista Identificativo del Turista per il quale si vogliono
	 *            estrarre le Preferenze Di Ricerca
	 * @return Insieme delle Preferenze Di Ricerca associate al Turista dato
	 * @throws RemoteException Eccezione Remota
	 */
	public BeanPreferenzaDiRicerca[] ottieniPreferenzeDiRicerca(
			int pIdTurista) throws RemoteException;

	/**
	 * Metodo per la cancellazione di una Preferenza di Ricerca dato il suo
	 * identificativo e il Turista
	 * 
	 * @param pIdTurista Identificativo del Turista per il quale si vuole
	 *            cancellare una Preferenza di Ricerca
	 * @param pIdPreferenzaDiRicerca Identificativo della Preferenza Di Ricerca
	 *            da cancellare
	 * @return La Preferenza Di Ricerca cancellata
	 * @throws RemoteException Eccezione Remota
	 */
	public BeanPreferenzaDiRicerca cancellaPreferenzeDiRicerca(int pIdTurista,
			int pIdPreferenzaDiRicerca) throws RemoteException;

	/**
	 * Metodo per l'estrazione della lista ai riferimenti ai Beni Culturali
	 * visitati da un Turista
	 * 
	 * @param pIdTurista Identificativo del Turista
	 * @return Lista di riferimenti ai Beni Culturali visitati
	 * @throws RemoteException Eccezione Remota
	 */
	public BeanVisitaBC[] ottieniBeniCulturaliVisitati(int pIdTurista)
			throws RemoteException;

	/**
	 * Metodo per l'estrazione della lista ai riferimenti ai Punti di Ristoro
	 * visitati da un Turista
	 * 
	 * @param pIdTurista Identificativo del Turista
	 * @return Lista di riferimenti ai Punti di Ristoro visitati
	 * @throws RemoteException Eccezione Remota
	 */
	public BeanVisitaPR[] ottieniPuntiDiRistoroVisitati(int pIdTurista)
			throws RemoteException;

	/**
	 * Metodo per l'inserimento di un Bene Culturale Visitato
	 * 
	 * @param pVisitaBC Pacchetto contenente tutte le informazioni relative alla
	 *            visita
	 * @return true se l'elemento è inserito con successo, false altrimenti
	 * @throws RemoteException Eccezione Remota
	 */
	public boolean inserisciBeneCulturaleVisitato(BeanVisitaBC pVisitaBC)
			throws RemoteException;

	/**
	 * Metodo per l'inserimento di un Punto di Ristoro Visitato
	 * 
	 * @param pVisitaPR Pacchetto contenente tutte le informazioni relative alla
	 *            visita
	 * @return true se l'elemento è inserito con successo, false altrimenti
	 * @throws RemoteException Eccezione Remota
	 */
	public boolean inserisciPuntoDiRistoroVisitato(BeanVisitaPR pVisitaPR)
			throws RemoteException;
	
	/** 
	 * Metodo per l'inserimento di un Bene Culturale nella lista dei Preferiti
	 *
	 * @param pIdTurista Identificativo del Turista
	 * @param pIdBeneCulturale Identificativo del Bene Culturale
	 * @return true se l'inserimento va a buon fine, false altrimenti
	 * @throws RemoteException Eccezione Remota
	 */
	public boolean inserisciBeneCulturalePreferito(int pIdTurista,
			int pIdBeneCulturale) throws RemoteException;
	
	/** 
	 * Metodo per l'inserimento di un Punto di Ristoro nella lista dei Preferiti
	 *
	 * @param pIdTurista Identificativo del Turista
	 * @param pIdPuntoDiRistoro Identificativo del Punto di Ristoro
	 * @return true se l'inserimento va a buon fine, false altrimenti
	 * @throws RemoteException Eccezione Remota
	 */
	public boolean inserisciPuntoDiRistoroPreferito(int pIdTurista,
			int pIdPuntoDiRistoro) throws RemoteException;
	
	/** 
	 * Metodo per la cancellazione di un Bene Culturale dalla lista dei Preferiti
	 *
	 * @param pIdTurista Identificativo del Turista
	 * @param pIdBeneCulturale Identificativo del Bene Culturale
	 * @return true se la cancellazione ha esito positivo, false altrimenti
	 * @throws RemoteException Eccezione Remota
	 */
	public boolean cancellaBeneCulturalePreferito(int pIdTurista,
			int pIdBeneCulturale) throws RemoteException;
	
	/** 
	 * Metodo per la cancellazione di un Punto di Ristoro dalla lista dei Preferiti
	 *
	 * @param pIdTurista Identificativo del Turista
	 * @param pIdPuntoDiRistoro Identificativo del Punto di Ristoro
	 * @return true se la cancellazione ha esito positivo, false altrimenti
	 * @throws RemoteException Eccezione Remota
	 */
	public boolean cancellaPuntoDiRistoroPreferito(int pIdTurista,
			int pIdPuntoDiRistoro) throws RemoteException;
	
	/** 
	 * Metodo per l'estrazione della lista dei Beni Culturali Preferiti
	 *
	 * @param pIdTurista Identificativo del Turista
	 * @return Lista di Beni Culturali Preferiti
	 * @throws RemoteException Eccezione Remota
	 */
	public BeanBeneCulturale[] ottieniBeniCulturaliPreferiti(int pIdTurista)
	throws RemoteException;
	
	/** 
	 * Metodo per l'estrazione della lista dei Punti di Ristoro
	 *
	 * @param pIdTurista Identificativo del Turista
	 * @return Lista di Punti di Ristoro Preferiti
	 * @throws RemoteException Eccezione Remota
	 */
	public BeanPuntoDiRistoro[] ottieniPuntiDiRistoroPreferiti(int pIdTurista)
	throws RemoteException;
	
	
}

