package unisa.gps.etour.control.GestioneUtentiRegistrati;

import java.rmi.RemoteException;
import java.util.ArrayList;

import unisa.gps.etour.bean.BeanTurista;
import unisa.gps.etour.bean.BeanVisitaBC;
import unisa.gps.etour.bean.BeanVisitaPR;

/**
 * Interfaccia per la gestione dei Turisti dal lato dell'Operatore Agenzia
 * 
 * @author Morelli Giuseppe
 * @version 0.1 � 2007 eTour Project - Copyright by SE@SA Lab DMI � University
 *          of Salerno
 */
public interface IGestioneTuristiAgenzia extends IGestioneTuristaComune
{

	/**
	 * Metodo per la cancellazione di un Turista dal DataBase
	 * 
	 * @param pIdTurista Identificativo del Turista da eliminare
	 * @return Booleano: true se l'operazione va a buon fine, false altrimenti
	 * @throws RemoteException Eccezione Remota
	 */
	public boolean cancellaTurista(int pIdTurista) throws RemoteException;

	/**
	 * Metodo per attivare un Turista registrato
	 * 
	 * @param pIdTurista Identificativo del Turista da attivare
	 * @return Booleano: true se l'operazione va a buon fine, false altrimenti
	 * @throws RemoteException Eccezione Remota
	 */
	public boolean attivaTurista(int pIdTurista) throws RemoteException;

	/**
	 * Metodo per disattivare un Turista attivato
	 * 
	 * @param pIdTurista Identificativo del Turista da disattivare
	 * @return Booleano: true se l'operazione va a buon fine, false altrimenti
	 * @throws RemoteException Eccezione Remota
	 */
	public boolean disattivaTurista(int pIdTurista) throws RemoteException;

	/**
	 * Metodo per ottenere una collezione di Turisti
	 * 
	 * @return ArrayList di BeanTurista
	 * @throws RemoteException Eccezione Remota
	 */
	public ArrayList<BeanTurista> ottieniTuristi() throws RemoteException;

	/**
	 * Metodo per ottenere una collezione di Turisti attivati o meno
	 * 
	 * @param statoAccount Booleano che permette di scegliere tra Turisti
	 *            attivati o meno
	 * @return ArrayList di BeanTurista
	 * @throws RemoteException Eccezione Remota
	 */
	public ArrayList<BeanTurista> ottieniTuristi(boolean statoAccount)
			throws RemoteException;

	/**
	 * Metodo per ottenere tutti i feedback rilasciati da un turista per i Punti
	 * di Ristoro
	 * 
	 * @param pIdTurista Identificativo per il Turista di cui prelevare i
	 *            feedback
	 * @return ArrayList contenente tutti i bean di feedback rilasciati
	 * @throws RemoteException Eccezione Remota
	 */
	public ArrayList<BeanVisitaPR> ottieniFeedbackPR(int pIdTurista)
			throws RemoteException;

	/**
	 * Metodo per ottenere tutti i feedback rilasciati da un turista per i Beni
	 * Culturali
	 * 
	 * @param pIdTurista Identificativo per il Turista di cui prelevare i
	 *            feedback
	 * @return ArrayList contenente tutti i bean di feedback rilasciati
	 * @throws RemoteException Eccezione Remota
	 */
	public ArrayList<BeanVisitaBC> ottieniFeedbackBC(int pIdTurista)
			throws RemoteException;

}
