package unisa.gps.etour.control.GestioneRicerche;

import java.rmi.Remote;
import java.rmi.RemoteException;

import unisa.gps.etour.bean.BeanBeneCulturale;
import unisa.gps.etour.bean.BeanPuntoDiRistoro;
import unisa.gps.etour.util.Punto3D;

/**
 * Interfaccia per la gestione delle ricerche
 * 
 * @author Penna Giuseppe
 * @version 0.1 &#169; 2007 eTour Project - Copyright by SE@SA Lab DMI -
 *          University of Salerno
 */
public interface IRicerca extends Remote
{
	
	// Costanti identificatrici delle tipologie di sito
	public final static byte BENE_CULTURALE = 0;
	public final static byte PUNTO_DI_RISTORO = 1;
	
	/**
	 * Metodo di inizializzazione per la ricerca
	 * 
	 * @param pIdTurista Identificativo del Turista. Passare -1 in caso di Utente Guest
	 * @param pParoleChiave Insieme delle parole chiave di ricerca
	 * @param pTagsId Insieme dei tag di ricerca
	 * @param pRaggioMax Distanza massima tra l'utente e il sito
	 * @param numeroElementiPerPagina Numero di elementi da ricercare in una
	 *            sessione di ricerca
	 * @param pPosizioneUtente Posizione dell'utente rilevata dal GPS
	 * @param pTipologiaSito Tipologia di sito da ricercare
	 * @return Numero di elementi emersi dalla ricerca. In caso di errore
	 *         ritorna -1
	 * @throws RemoteException Eccezione remota
	 */
	public int ricerca(int pIdTurista, String pParoleChiave, int[] pTagsId, double pRaggioMax,
			int pElementiPerPagina, Punto3D pPosizioneUtente,
			byte pTipologiaSito) throws RemoteException;

	/**
	 * Metodo per la restituzione della lista di Beni Culturali emersi dalla
	 * ricerca, relativi ad un determinato intervallo
	 * 
	 * @param pPagina Intervallo di elementi da includere nei risultati
	 * @return Insieme dei Beni Culturali relativi all'intervallo dei risultati
	 *         di ricerca selezionato. In caso di errore ritorna null
	 * @throws RemoteException Eccezione Remota
	 */
	public BeanBeneCulturale[] ottieniPaginaRisultatiBeneCulturale(int pPagina)
			throws RemoteException;

	/**
	 * Metodo per la restituzione della lista di Punti Di Ristoro emersi dalla
	 * ricerca, relativi ad un determinato intevallo
	 * 
	 * @param pPagina Intervallo di elementi da includere nei risultati
	 * @return Insieme dei Punti di Ristoro relativi all'intervallo dei
	 *         risultati di ricerca selezionato. In caso di errore ritorna null
	 * @throws RemoteException Eccezione Remota
	 */
	public BeanPuntoDiRistoro[] ottieniPaginaRisultatiPuntoDiRistoro(int pPagina)
			throws RemoteException;

	/**
	 * Metodo per la restituzione del numero degli elementi risultati dalla
	 * ricerca
	 * 
	 * @return Numero di elementi emersi in fase di ricerca. Nel caso in cui non
	 *         sia stata inizializzata la ricerca ritorna -1
	 */
	public int ottieniNumeroElementiRicerca() throws RemoteException;

	/**
	 * Metodo per la restituzione del numero di pagine risultate dalla ricerca
	 * 
	 * @return Numero di pagine emerse in fase di ricerca. Nel caso in cui non
	 *         sia stata inizializzata la ricerca ritorna -1
	 */
	public int ottieniNumeroPagineRicerca() throws RemoteException;

}
