package unisa.gps.etour.gui.operatoreagenzia;

import java.awt.*;
import java.awt.event.*;
import java.io.FileNotFoundException;
import java.rmi.RemoteException;
import java.rmi.registry.LocateRegistry;
import java.rmi.registry.Registry;
import java.util.ArrayList;
import java.util.Date;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.text.BadLocationException;
import javax.swing.text.Document;
import javax.swing.border.*;
import unisa.gps.etour.bean.BeanNews;
import unisa.gps.etour.control.GestioneAdvertisement.IGestioneAdvertisementAgenzia;
import unisa.gps.etour.gui.DeskManager;
import unisa.gps.etour.gui.HelpManager;
import unisa.gps.etour.gui.operatoreagenzia.document.LimitedDocument;
import unisa.gps.etour.gui.operatoreagenzia.tables.NewsTableModel;
import unisa.gps.etour.gui.operatoreagenzia.tables.PrioritaRenderer;
import unisa.gps.etour.gui.operatoreagenzia.tables.ScrollableTable;
import unisa.gps.etour.gui.operatoreagenzia.tables.TestoNewsRenderer;
import unisa.gps.etour.util.Data;

/**
 * Questa classe realizza l'interfaccia per la gestione delle news per l'attore
 * Operatore Agenzia.
 * 
 * @version 1.0
 * @author Mario Gallo 
 * 
 * &#169; 2007 eTour Project - Copyright by SE@SA Lab DMI -
 *         University of Salerno
 */
public class News extends JInternalFrame
{
	private JPanel jContentPane = null;
	private JToolBar NewsToolbar = null;
	private JButton btnEliminaN = null;
	private JPanel rightPanel = null;
	private JPanel formNews = null;
	private JSlider prSlider = null;
	private JButton btnInsertModify = null;
	private JButton btnReset = null;
	private JPanel panelHelp = null;
	private JTextPane textGuida = null;
	private JScrollPane scrollTableNews = null;
	private JTable tableNews = null;
	private JTextArea testoNews = null;
	private JComboBox durataNews = null;
	private JButton btnModificaN = null;
	private JLabel labelCaratteri;
	private int idNews = -1;
	private NewsTableModel tableModel;
	private HelpManager newsHelp;
	protected DeskManager desktopManager;
	protected JDesktopPane jDesktopPane;
	private IGestioneAdvertisementAgenzia gestioneNews;

	/**
	 * Questo &egrave; il costruttore di default.
	 */
	public News()
	{
		super("News");
		setPreferredSize(Home.CHILD_SIZE);
		setMinimumSize(new Dimension(600, 480));
		setResizable(true);
		setFrameIcon(new ImageIcon(getClass().getResource(
				Home.URL_IMAGES + "News32.png")));
		setIconifiable(true);
		setMaximizable(true);
		setClosable(true);

		// Setting up dell'help manager per la gestione delle news.
		textGuida = new JTextPane();
		try
		{
			newsHelp = new HelpManager(Home.URL_HELP + "News.txt", textGuida);
		}
		catch (FileNotFoundException e)
		{
			textGuida
					.setText("<html><b>Guida in linea non disponibile!</b></html>");
		}

		setContentPane(getJContentPane());

		addInternalFrameListener(new InternalFrameAdapter()
		{

			/*
			 * All'inserimento del frame nel desktop pane recupera desktop pane
			 * e desktop manager e inizializza gli oggetti remoti per la
			 * gestione dei beni culturali.
			 */

			public void internalFrameOpened(InternalFrameEvent pEvent)
			{
				JInternalFrame frame = pEvent.getInternalFrame();
				jDesktopPane = frame.getDesktopPane();
				desktopManager = (DeskManager) jDesktopPane.getDesktopManager();

				// Setting up degli oggetti remoti per la gestione dei beni
				// culturali.
				try
				{
					Registry reg = LocateRegistry.getRegistry(Home.HOST);
					gestioneNews = (IGestioneAdvertisementAgenzia) reg
							.lookup("GestioneAdvertisementAgenzia");

					// Caricamento dei dati.
					caricaTabella();
				}
				/*
				 * Due eccezioni: RemoteException e NotBoundException. Il
				 * risultato è lo stesso. La gestione non è operabile e la
				 * finestra dopo il messaggio di errore verrà chiusa.
				 */
				catch (Exception ex)
				{
					JLabel errore = new JLabel(
							"<html><h2>Impossibile comunicare con il server di eTour.</h2>"
									+ "<h3><u>La finestra di gestione richiesta verrà chiusa.</u></h3>"
									+ "<p><b>Possibili Cause:</b>"
									+ "<ul><li>Nessuna connessione alla rete.</li>"
									+ "<li>Server inattivo.</li>"
									+ "<li>Server intasato.</li></ul>"
									+ "<p>Si prega di riprovare in un secondo momento.</p>"
									+ "<p>Se l'errore dovesse persistere, si prega di contattare l'assistenza tecnica.</p>"
									+ "<p>Ci scusiamo per il disagio.</html>");
					ImageIcon err = new ImageIcon(getClass().getResource(
							Home.URL_IMAGES + "error48.png"));
					JOptionPane.showMessageDialog(jDesktopPane, errore,
							"Errore!", JOptionPane.ERROR_MESSAGE, err);
					frame.dispose();
				}
			}
		});
	}

	/**
	 * Inizializza il content pane del frame interno.
	 * 
	 * @return javax.swing.JPanel - il content pane.
	 */
	private JPanel getJContentPane()
	{
		if (jContentPane == null)
		{
			jContentPane = new JPanel();
			jContentPane.setLayout(new BorderLayout());
			jContentPane.add(getNewsToolbar(), BorderLayout.NORTH);
			jContentPane.add(getRightPanel(), BorderLayout.EAST);
			jContentPane.add(getTableNews(), BorderLayout.CENTER);
		}
		return jContentPane;
	}

	/**
	 * Questo metodo inizializza la toolbar per la gestione delle news.
	 * 
	 * @return javax.swing.JToolBar - la toolbar di gestione news.
	 */
	private JToolBar getNewsToolbar()
	{
		if (NewsToolbar == null)
		{
			NewsToolbar = new JToolBar();
			NewsToolbar.setFloatable(false);
			NewsToolbar.add(getBtnModificaN());
			NewsToolbar.addSeparator();
			NewsToolbar.add(getBtnEliminaN());
		}
		return NewsToolbar;
	}

	/**
	 * Questo metodo inizializza il bottone di modifica di una news.
	 * 
	 * @return javax.swing.JButton - il bottone di modifica.
	 */
	private JButton getBtnModificaN()
	{
		if (btnModificaN == null)
		{
			btnModificaN = new JButton();
			btnModificaN.setText("Modifica News");
			btnModificaN.setIcon(new ImageIcon(getClass().getResource(
					Home.URL_IMAGES + "ModificaNews32.png")));
			btnModificaN.setCursor(Cursor
					.getPredefinedCursor(Cursor.HAND_CURSOR));
			btnModificaN.setName("btnModifica");
			btnModificaN.addMouseListener(newsHelp);
			btnModificaN.setEnabled(false);
			btnModificaN.addActionListener(new ActionListener()
			{
				public void actionPerformed(ActionEvent pActionEvent)
				{
					int selectedRow = tableNews.getSelectedRow();
					if (idNews == -1) // In questo modo so già se stava
										// modificando
					{
						btnInsertModify.setText("Modifica");
						btnInsertModify.setIcon(new ImageIcon(getClass()
								.getResource(Home.URL_IMAGES + "Salva16.png")));
						btnReset.setText("Annulla");
						btnReset.setIcon(new ImageIcon(getClass().getResource(
								Home.URL_IMAGES + "Annulla16.png")));
						formNews.setBorder(BorderFactory.createTitledBorder(
								BorderFactory.createLineBorder(new Color(51,
										102, 255), 3), "Modifica News",
								TitledBorder.DEFAULT_JUSTIFICATION,
								TitledBorder.DEFAULT_POSITION, new Font(
										"Dialog", Font.BOLD, 12), new Color(0,
										102, 204)));
					}
					String testo = (String) tableModel.getValueAt(selectedRow,
							0);
					int numCaratteri = 200 - testo.length() + 1;
					labelCaratteri.setText("# Caratteri:" + numCaratteri);
					Document docTesto = testoNews.getDocument();
					try
					{
						docTesto.remove(0, docTesto.getLength());
						docTesto.insertString(0, testo, null);
					}
					catch (BadLocationException e)
					{
						e.printStackTrace();
					}
					Date scadenza = (Date) tableModel
							.getValueAt(selectedRow, 3);
					durataNews.setSelectedIndex(Data.getNumDays(scadenza));
					idNews = tableModel.getID(selectedRow);
					prSlider.setValue((Integer) tableModel.getValueAt(
							selectedRow, 1));
				}

			});
		}
		return btnModificaN;
	}

	/**
	 * Questo metodo inizializza il bottone di Eliminazione di una news.
	 * 
	 * @return javax.swing.JButton - il bottone per l'eliminazione.
	 */
	private JButton getBtnEliminaN()
	{
		if (btnEliminaN == null)
		{
			btnEliminaN = new JButton();
			btnEliminaN.setText("Elimina News");
			btnEliminaN.setIcon(new ImageIcon(getClass().getResource(
					Home.URL_IMAGES + "EliminaNews32.png")));
			btnEliminaN.setEnabled(false);
			btnEliminaN.setCursor(Cursor
					.getPredefinedCursor(Cursor.HAND_CURSOR));
			btnEliminaN.setName("btnElimina");
			btnEliminaN.addMouseListener(newsHelp);
			btnEliminaN.addActionListener(new ActionListener()
			{
				public void actionPerformed(ActionEvent pEvent)
				{
					int selectedRow = tableNews.getSelectedRow();

					// Costruzione della dialog per la conferma
					// dell'eliminazione
					JPanel root = new JPanel(new BorderLayout());
					JLabel message = new JLabel(
							"Sei sicuro di voler eliminare la news selezionata?");
					message.setFont(new Font("Dialog", Font.BOLD, 14));
					JLabel avviso = new JLabel(
							"I dati eliminati non potranno essere più recuperati.",
							SwingConstants.CENTER);
					avviso.setIcon(new ImageIcon(getClass().getResource(
							Home.URL_IMAGES + "warning16.png")));
					root.add(message, BorderLayout.NORTH);
					root.add(avviso, BorderLayout.SOUTH);
					String[] options = { "Elimina", "Annulla" };

					// La dialog viene visualizzata a schermo
					int scelta = JOptionPane.showInternalOptionDialog(
							jContentPane, root, "Conferma Eliminazione News",
							JOptionPane.YES_NO_OPTION,
							JOptionPane.QUESTION_MESSAGE, new ImageIcon(
									getClass().getResource(
											Home.URL_IMAGES
													+ "EliminaNews48.png")),
							options, options[1]);

					// Se si è scelto di confermare l'eliminazione
					if (scelta == JOptionPane.YES_OPTION)
					{
						try
						{
							gestioneNews.cancellaNews(tableModel
									.getID(selectedRow));
							tableModel.removeNews(selectedRow);
							JOptionPane
									.showInternalMessageDialog(
											jContentPane,
											"La news selezionata è stata eliminata correttamente",
											"News Eliminata!",
											JOptionPane.OK_OPTION,
											new ImageIcon(
													getClass()
															.getResource(
																	Home.URL_IMAGES
																			+ "ok48.png")));
							azzeraForm();
						}
						catch (Exception ex)
						{
							JLabel errore = new JLabel(
									"<html><h2>Impossibile comunicare con il server di eTour.</h2>"
											+ "<h3><u>L'operazione di eliminazione richiesta non può essere completata.</u></h3>"
											+ "<p>Si prega di riprovare in un secondo momento.</p>"
											+ "<p>Se l'errore dovesse persistere, si prega di contattare l'assistenza tecnica.</p>"
											+ "<p>Ci scusiamo per il disagio.</html>");
							ImageIcon err = new ImageIcon(getClass()
									.getResource(
											Home.URL_IMAGES + "error48.png"));
							JOptionPane.showMessageDialog(jDesktopPane, errore,
									"Errore!", JOptionPane.ERROR_MESSAGE, err);
						}
					}
				}
			});
		}
		return btnEliminaN;
	}

	/**
	 * Questo metodo inizializza il pannello che realizza l'interfaccia sul lato
	 * destro della gestione delle news.
	 * 
	 * @return javax.swing.JPanel - il pannello destro.
	 */
	private JPanel getRightPanel()
	{
		if (rightPanel == null)
		{
			rightPanel = new JPanel();
			rightPanel.setLayout(new GridBagLayout());
			GridBagConstraints g = new GridBagConstraints();
			g.fill = GridBagConstraints.BOTH;
			g.gridx = 0;
			g.gridy = 0;
			g.weighty = 0.7;
			rightPanel.add(getFormNews(), g);
			g.weighty = 0.3;
			g.gridy = 1;
			rightPanel.add(getPanelHelp(), g);

		}
		return rightPanel;
	}

	/**
	 * Questo metodo inizializza il form per l'inserimento e la modifica di una
	 * news.
	 * 
	 * @return javax.swing.JPanel - il form
	 */
	private JPanel getFormNews()
	{
		if (formNews == null)
		{
			formNews = new JPanel(new GridBagLayout());
			formNews.setBorder(BorderFactory.createTitledBorder(BorderFactory
					.createLineBorder(new Color(51, 102, 255), 3),
					"Nuova News", TitledBorder.DEFAULT_JUSTIFICATION,
					TitledBorder.DEFAULT_POSITION, new Font("Dialog",
							Font.BOLD, 12), new Color(0, 102, 204)));
			GridBagConstraints g = new GridBagConstraints();
			g.anchor = GridBagConstraints.CENTER;
			g.gridx = 0;
			g.gridy = 0;
			g.weighty = 0.1;
			g.gridwidth = 1;
			g.gridheight = 1;
			g.insets = new Insets(5, 5, 5, 5);
			formNews.add(new JLabel("Testo della news:"), g);
			labelCaratteri = new JLabel("# Caratteri:200");
			g.gridx = 1;
			formNews.add(labelCaratteri, g);
			g.gridx = 0;
			g.gridwidth = 2;
			g.gridy = 1;
			g.weighty = 0.3;
			g.fill = GridBagConstraints.VERTICAL;
			JScrollPane scrollTesto = new JScrollPane(getTestoNews());
			scrollTesto
					.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
			formNews.add(scrollTesto, g);
			g.weighty = 0.1;
			g.gridy = 2;
			g.fill = GridBagConstraints.NONE;
			formNews.add(new JLabel("Priorità:"), g);
			g.gridy = 3;
			formNews.add(getPrSlider(), g);
			JPanel nuovo = new JPanel(new FlowLayout());
			nuovo.add(new JLabel("Durata della news:"));
			durataNews = new JComboBox();
			for (int i = 0; i <= 60; i++)
			{
				durataNews.addItem("" + i);
			}
			durataNews.setSelectedIndex(0);
			durataNews.setEditable(false);
			durataNews.setName("durata");
			durataNews.addMouseListener(newsHelp);
			nuovo.add(durataNews);
			nuovo.add(new JLabel("giorni"));
			g.gridy = 4;
			g.fill = GridBagConstraints.HORIZONTAL;
			formNews.add(nuovo, g);
			g.gridwidth = 1;
			g.gridy = 5;
			formNews.add(getBtnInsertModify(), g);
			g.gridx = 1;
			formNews.add(getBtnReset(), g);
		}
		return formNews;
	}

	/**
	 * Questo metodo inizializza lo slider per settare la priorit&agrave; di una
	 * news.
	 * 
	 * @return javax.swing.JSlider - lo slider con ticks da 1 a 5.
	 */
	private JSlider getPrSlider()
	{
		if (prSlider == null)
		{
			prSlider = new JSlider(JSlider.HORIZONTAL, 5, 1);
			prSlider.setMinimum(1);
			prSlider.setMaximum(5);
			prSlider.setMajorTickSpacing(1);
			prSlider.setLabelTable(prSlider.createStandardLabels(1));
			prSlider.setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
			prSlider.setName("priorità");
			prSlider.addMouseListener(newsHelp);
			prSlider.setPaintLabels(true);
			prSlider.setPaintTicks(true);
			prSlider.setPaintTicks(true);
			prSlider.setSnapToTicks(true);
		}
		return prSlider;
	}

	/**
	 * Questo metodo inizializza il pulsante di sottomissione della form di
	 * inserimento/modifica.
	 * 
	 * @return javax.swing.JButton - il pulsante di sottomissione della form.
	 */
	private JButton getBtnInsertModify()
	{
		if (btnInsertModify == null)
		{
			btnInsertModify = new JButton();
			btnInsertModify.setText("Inserisci");
			btnInsertModify.setIcon(new ImageIcon(getClass().getResource(
					Home.URL_IMAGES + "nuovaNews16.png")));
			btnInsertModify.setCursor(Cursor
					.getPredefinedCursor(Cursor.HAND_CURSOR));
			btnInsertModify.setName("btnInserisci");
			btnInsertModify.addMouseListener(newsHelp);
			btnInsertModify.setFont(new Font("Dialog", Font.BOLD, 12));
			btnInsertModify.setHorizontalTextPosition(SwingConstants.TRAILING);
			btnInsertModify.addActionListener(new ActionListener()
			{

				public void actionPerformed(ActionEvent pEvent)
				{
					if (testoNews.getText().length() == 0)
					{
						JOptionPane.showInternalMessageDialog(jContentPane,
								"Il testo di una news non pu� essere vuoto!",
								"Errore Nuova News", JOptionPane.ERROR_MESSAGE,
								new ImageIcon(getClass().getResource(
										Home.URL_IMAGES + "error32.png")));
						return;
					}
					if (idNews != -1) // Si sta effettuando una modifica
					{
						// Costruzione della dialog per la conferma della
						// modifica
						JPanel root = new JPanel(new BorderLayout());
						JLabel message = new JLabel(
								"Modificare la news selezionata con"
										+ " i nuovi dati?");
						message.setFont(new Font("Dialog", Font.BOLD, 14));
						JLabel avviso = new JLabel(
								"I dati precedenti non potranno essere pi� recuperati.",
								SwingConstants.CENTER);
						avviso.setIcon(new ImageIcon(getClass().getResource(
								Home.URL_IMAGES + "warning16.png")));
						root.add(message, BorderLayout.NORTH);
						root.add(avviso, BorderLayout.SOUTH);
						String[] options = { "Modifica", "Annulla" };
						// La dialog viene visualizzata a schermo
						int scelta = JOptionPane
								.showInternalOptionDialog(
										jContentPane,
										root,
										"Conferma Modifica News",
										JOptionPane.YES_NO_OPTION,
										JOptionPane.QUESTION_MESSAGE,
										new ImageIcon(getClass().getResource(
												Home.URL_IMAGES
														+ "ModificaNews48.png")),
										options, options[1]);
						// Se si è scelto di confermare la modifica
						if (scelta == JOptionPane.YES_OPTION)
						{
							try
							{
								Date scadenza = new Date();
								scadenza.setDate(scadenza.getDate()
										+ durataNews.getSelectedIndex());
								BeanNews nuovo = new BeanNews(testoNews
										.getText(), new Date(), scadenza,
										prSlider.getValue(), idNews);
								gestioneNews.modificaNews(nuovo);
								tableModel.updateNews(nuovo);
								JOptionPane
										.showInternalMessageDialog(
												jContentPane,
												"La news selezionata è stata modificata correttamente.",
												"News Modificata!",
												JOptionPane.OK_OPTION,
												new ImageIcon(
														getClass()
																.getResource(
																		Home.URL_IMAGES
																				+ "ok48.png")));
							}
							catch (Exception ex)
							{
								JLabel errore = new JLabel(
										"<html><h2>Impossibile comunicare con il server di eTour.</h2>"
												+ "<h3><u>L'operazione di modifica richiesta non può essere completata.</u></h3>"
												+ "<p>Si prega di riprovare in un secondo momento.</p>"
												+ "<p>Se l'errore dovesse persistere, si prega di contattare l'assistenza tecnica.</p>"
												+ "<p>Ci scusiamo per il disagio.</html>");
								ImageIcon err = new ImageIcon(
										getClass()
												.getResource(
														Home.URL_IMAGES
																+ "error48.png"));
								JOptionPane.showMessageDialog(jDesktopPane,
										errore, "Errore!",
										JOptionPane.ERROR_MESSAGE, err);
							}
						}
					}
					else
					// Si sta effettuando un inserimento
					{
						Date scadenza = new Date();
						scadenza.setDate(scadenza.getDate()
								+ durataNews.getSelectedIndex());
						BeanNews nuovo = new BeanNews(testoNews.getText(),
								new Date(), scadenza, prSlider.getValue(), 33);
						try
						{
							boolean ok = gestioneNews.inserisciNews(nuovo);
							if (ok)
							{
								caricaTabella();
								tableModel.insertNews(nuovo);
								JOptionPane
										.showInternalMessageDialog(
												jContentPane,
												"La news selezionata è stata inserita correttamente nel sistema.",
												"Nuova News inserita!",
												JOptionPane.OK_OPTION,
												new ImageIcon(
														getClass()
																.getResource(
																		Home.URL_IMAGES
																				+ "nuovaNews48.png")));
							}
						}
						catch (RemoteException e)
						{
							JLabel errore = new JLabel(
									"<html><h2>Impossibile comunicare con il server di eTour.</h2>"
											+ "<h3><u>L'operazione di inserimento richiesta non può essere completata.</u></h3>"
											+ "<p>Si prega di riprovare in un secondo momento.</p>"
											+ "<p>Se l'errore dovesse persistere, si prega di contattare l'assistenza tecnica.</p>"
											+ "<p>Ci scusiamo per il disagio.</html>");
							ImageIcon err = new ImageIcon(getClass()
									.getResource(
											Home.URL_IMAGES + "error48.png"));
							JOptionPane.showMessageDialog(jDesktopPane, errore,
									"Errore!", JOptionPane.ERROR_MESSAGE, err);
						}
					}
					tableModel.fireTableDataChanged();
					azzeraForm();
				}
			});

		}
		return btnInsertModify;
	}

	/**
	 * Questo metodo inizializza il pulsante per l'azzeramento della form o per
	 * annullare l'azione di modifica su una news selezionata.
	 * 
	 * @return javax.swing.JButton - il pulsante sopra descritto.
	 */
	private JButton getBtnReset()
	{
		if (btnReset == null)
		{
			btnReset = new JButton();
			btnReset.setText("Azzera");
			btnReset.setHorizontalTextPosition(SwingConstants.LEADING);
			btnReset.setPreferredSize(new Dimension(103, 26));
			btnReset.setIcon(new ImageIcon(getClass().getResource(
					Home.URL_IMAGES + "azzera16.png")));
			btnReset.setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
			btnReset.setName("btnAzzera");
			btnReset.addMouseListener(newsHelp);
			btnReset.addActionListener(new ActionListener()
			{
				public void actionPerformed(ActionEvent arg0)
				{
					azzeraForm();
				}
			});
		}
		return btnReset;
	}

	/**
	 * Questo metodo inizializza il pannello della guida in linea.
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getPanelHelp()
	{
		if (panelHelp == null)
		{
			panelHelp = new JPanel();
			panelHelp.setLayout(new BorderLayout());
			panelHelp.setBorder(BorderFactory.createTitledBorder(BorderFactory
					.createLineBorder(new Color(51, 102, 255), 3),
					"Guida in Linea", TitledBorder.DEFAULT_JUSTIFICATION,
					TitledBorder.DEFAULT_POSITION, new Font("Dialog",
							Font.BOLD, 12), new Color(0, 102, 204)));
			textGuida.setEditable(false);
			textGuida.setContentType("text/html");
			textGuida.setOpaque(false);
			textGuida
					.setText("<html>Muovi il puntatore del mouse su un controllo "
							+ "d'interesse per visualizzare la guida sensibile al contesto.</html>");
			textGuida.setName("textGuida");
			textGuida.addMouseListener(newsHelp);
			panelHelp.setPreferredSize(new Dimension(200, 100));
			panelHelp.add(textGuida, BorderLayout.CENTER);
		}
		return panelHelp;
	}

	/**
	 * Questo metodo inizializza la tabella contenente tutte le news presenti
	 * nel sistema.
	 * 
	 * @return javax.swing.JTable
	 */
	private JScrollPane getTableNews()
	{
		if (tableNews == null)
		{
			tableModel = new NewsTableModel();
			tableNews = new ScrollableTable(tableModel);
			tableNews.setRowHeight(64);
			tableNews.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
			tableNews.setSelectionBackground(new Color(0xe6, 0xe6, 0xfa));
			tableNews.setColumnSelectionAllowed(false);
			ListSelectionModel selectionModel = tableNews.getSelectionModel();
			selectionModel.addListSelectionListener(new ListSelectionListener()
			{
				public void valueChanged(ListSelectionEvent event)
				{
					int selectedRow = tableNews.getSelectedRow();
					btnModificaN.setEnabled((selectedRow != -1) ? true : false);
					btnEliminaN.setEnabled((selectedRow != -1) ? true : false);
				}
			});
			tableNews.addKeyListener(new KeyAdapter()
			{
				public void keyPressed(KeyEvent pKey)
				{
					int keyCode = pKey.getKeyCode();
					if (keyCode == KeyEvent.VK_ENTER)
					{
						btnModificaN.doClick();
					}
					else if ((keyCode == KeyEvent.VK_CANCEL)
							|| (keyCode == KeyEvent.VK_BACK_SPACE))
					{
						btnEliminaN.doClick();
					}
				}
			});

			scrollTableNews = new JScrollPane();
			scrollTableNews.setViewportView(tableNews);
			scrollTableNews
					.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
		}
		return scrollTableNews;
	}

	/**
	 * Questo metodo inizializza la text area che contiene il testo di una news.
	 * 
	 * @return javax.swing.JTextArea - la text area.
	 */
	private JTextArea getTestoNews()
	{
		if (testoNews == null)
		{
			testoNews = new JTextArea();
			testoNews.setWrapStyleWord(true);
			testoNews.setLineWrap(true);
			testoNews.setBorder(BorderFactory.createLoweredBevelBorder());
			testoNews.setColumns(18);
			testoNews.setLineWrap(true);
			testoNews.setRows(4);
			testoNews.setDocument(new LimitedDocument(200));
			testoNews.setName("testo");
			testoNews.addMouseListener(newsHelp);
			testoNews.addKeyListener(new KeyAdapter()
			{
				public void keyTyped(KeyEvent pKeyEvent)
				{
					char keyChar = pKeyEvent.getKeyChar();
					if (Character.isDigit(keyChar)
							|| Character.isLetter(keyChar)
							|| Character.isWhitespace(keyChar))
					{
						int len = testoNews.getText().length();
						if (len != 200)
						{
							labelCaratteri
									.setText("# Caratteri:"
											+ (200 - (testoNews.getText()
													.length() + 1)));
						}
					}
				}

				public void keyPressed(KeyEvent pKeyEvent)
				{
					int keyCode = pKeyEvent.getKeyCode();
					if (keyCode == KeyEvent.VK_CANCEL
							|| keyCode == KeyEvent.VK_BACK_SPACE)
					{
						int len = testoNews.getText().length();
						if (len != 0)
						{
							labelCaratteri.setText("# Caratteri:"
									+ (200 - len + 1));
						}
					}
				}
			});

		}
		return testoNews;
	}

	/**
	 * Questo metodo azzera i campi della form.
	 */
	private void azzeraForm()
	{
		btnInsertModify.setText("Inserisci");
		btnReset.setText("Azzera");
		btnReset.setIcon(new ImageIcon(getClass().getResource(
				Home.URL_IMAGES + "azzera16.png")));
		formNews.setBorder(BorderFactory.createTitledBorder(BorderFactory
				.createLineBorder(new Color(51, 102, 255), 3), "Nuova News",
				TitledBorder.DEFAULT_JUSTIFICATION,
				TitledBorder.DEFAULT_POSITION,
				new Font("Dialog", Font.BOLD, 12), new Color(0, 102, 204)));
		durataNews.setSelectedIndex(0);
		testoNews.setText("");
		tableNews.clearSelection();
		prSlider.setValue(1);
		labelCaratteri.setText("# Caratteri:200");
		idNews = -1;
	}

	/**
	 * Questo metodo importa le news scaricate dal server nella tabella.
	 */
	private void caricaTabella()
	{
		ArrayList<BeanNews> news = null;
		try
		{
			news = gestioneNews.ottieniAllNews();
		}
		// In caso di errore visualizza un messaggio di errore.
		catch (RemoteException e)
		{
			JLabel errore = new JLabel(
					"<html><h2>Impossibile comunicare con il server di eTour.</h2>"
							+ "<h3><u>La lista delle News non è stata caricata.</u></h3>"
							+ "<p>Si prega di riprovare in un secondo momento.</p>"
							+ "<p>Se l'errore dovesse persistere, si prega di contattare l'assistenza tecnica.</p>"
							+ "<p>Ci scusiamo per il disagio.</html>");
			ImageIcon err = new ImageIcon(getClass().getResource(
					Home.URL_IMAGES + "error48.png"));
			JOptionPane.showInternalMessageDialog(this, errore, "Errore!",
					JOptionPane.ERROR_MESSAGE, err);
		}
		finally
		{
			tableModel = new NewsTableModel(news);
			tableNews.setModel(tableModel);
			// Testo della news
			tableNews.getColumnModel().getColumn(0).setPreferredWidth(320);
			tableNews.getColumnModel().getColumn(0).setCellRenderer(
					new TestoNewsRenderer());
			// Priorità
			tableNews.getColumnModel().getColumn(1).setPreferredWidth(100);
			tableNews.getColumnModel().getColumn(1).setCellRenderer(
					new PrioritaRenderer());
			// Data di inserimento
			tableNews.getColumnModel().getColumn(2).setPreferredWidth(80);
			// Data di scadenza
			tableNews.getColumnModel().getColumn(3).setPreferredWidth(80);
		}

	}
}
