/**
 * Classe di test per la NewsTableModel
 *
 * @author Mario Gallo
 * @version 0.1
 * 
 * &#169; 2007 eTour Project - Copyright by SE@SA Lab DMI - University of Salerno
 */
package unisa.gps.etour.gui.operatoreagenzia.tables.test;

import java.util.ArrayList;
import java.util.Date;

import unisa.gps.etour.bean.BeanNews;
import unisa.gps.etour.gui.operatoreagenzia.tables.NewsTableModel;
import junit.framework.TestCase;

public class NewsTableModelTest extends TestCase
{

	private NewsTableModel tableModel;
	private BeanNews aNews;
	private BeanNews aNewsModificata;
	
	public NewsTableModelTest(String pName)
	{
		super(pName);
		aNews = new BeanNews("Una news di esempio",new Date(),new Date(),2,1);
		aNewsModificata = new BeanNews("Una news modificata di esempio",new Date(),new Date(),3,1);
	}

	protected void setUp() throws Exception
	{
		super.setUp();
		tableModel = new NewsTableModel();
	}

	/*
	 * Verifica il comportamento del costruttore con un arraylist di BeanNews.
	 */
	public void testCostruttoreConArrayList()
	{
		ArrayList<BeanNews> test = new ArrayList<BeanNews>();
		for (int i = 0; i < 10; i++)
		{
			test.add(new BeanNews("un testo" + i, new Date(),new Date(),5,i));
		}
		tableModel = new NewsTableModel(test);
		for (int i = 0; i < 10; i++)
		{
			assertSame(test.get(i).getId(), tableModel.getID(i));
		}
		
	}

	/*
	 * Verifica il comporamento del costruttore con un arraylist nullo.
	 */
	public void testCostruttoreConArrayListNull()
	{
		tableModel = new NewsTableModel(null);
	}
	
	/*
	 * Verifica il comportamento del costruttore con un arraylist vuoto.
	 */
	public void testCostruttoreConArrayListVuoto()
	{
		tableModel = new NewsTableModel(new ArrayList<BeanNews>());	
	}
	
	/*
	 * Verifica il comportamento del metodo con dei parametri corretti.
	 */
	public void testGetValueAtParametriCorretti()
	{
		// Inserisce nel model due bean di test.
		tableModel.insertNews(aNews);
		tableModel.insertNews(aNewsModificata);
		
		//Verifica i dati inseriti.
		assertSame(aNews.getNews(), tableModel.getValueAt(0, 0));
		assertSame(aNews.getPriorita(), tableModel.getValueAt(0, 1));
		assertSame(aNewsModificata.getNews(), tableModel.getValueAt(1, 0));
		assertSame(aNewsModificata.getPriorita(), tableModel.getValueAt(1, 1));	
	}
	
	/*
	 * Verifica il comporamento del metodo con un indice di riga sballato.
	 */
	public void testGetValueAtRigaSballata()
	{
		try
		{
			tableModel.getValueAt(12, 0);
			fail("Dovrebbe essere sollevata un'eccezione");
		}
		catch (IllegalArgumentException success)
		{
		}
	}

	/*
	 * Verifica il comporamento del metodo con un indice di colonna sballato.
	 */
	public void testGetValueAtColonnaSballata()
	{
		try
		{
			tableModel.getValueAt(0, -121334);
			fail("Dovrebbe essere sollevata un'eccezione");
		}
		catch (IllegalArgumentException success)
		{
		}
	}

	/*
	 * Verifica il comporamento del metodo con parametro corretto.
	 */
	public void testInsertNewsParametroCorretto()
	{
		tableModel.insertNews(aNews);
		assertSame(aNews.getId(),tableModel.getID(0));		
	}
	
	/*
	 * Verifica il comporamento del metodo con parametro a null
	 */
	public void testInsertNewsParametroNull()
	{
		try
		{
			tableModel.insertNews(null);
			fail("Dovrebbe essere sollevata un'eccezione");
		}
		catch (IllegalArgumentException success)
		{
		}
	}

	/*
	 * Verifica il comporamento del metodo con parametro corretto.
	 */
	public void testUpdateNewsParametroCorretto()
	{
		tableModel.insertNews(aNews);
		tableModel.updateNews(aNewsModificata);
		assertSame(aNewsModificata.getNews(),tableModel.getValueAt(0, 0));
		assertSame(aNewsModificata.getPriorita(),tableModel.getValueAt(0, 1));
		assertSame(aNewsModificata.getId(),tableModel.getID(0));
	}
	
	/*
	 * Verifica il comporamento del metodo con parametro a null
	 */
	public void testUpdateNewsParametroNull()
	{
		
		try
		{
			tableModel.updateNews(null);
			fail("Dovrebbe essere sollevata un'eccezione");
		}
		catch (IllegalArgumentException success)
		{
		}
	}

	/*
	 * Verifica il comporamento del metodo con parametro corretto.
	 */	
	public void testRemoveNewsParametroCorretto()
	{
		tableModel.insertNews(aNews);
		assertSame(aNews.getId(), tableModel.removeNews(0));
	}
	
	/*
	 * Verifica il comporamento del metodo con indice di riga sballato.
	 */
	public void testRemoveNewsRigaSballata()
	{
		try
		{
			tableModel.removeNews(-1231);
			fail("Dovrebbe essere sollevata un'eccezione");
		}
		catch (IllegalArgumentException success){}
	}

}
