package unisa.gps.etour.util;

/**
 * Bean che contiene le coordinate di un punto sulla superfice della terrae e
 * che ne realizza il calcolo della distanza dal sistema. I valori delle
 * coordinate devono essere rappresentati in radianti e devono rientrare in un
 * range prefissato: da 0 a -Pi greco / 4 per la latitudine a Sud dell'equatore
 * da 0 a +Pi greco / 4 per la latitudine a Nord dell'equatore da 0 a -Pi greco /
 * 2 per la longitudine a Ovest del meridiano di Greenwitch da 0 a +Pi greco / 2
 * per la longitudine a Est del meridiano di Greenwitch
 * 
 * @author Miranda Mauro
 * @version 0.1 � 2007 eTour Project - Copyright by SE@SA Lab DMI � University
 *          of Salerno
 */

public class Punto3D
{
	// Raggio della terra
	final double EARTH_RADIUS = 6371.0;
	private double latitudine, longitudine, altitudine;

	public Punto3D()
	{
		latitudine = longitudine = altitudine = 0;
	}

	public Punto3D(double pLatitudine, double pLongitudine, double pAltitudine)
	{
		latitudine = pLatitudine;
		longitudine = pLongitudine;
		altitudine = pAltitudine;
	}

	/**
	 * Restituisce la latitudine
	 * 
	 * @return
	 */
	public double getLatitudine()
	{
		return latitudine;
	}

	/**
	 * Imposta la latitudine
	 * 
	 * @param pLatitudine
	 */
	public void setLatitudine(double pLatitudine)
	{
		this.latitudine = pLatitudine;
	}

	/**
	 * Restituisce la longitudine
	 * 
	 * @return
	 */
	public double getLongitudine()
	{
		return longitudine;
	}

	/**
	 * Imposta la longitudine
	 * 
	 * @param pLongitudine
	 */
	public void setLongitudine(double pLongitudine)
	{
		this.longitudine = pLongitudine;
	}

	/**
	 * Restituisce l'altitudine
	 * 
	 * @return
	 */
	public double getAltitudine()
	{
		return altitudine;
	}

	/**
	 * Imposta l'altitudine
	 * 
	 * @param pAltitudine
	 */
	public void setAltitudine(double pAltitudine)
	{
		this.altitudine = pAltitudine;
	}

	/**
	 * Calcola la distanza tra il punto e un altro punto dato come argomento
	 * 
	 * @param p
	 * @return
	 */
	public double distanza(Punto3D p)
	{
		double differenzaLongitudine = this.longitudine - p.longitudine;
		double p1 = Math.pow(Math.cos(p.latitudine)
				* Math.sin(differenzaLongitudine), 2);
		double p2 = Math.pow(Math.cos(latitudine) * Math.sin(p.latitudine)
				- Math.sin(latitudine) * Math.cos(p.latitudine)
				* Math.cos(differenzaLongitudine), 2);
		double p3 = Math.sin(latitudine) * Math.sin(p.latitudine)
				+ Math.cos(latitudine) * Math.cos(p.latitudine)
				* Math.cos(differenzaLongitudine);
		return (Math.atan(Math.sqrt(p1 + p2) / p3) * EARTH_RADIUS);
	}

	/**
	 * Metodo che crea un punto 3d a partire da coordinate misurate in gradi. Il
	 * punto 3d invece rappresenta le coordinate in radianti
	 * 
	 * @param pLatitudine Latitudine in gradi
	 * @param pLongitudine Longitudine in gradi *
	 * @param pAltitudine
	 * @return Punto3D con le coordinate in radianti
	 */
	public static Punto3D gradiRadianti(double pLatitudine,
			double pLongitudine, double pAltitudine)
	{
		Punto3D punto = new Punto3D();
		punto.setLatitudine(pLatitudine * Math.PI / 180);
		punto.setLongitudine(pLongitudine * Math.PI / 180);
		punto.setAltitudine(pAltitudine);
		return punto;
	}
}
