﻿package unisa.gps.etour.control.GestioneRicerche;

import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Hashtable;

import unisa.gps.etour.bean.BeanBeneCulturale;
import unisa.gps.etour.bean.BeanPuntoDiRistoro;
import unisa.gps.etour.bean.BeanTag;
import unisa.gps.etour.repository.DBBeneCulturale;
import unisa.gps.etour.repository.DBPuntoDiRistoro;
import unisa.gps.etour.repository.DBTag;
import unisa.gps.etour.repository.IDBBeneCulturale;
import unisa.gps.etour.repository.IDBPuntoDiRistoro;
import unisa.gps.etour.repository.IDBTag;
import unisa.gps.etour.util.ControlloDati;
import unisa.gps.etour.util.MessaggiErrore;
import unisa.gps.etour.util.Punto3D;

public class Ricerca extends UnicastRemoteObject implements IRicerca
{

	private static final long serialVersionUID = -6009809097302884655L;

	public Ricerca() throws RemoteException
	{
		super();
	}

	// Parametri di ricerca
	private int pIdTurista = -1;
	private String pParoleChiave;
	private double pRaggioMax;
	private Punto3D pPosizioneUtente;
	private int pElementiPerPagina = -1;
	private byte pTipologiaSito = -1;

	// Oggetti per la connessione al database
	private IDBBeneCulturale BeneCulturale = new DBBeneCulturale();
	private IDBPuntoDiRistoro PuntoDiRistoro = new DBPuntoDiRistoro();
	private IDBTag Tag = new DBTag();

	// Lista dei Tag ottenuti
	private ArrayList<BeanTag> pTags;

	// Risultati di ricerca partizionati per pagine
	private Hashtable<Integer, ArrayList<BeanBeneCulturale>> risultatiRicercaBeniCulturali = new Hashtable<Integer, ArrayList<BeanBeneCulturale>>();
	private Hashtable<Integer, ArrayList<BeanPuntoDiRistoro>> risultatiRicercaPuntiDiRistoro = new Hashtable<Integer, ArrayList<BeanPuntoDiRistoro>>();

	// Quantità di Elementi e di Pagine risultati in fase di ricerca
	private int numeroPagineRisultati = -1;
	private int numeroElementiRicerca = -1;

	/* (non-Javadoc)
	 * @see unisa.gps.etour.control.GestioneRicerche.IRicerca#ricerca(int, java.lang.String, int[], double, int, unisa.gps.etour.util.Punto3D, byte)
	 */
	public int ricerca(int pIdTurista, String pParoleChiave, int[] pTagsId,
			double pRaggioMax, int pElementiPerPagina,
			Punto3D pPosizioneUtente, byte pTipologiaSito)
			throws RemoteException
	{
		// Azzera i parametri della ricerca precedente
		azzeraParametri();

		// Inizia il controllo di correttezza dei parametri di ricerca
		try
		{
			this.pIdTurista = pIdTurista;
			this.pTags = arrayToArrayListTag(pTagsId);
			this.pParoleChiave = ControlloDati.correggiStringa(pParoleChiave,
					true, true, "", ControlloDati.LUNGHEZZA_MASSIMA);
			this.pRaggioMax = pRaggioMax;
			this.pPosizioneUtente = pPosizioneUtente;
			this.pTipologiaSito = pTipologiaSito;

			// Se i parametri sono validi ottengo il numero di risultati
			if (pElementiPerPagina > 0
					&& ((pTipologiaSito == 0) || (pTipologiaSito == 1))
					&& pRaggioMax > 0
					&& pPosizioneUtente != null
					&& (this.numeroElementiRicerca = ottieniNumeroElementiRicercaSpecializzato()) >= 0)
			{
				this.pElementiPerPagina = pElementiPerPagina;
				this.numeroPagineRisultati = calcolaNumeroPagineRicerca();

				return this.numeroElementiRicerca;
			}

			// Altrimenti ritorno -1
			else
				return -1;
		}
		catch (SQLException e)
		{
			throw new RemoteException(MessaggiErrore.ERRORE_DBMS);
		}
		catch (Exception e)
		{
			throw new RemoteException(MessaggiErrore.ERRORE_SCONOSCIUTO);
		}

	}

	/* (non-Javadoc)
	 * @see unisa.gps.etour.control.GestioneRicerche.IRicerca#ottieniPaginaRisultatiBeneCulturale(int)
	 */
	public BeanBeneCulturale[] ottieniPaginaRisultatiBeneCulturale(int pPagina)
			throws RemoteException
	{
		// Controlla che la tipologia di sito sia valida e che la pagina sia
		// compresa nel range
		if ((this.pTipologiaSito == BENE_CULTURALE) && controllaPagina(pPagina))
		{
			// Trasforma la lista ottenuta in array
			BeanBeneCulturale[] risultati = new BeanBeneCulturale[calcolaNumeroElementiPagina(pPagina)];
			// risultati = ricercaBeniCulturaliPerPagina(pPagina).toArray(
			// risultati);

			risultati = ricercaPerPagina(pPagina).toArray(risultati);

			return risultati;
		}
		else
			return null;
	}

	/* (non-Javadoc)
	 * @see unisa.gps.etour.control.GestioneRicerche.IRicerca#ottieniPaginaRisultatiPuntoDiRistoro(int)
	 */
	public BeanPuntoDiRistoro[] ottieniPaginaRisultatiPuntoDiRistoro(int pPagina)
			throws RemoteException
	{
		// Controlla che la tipologia di sito sia valida e che la pagina sia
		// compresa nel range
		if ((this.pTipologiaSito == PUNTO_DI_RISTORO)
				&& controllaPagina(pPagina))
		{
			// Trasforma la lista ottenuta in array
			BeanPuntoDiRistoro[] risultati = new BeanPuntoDiRistoro[calcolaNumeroElementiPagina(pPagina)];
			// risultati = ricercaPuntiDiRistoroPerPagina(pPagina).toArray(
			// risultati);

			risultati = ricercaPerPagina(pPagina).toArray(risultati);

			return risultati;
		}
		else
			return null;
	}

	/* (non-Javadoc)
	 * @see unisa.gps.etour.control.GestioneRicerche.IRicerca#ottieniNumeroElementiRicerca()
	 */
	public int ottieniNumeroElementiRicerca() throws RemoteException
	{
		return numeroElementiRicerca;
	}

	/* (non-Javadoc)
	 * @see unisa.gps.etour.control.GestioneRicerche.IRicerca#ottieniNumeroPagineRicerca()
	 */
	public int ottieniNumeroPagineRicerca() throws RemoteException
	{
		return numeroPagineRisultati;
	}

	/**
	 * Metodo per la restituzione del numero di risultati emersi dalla ricerca
	 * 
	 * @return Numero di risultati emersi dalla ricerca. In caso di errore
	 *         ritorna -1
	 * @throws SQLException Eccezione Connessione al Database
	 */
	private int ottieniNumeroElementiRicercaSpecializzato() throws SQLException
	{
		// Controlla la tipologia di sito e ottiene il numero di risultati
		if (controllaIdTurista())
		{
			switch (pTipologiaSito)
			{
				case BENE_CULTURALE:
					return pIdTurista > 0 ? BeneCulturale
							.ottieniNumeroElementiRicercaAvanzata(
									this.pIdTurista, this.pParoleChiave,
									this.pTags, this.pPosizioneUtente,
									this.pRaggioMax) : BeneCulturale
							.ottieniNumeroElementiRicerca(this.pParoleChiave,
									this.pTags, this.pPosizioneUtente,
									this.pRaggioMax);
				case PUNTO_DI_RISTORO:
					return pIdTurista > 0 ? PuntoDiRistoro
							.ottieniNumeroElementiRicercaAvanzata(
									this.pIdTurista, this.pParoleChiave,
									this.pTags, this.pPosizioneUtente,
									this.pRaggioMax) : PuntoDiRistoro
							.ottieniNumeroElementiRicerca(this.pParoleChiave,
									this.pTags, this.pPosizioneUtente,
									this.pRaggioMax);
			}
		}
		return -1;
	}

	/**
	 * Metodo per la restituzione della lista dei risultati
	 * 
	 * @param pNumeroPagina Intervallo di risultati da restituire
	 * @return Contenitore dei siti emersi in fase di ricerca dato l'intervallo
	 * @throws RemoteException Eccezione Remota
	 */
	private ArrayList<?> ricercaPerPagina(int pNumeroPagina)
			throws RemoteException
	{

		// Dichiara la lista dei siti relativi alla pagina in input
		ArrayList<?> risultatiPaginaCorrente = null;
		try
		{
			if (controllaIdTurista())
			{
				// Controlla la tipologia di sito
				switch (pTipologiaSito)
				{
					case BENE_CULTURALE:
						// Se la lista non è gia stata ottenuta in precedenza la
						// ricerca
						if ((risultatiPaginaCorrente = risultatiRicercaBeniCulturali
								.get(pNumeroPagina)) == null)
						{
							risultatiPaginaCorrente = (ArrayList<?>) (pIdTurista > 0 ? BeneCulturale
									.ricercaAvanzata(pIdTurista, pParoleChiave,
											pTags, pNumeroPagina,
											pElementiPerPagina,
											pPosizioneUtente, pRaggioMax)
									: BeneCulturale.ricerca(pParoleChiave,
											pTags, pNumeroPagina,
											pElementiPerPagina,
											pPosizioneUtente, pRaggioMax));
							risultatiRicercaBeniCulturali
									.put(
											pNumeroPagina,
											(ArrayList<BeanBeneCulturale>) risultatiPaginaCorrente);
						}
					break;

					case PUNTO_DI_RISTORO:
						// Se la lista non è gia stata ottenuta in precedenza la
						// ricerca
						if ((risultatiPaginaCorrente = risultatiRicercaPuntiDiRistoro
								.get(pNumeroPagina)) == null)
						{
							risultatiPaginaCorrente = (ArrayList<?>) (pIdTurista > 0 ? PuntoDiRistoro
									.ricercaAvanzata(pIdTurista, pParoleChiave,
											pTags, pNumeroPagina,
											pElementiPerPagina,
											pPosizioneUtente, pRaggioMax)
									: PuntoDiRistoro.ricerca(pParoleChiave,
											pTags, pNumeroPagina,
											pElementiPerPagina,
											pPosizioneUtente, pRaggioMax));
							risultatiRicercaPuntiDiRistoro
									.put(
											pNumeroPagina,
											(ArrayList<BeanPuntoDiRistoro>) risultatiPaginaCorrente);
						}
					break;
				}
			}
			// Ritorna i risultati ottenuti
			return risultatiPaginaCorrente;
		}

		catch (SQLException e)
		{
			throw new RemoteException(MessaggiErrore.ERRORE_DBMS);
		}
		catch (Exception e)
		{
			throw new RemoteException(MessaggiErrore.ERRORE_SCONOSCIUTO);
		}

	}

	/**
	 * Metodo per il calcolo del numero delle pagine risultate in fase di
	 * ricerca
	 * 
	 * @return Numero di Pagine risultate in fase di ricerca
	 */
	private int calcolaNumeroPagineRicerca()
	{
		// Se il numero di pagine è maggiore o uguale a 0 ritorna il numero di
		// pagine
		if (numeroPagineRisultati >= 0)
			return numeroPagineRisultati;
		// Se il numero di elementi è multiplo del numero di elementi per pagina
		// ritorna il loro rapporto
		if ((numeroElementiRicerca % pElementiPerPagina == 0))
			return (numeroElementiRicerca / pElementiPerPagina);
		// Altrimenti ritorna il loro rapporto + 1
		return (numeroElementiRicerca / pElementiPerPagina) + 1;
	}

	/**
	 * Metodo per il calcolo del numero di elementi contenuti nell'intevallo di
	 * una determinata pagina
	 * 
	 * @param nPagina Intervallo di risultati
	 * @return Numero di elementi contenuti in una pagina
	 */
	private int calcolaNumeroElementiPagina(int nPagina)
	{
		// Se la il numero di pagina è l'ultima ritorna il loro modulo
		if (nPagina == (numeroPagineRisultati - 1))
			return (numeroElementiRicerca % pElementiPerPagina);
		// Altrimenti ritorna il numero di elementi per pagina
		else
			return (pElementiPerPagina);
	}

	/**
	 * Metodo per il controllo di validità di una pagina
	 * 
	 * @param pPagina Pagina da controllare
	 * @return true se la pagina è valida
	 */
	private boolean controllaPagina(int pPagina)
	{
		// Se il numero di pagina è compreso nel range ritorna vero
		return (pPagina >= 0 && pPagina < numeroPagineRisultati);
	}

	/** 
	 * Metodo per il controllo di correttezza dell'identificativo del Turista
	 *
	 * @return true se l'identificativo è valido, false altrimenti
	 */
	private boolean controllaIdTurista()
	{
		return ((pIdTurista > 0) || (pIdTurista == -1));
	}

	/**
	 * Metodo per il reset di tutte le variabili relative ad una ricerca
	 * 
	 * @return Costante che indica il corretto azzeramento
	 */
	private int azzeraParametri()
	{

		// Azzera tutti i parametri di ricerca
		pIdTurista = -1;
		pTipologiaSito = -1;
		pElementiPerPagina = -1;
		numeroElementiRicerca = -1;
		numeroPagineRisultati = -1;

		risultatiRicercaBeniCulturali.clear();
		risultatiRicercaPuntiDiRistoro.clear();

		return -1;
	}

	/**
	 * Metodo per la ricerca e conversione dei Tag nella struttara dati Lista
	 * 
	 * @param pTagsId Lista degli identificativi dei tag di ricerca
	 * @return Lista dei tag di ricerca
	 * @throws SQLException
	 */
	private ArrayList<BeanTag> arrayToArrayListTag(int[] pTagsId)
			throws SQLException
	{
		// Inizializza la lista di Tag
		ArrayList<BeanTag> pTags = new ArrayList<BeanTag>();
		// Se la lista degli identificatori dei tag non è vuota ottengo i tag
		if (pTagsId != null)
		{
			BeanTag currentTag = null;
			try
			{

				// Ciclo per l'inserimento dei tag trovati nella lista
				for (int i = 0; i < pTagsId.length; i++)
				{
					// Se l'identificativo è maggiore di 0
					if (pTagsId[i] > 0)
					{
						currentTag = Tag.ottieniTag(pTagsId[i]);
						// Controllo di correttezza sui tag
						if (ControlloDati.checkBeanTag(currentTag))
							pTags.add(currentTag);
					}
				}

			}
			catch (SQLException e)
			{
				throw new SQLException(MessaggiErrore.ERRORE_DBMS);
			}
		}
		// ritorna la lista di tag
		return pTags;
	}
}
