package unisa.gps.etour.gui.operatoreagenzia;

import java.awt.*;
import java.awt.event.*;
import java.rmi.RemoteException;
import java.rmi.registry.LocateRegistry;
import java.rmi.registry.Registry;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.StringTokenizer;
import java.util.Vector;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.table.TableColumn;
import javax.swing.text.JTextComponent;
import javax.swing.border.*;
import unisa.gps.etour.bean.BeanBeneCulturale;
import unisa.gps.etour.bean.BeanTag;
import unisa.gps.etour.bean.BeanVisitaBC;
import unisa.gps.etour.control.GestioneBeniCulturali.IGestioneBeniCulturaliAgenzia;
import unisa.gps.etour.control.GestioneTag.IGestioneTagComune;
import unisa.gps.etour.gui.operatoreagenzia.document.LimitedDocument;
import unisa.gps.etour.gui.operatoreagenzia.document.NumericDocument;
import unisa.gps.etour.gui.operatoreagenzia.document.OnlyCharactersDocument;
import unisa.gps.etour.gui.operatoreagenzia.tables.FeedBackTableModel;
import unisa.gps.etour.gui.operatoreagenzia.tables.MediaVotiRenderer;
import unisa.gps.etour.util.Punto3D;


/**
 * Classe che modella l'interfaccia per la visualizzazione della scheda, 
 * la modifica dei dati e l'inserimento di un nuovo bene culturale.
 * 
 * @version 1.0
 * @author Mario Gallo
 * 
 * &#169; 2007 eTour Project - Copyright by SE@SA Lab DMI -
 *         University of Salerno 
 * 
 */
public class SchedaBC extends JInternalFrame implements IScheda
{	

	private static final String[] txts = { "Nome", "Indirizzo", "Città", "Località", "CAP",
			"Provincia", "Posizione Geografica", "Telefono",
			"Orario di Apertura", "Orario di Chiusura",
			"Giorno di Chiusura", "Costo biglietto", "Descrizione" }; 
	private static final String[] help = {
			"Inserire il nome del bene culturale.",
			"Inserire l'indirizzo dove è situato il bene culturale.",
			"Inserire la citta' dove è situato il bene culturale.",
			"Inserire la localita' di appartenenza del bene culturale.",
			"Inserire il Codice di Avviamento Postale per la zona in cui è situato il bene culturale.",
			"Selezionare la provincia di appartenza per il bene culturale.",
			"Inserire le coordinate tridimensionali relative alla posizione del " +
			"bene culturale.",
			"Inserire il recapito telefono per l'ufficio di gestione del bene culturale.",
			"Selezionare l'orario di apertura al pubblico del bene culturale.",
			"Selezionare l'orario di chiusura al pubblico del bene culturale.",
			"Selezionare il giorno di chiusura settimanale.",
			"Indicare il costo del biglietto di ingresso al bene culturale.",
			"<html>Inserire una descrizione completa ed esaustiva per il bene culturale.<br>Attenzione! Tale " +
			"descrizione verrà utilizzata come sorgente delle parole chiave<br>nelle ricerche effettuate dai " +
			"turisti</html>",
			"<html>Seleziona i tag di ricerca per il bene culturale.<br>I tag di ricerca consentono ai turisti di ricercare" + 
			" i siti con le caratteristiche di interesse.</html>"};
	private JPanel jContentPane = null;
	private JToolBar toolbarSchedaBC = null;
	private JToggleButton btnModifica = null;
	private JButton btnSalva = null;
	private JButton btnAnnulla = null;
	private JButton btnModificaCommento = null;
	private JTabbedPane jTabbedPane = null;
	private JPanel statistiche = null;
	private JPanel feedback = null;
	private JTextField indirizzo2 = null;
	private JComboBox indirizzo1 = null;
	private JTextField cittaBC = null;
	private JTextField localitaBC = null;
	private JTextField capBC = null;
	private JTextField posGeoX = null;
	private JScrollPane jScrollPane = null;
	private JTextArea descrizioneBC = null;
	private JTextField telefonoBC = null;
	private JComboBox oreAP = null;
	private JComboBox minAP = null;
	private TagPanel pannelloTag;
	private JTextField costoBC = null;
	private JComboBox oreCH = null;
	private JComboBox minCH = null;
	private JComboBox provBC = null;
	private JPanel datiBC = null;
	private JTextField nomeBC = null;
	private JScrollPane scrollPaneFeedback = null;
	private JTable tableFeedback = null;
	private JLabel txtNomeBene = null;
	private JLabel mediaVotoBC = null;
	private JPanel statMeseCorrente = null;
	private JPanel statTotali = null;
	private JTextField posGeoY = null;
	private JTextField posGeoZ = null;
	private Vector<JLabel> suggerimenti;
	private BeanBeneCulturale bc;
	private JComboBox giornoChiusura;
	private JLabel[] statMeseC;
	private JLabel[] statT;
	private BeniCulturali parent;
	private FeedBackTableModel feedbackModel;
	protected IGestioneTagComune tag;
	protected IGestioneBeniCulturaliAgenzia gestioneBC;
	private ArrayList<Integer> idTag = null;

	/**
	 * Il costruttore di default modella l'interfaccia relativa all'inserimento di
	 * un nuovo bene culturale.
	 * 
	 */
	public SchedaBC(BeniCulturali pParent)
	{
		super("Nuovo Bene Culturale");
		frameIcon = new ImageIcon(getClass().getResource(
			Home.URL_IMAGES + "nuovoBC.png"));
		closable = true;
		resizable = false;
		iconable = true;
		setSize(600, 560);
		setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
		suggerimenti = new Vector<JLabel>();
		parent = pParent;
		bc = null;
		
		//Inizializzazione del content pane
		jContentPane = new JPanel();
		jContentPane.setLayout(new BorderLayout());
		jContentPane.add(getToolbarSchedaBC(), BorderLayout.SOUTH);
		jTabbedPane = new JTabbedPane();
		jTabbedPane.addTab(	"Dati Bene Culturale",
						new ImageIcon(getClass().getResource(
							Home.URL_IMAGES + "dati.png")),
						getDatiBCForm(), null);
		jContentPane.add(jTabbedPane, BorderLayout.CENTER);
		setContentPane(jContentPane);
		
		//Dialogo di chiusura per la chiusura della finestra di inserimento.
		addInternalFrameListener(new InternalFrameAdapter()
		{
			/*
			 * All'inserimento del frame nel desktop pane recupera desktop pane
			 * e desktop manager e inizializza gli oggetti remoti per la gestione
			 * dei beni culturali.
			 */ 
			
			public void internalFrameOpened(InternalFrameEvent pEvent)
			{
				JInternalFrame frame = pEvent.getInternalFrame();
			
				//Setting up degli oggetti remoti per la gestione dei beni culturali.
				try
				{
					Registry reg = LocateRegistry.getRegistry(Home.HOST);
					tag = 
						(IGestioneTagComune)reg.lookup("GestioneTagComune");
					gestioneBC = 
						(IGestioneBeniCulturaliAgenzia) reg.lookup("GestioneBeniCulturaliAgenzia");
					//Caricamento dei dati.
					caricaTags();
				}
				/*
				 * Due eccezioni: RemoteException e NotBoundException. Il
				 * risultato è lo stesso. La gestione non è operabile e la
				 * finestra dopo il messaggio di errore verrà chiusa.
				 */
				catch (Exception ex)
				{
					JLabel errore = new JLabel(
							"<html><h2>Impossibile comunicare con il server di eTour.</h2>"
									+ "<h3><u>La scheda per l'inserimento di un nuovo bene culturale verrà chiusa.</u></h3>"
									+ "<p><b>Possibili Cause:</b>"
									+ "<ul><li>Nessuna connessione alla rete.</li>"
									+ "<li>Server inattivo.</li>"
									+ "<li>Server intasato.</li></ul>"
									+ "<p>Si prega di riprovare in un secondo momento.</p>"
									+ "<p>Se l'errore dovesse persistere, si prega di contattare l'assistenza tecnica.</p>"
									+ "<p>Ci scusiamo per il disagio.</html>");
					ImageIcon err = new ImageIcon(getClass().getResource(
							Home.URL_IMAGES + "error48.png"));
					JOptionPane.showMessageDialog(frame, errore,
							"Errore!", JOptionPane.ERROR_MESSAGE, err);
					frame.dispose();
				}
			}
			
			public void internalFrameClosing(InternalFrameEvent pEvent)
			{
				JPanel root = new JPanel(new BorderLayout());
				JLabel message = new JLabel(
					"Sei sicuro di voler annullare la creazione di un nuovo bene culturale?");
				message.setFont(new Font("Dialog", Font.BOLD, 14));
				JLabel avviso = new JLabel(
						"Attenzione! I dati non salvati verranno persi.", SwingConstants.CENTER);
				avviso.setIcon(new ImageIcon(getClass().getResource(
					Home.URL_IMAGES + "warning16.png")));
				root.add(message, BorderLayout.NORTH);
				root.add(avviso, BorderLayout.SOUTH);
				String[] options = { "Chiudi", "Annulla" };
				int choice = JOptionPane.showInternalOptionDialog(jContentPane, root, 
						"Conferma chiusura",
						JOptionPane.OK_CANCEL_OPTION, JOptionPane.QUESTION_MESSAGE, frameIcon, options, options[1]);  
				if (choice == JOptionPane.OK_OPTION)
				{
					parent.closeScheda((SchedaBC)pEvent.getInternalFrame());
				}
			}
		});
		
		//Inizializzazione dei pulsanti
		btnModifica.setVisible(false);
		btnSalva.setVisible(true);
		btnAnnulla.setVisible(true);
		btnAnnulla.setText("Azzera");
		
		
	}

	/**
	 * Questo costruttore modella le interfacce relative alla modifica dei dati e alla
	 * visualizzazione della scheda di un bene culturale.
	 * 
	 * @param BeanBeneCulturale pBc - il bean contenente i dati del 
	 * bene culturale selezionato.
	 * @param boolean pModifica<ul><li>true - i campi saranno editabili e quindi si sta 
	 * procedendo alla modifica dei dati di un bene culturale.<li>false - i campi non saranno
	 * editabili e quindi si sta visualizzando la scheda di un bene culturale.</ul>
	 * 
	 */
	public SchedaBC(BeniCulturali pParent, BeanBeneCulturale pBc, boolean pModifica)
	{
		super();
		frameIcon = new ImageIcon(getClass().getResource(
				Home.URL_IMAGES + "scheda.png"));
		closable = true;
		resizable = false;
		iconable = true;
		setSize(600, 560);
		setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
			
		//Inizializzazione variabili di istanza
		bc = pBc;
		this.parent = pParent;
		suggerimenti = new Vector<JLabel>();
		initializeSchedaBC();

		if (pModifica) // Si sta modificando i dati del bene culturale.
		{
			btnModifica.setSelected(true);
			btnSalva.setVisible(true);
			btnAnnulla.setVisible(true);
			btnAnnulla.setText("Reimposta");
		}
		else // Si sta visualizzando la scheda di un bene culturale.
		{
			mostraNascondiSuggerimenti();
			attivaDisattivaEdit();
		}

	}

	/**
	 * Questo metodo ritorna l'id del bene culturale di cui si sta visualizzando la
	 * scheda o modificando i dati.
	 * 
	 * @return int - l'id del bene culturale.
	 * 
	 */
	public int getId()
	{
		if (bc == null) 
		{
			return -1;
		}
		return bc.getId();
	}

	/** 
	 * 
	 * Questo metodo inizializza l'interfaccia per la visualizzazione della scheda di
	 * un bene culturale.
	 *
	 * @return void
	 */
	private void initializeSchedaBC()
	{
		setTitle(bc.getNome());
		
		// Dialogo di chiusura del frame
		addInternalFrameListener(new InternalFrameAdapter()
		{
			/*
			 * All'inserimento del frame nel desktop pane recupera desktop pane
			 * e desktop manager e inizializza gli oggetti remoti per la gestione
			 * dei beni culturali.
			 */ 
			
			public void internalFrameOpened(InternalFrameEvent pEvent)
			{
				JInternalFrame frame = pEvent.getInternalFrame();
			
				//Setting up degli oggetti remoti per la gestione dei beni culturali.
				try
				{
					Registry reg = LocateRegistry.getRegistry(Home.HOST);
					tag = 
						(IGestioneTagComune)reg.lookup("GestioneTagComune");
					gestioneBC = 
						(IGestioneBeniCulturaliAgenzia) reg.lookup("GestioneBeniCulturaliAgenzia");
					//Caricamento dei dati.
					caricaTags();
					caricaStatistiche();
				}
				/*
				 * Due eccezioni: RemoteException e NotBoundException. Il
				 * risultato è lo stesso. La gestione non è operabile e la
				 * finestra dopo il messaggio di errore verrà chiusa.
				 */
				catch (Exception ex)
				{
					JLabel errore = new JLabel(
							"<html><h2>Impossibile comunicare con il server di eTour.</h2>"
									+ "<h3><u>La scheda del bene culturale richiesta verrà chiusa.</u></h3>"
									+ "<p><b>Possibili Cause:</b>"
									+ "<ul><li>Nessuna connessione alla rete.</li>"
									+ "<li>Server inattivo.</li>"
									+ "<li>Server intasato.</li></ul>"
									+ "<p>Si prega di riprovare in un secondo momento.</p>"
									+ "<p>Se l'errore dovesse persistere, si prega di contattare l'assistenza tecnica.</p>"
									+ "<p>Ci scusiamo per il disagio.</html>");
					ImageIcon err = new ImageIcon(getClass().getResource(
							Home.URL_IMAGES + "error48.png"));
					JOptionPane.showMessageDialog(frame, errore,
							"Errore!", JOptionPane.ERROR_MESSAGE, err);
					frame.dispose();
				}
			}
			
			public void internalFrameClosing(InternalFrameEvent pEvent)
			{
				//Se si sta effettuando una modifica chiede conferma.
				if (btnModifica.isSelected())
				{
					JPanel root = new JPanel(new BorderLayout());
					JLabel message = new JLabel(
						"Sei sicuro di voler chiudere la scheda di questo bene culturale?");
					message.setFont(new Font("Dialog", Font.BOLD, 14));
					JLabel avviso = new JLabel(
						"Attenzione! I dati non salvati verranno persi.", SwingConstants.CENTER);
					avviso.setIcon(new ImageIcon(getClass().getResource(
							Home.URL_IMAGES + "warning16.png")));
					root.add(message, BorderLayout.NORTH);
					root.add(avviso, BorderLayout.SOUTH);
					String[] options = { "Chiudi", "Annulla" };
					int choice = JOptionPane.showInternalOptionDialog(jContentPane, root, 
							"Conferma chiusura Scheda Bene Culturale " + bc.getNome(),
							JOptionPane.OK_CANCEL_OPTION, JOptionPane.QUESTION_MESSAGE, frameIcon, options, options[1]);  
					if (choice == JOptionPane.OK_OPTION)
					{
						parent.closeScheda((SchedaBC)pEvent.getInternalFrame());
					}
				}
				// Altrimenti chiude direttamente la finestra.
				else 
				{
					parent.closeScheda((SchedaBC)pEvent.getInternalFrame());	
				}
			}
		});
		
		//Inizializza il content pane.
		jContentPane = new JPanel();
		jContentPane.setLayout(new BorderLayout());
		jContentPane.add(getToolbarSchedaBC(), BorderLayout.SOUTH);
		jTabbedPane = new JTabbedPane();
		jTabbedPane.addTab("Dati Bene Culturale",
						new ImageIcon(getClass().getResource(
								Home.URL_IMAGES + "dati.png")),
						getDatiBCForm(), null);
		JScrollPane nuovo = new JScrollPane(getStatistiche());
		nuovo.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
		jTabbedPane.addTab("Statistiche", 
				new ImageIcon(getClass().getResource(
					Home.URL_IMAGES + "stat24.png")),nuovo, null);
		jTabbedPane.addTab(	"Feedback Ricevuti",
						new ImageIcon(getClass().getResource(
								Home.URL_IMAGES + "feedback.png")),
						getFeedback(), null);
		jContentPane.add(jTabbedPane, BorderLayout.CENTER);
		setContentPane(jContentPane);
		jTabbedPane.addChangeListener(new ChangeListener()
		{

			public void stateChanged(ChangeEvent pChange)
			{
				// Dati bene culturale
				if (jTabbedPane.getSelectedIndex() == 0)
				{
					toolbarSchedaBC.setVisible(true);
					btnModifica.setVisible(true);
					if (btnModifica.isSelected())
					{
						btnSalva.setVisible(true);
						btnAnnulla.setVisible(true);
					}
					btnModificaCommento.setVisible(false);
				}
				// Statistiche
				else if (jTabbedPane.getSelectedIndex() == 1)
				{
					toolbarSchedaBC.setVisible(false);
				}
				// Feedback ricevuti
				else
				{
					if (btnModifica.isSelected())
					{
						btnSalva.setVisible(false);
						btnAnnulla.setVisible(false);
					}
					toolbarSchedaBC.setVisible(true);
					btnModificaCommento.setVisible(true);
					btnModifica.setVisible(false);

				}

			}

		});
		
		//Carica i dati del bene culturale e le statistiche.
		caricaDatiForm();
	}

	/**
	 * 
	 * Questo metodo carica i dati del bene culturale fornito al costruttore
	 * nella form.
	 * 
	 */
	private void caricaDatiForm()
	{
		nomeBC.setText(bc.getNome());
		capBC.setText(bc.getCap());
		cittaBC.setText(bc.getCitta());
		costoBC.setText("" + bc.getCostoBiglietto());
		descrizioneBC.setText(bc.getDescrizione());
		StringTokenizer tokenizer = new StringTokenizer(bc.getVia());
		String stringa = tokenizer.nextToken();
		indirizzo1.setSelectedItem(stringa);
		indirizzo2.setText(bc.getVia().substring(stringa.length()));
		provBC.setSelectedItem(bc.getProvincia());
		Punto3D pos = bc.getPosizione();
		posGeoX.setText("" + pos.getLatitudine());
		posGeoY.setText("" + pos.getLongitudine());
		posGeoZ.setText("" + pos.getAltitudine());
		telefonoBC.setText(bc.getTelefono());
		int minuti = bc.getOrarioApertura().getMinutes();
		if (minuti == 0)
		{
			minAP.setSelectedIndex(0);
		}
		else
		{
			minAP.setSelectedItem(minuti);
		}
		int ore = bc.getOrarioApertura().getHours();
		if (ore < 10)
		{
			oreAP.setSelectedItem("0" + ore);
		}
		else
		{
			oreAP.setSelectedItem(ore);
		}
		minuti = bc.getOrarioChiusura().getMinutes();
		if (minuti == 0)
		{
			minCH.setSelectedIndex(0);
		}
		else
		{
			minCH.setSelectedItem(minuti);
		}
		ore = bc.getOrarioChiusura().getHours();
		if (ore < 10)
		{
			oreCH.setSelectedItem("0" + ore);
		}
		else
		{
			oreCH.setSelectedItem(ore);
		}
	}

	/**
	 * 
	 * Questo metodo carica le statistiche relative al bene culturale fornito
	 * in input al costruttore della classe.
	 * 
	 */
	private void caricaStatistiche()
	{
		txtNomeBene.setText(bc.getNome());
		double voto = bc.getMediaVoti();
		if (voto >= 4)
		{
			mediaVotoBC.setIcon(new ImageIcon(getClass().getResource(
					Home.URL_IMAGES + "stella5.gif")));
		}
		else if (voto < 4 && voto >= 3)
		{
			mediaVotoBC.setIcon(new ImageIcon(getClass().getResource(
					Home.URL_IMAGES + "stella4.gif")));
		}
		else if (voto < 3 && voto >= 2)
		{	
			mediaVotoBC.setIcon(new ImageIcon(getClass().getResource(
					Home.URL_IMAGES + "stella3.gif")));
		}
		else if (voto < 2 && voto >= 1)
		{
			mediaVotoBC.setIcon(new ImageIcon(getClass().getResource(
					Home.URL_IMAGES + "stella2.gif")));
		}
		else
		{
			mediaVotoBC.setIcon(new ImageIcon(getClass().getResource(
					Home.URL_IMAGES + "stella1.gif")));
		}
		
		ArrayList<Integer> stats = null;
		try
		{
			 stats = 
				gestioneBC.ottieniStatisticheBeneCulturale(bc.getId());
			
		}
		catch (RemoteException e)
		{
		}
		statMeseC[0].setText("" + stats.get(0));
		statMeseC[1].setText("" + stats.get(1));
		statMeseC[2].setText("" + stats.get(2));
		statMeseC[3].setText("" + stats.get(3));
		statMeseC[4].setText("" + stats.get(4));
		statMeseC[5].setText("" + stats.get(5));
		statT[0].setText("142");
		statT[1].setText("112");
		statT[2].setText("132");
		statT[3].setText("212");
		statT[4].setText("152");
		statT[5].setText("748");
		//FINE TEST
	}

	/**
	 * 
	 * Questo metodo mostra o nasconde i suggerimenti presenti accanto alle label
	 * della form.
	 * 
	 */
	private void mostraNascondiSuggerimenti()
	{
		Iterator<JLabel> s = suggerimenti.iterator();
		while (s.hasNext())
		{
			JLabel corrente = s.next();
			corrente.setVisible(corrente.isVisible() ? false : true);
		}
	}

	/**
	 * 
	 * Questo metodo rende editabile o meno la form.
	 * 
	 */
	private void attivaDisattivaEdit()
	{
		Component[] componenti = datiBC.getComponents();
		for (int i = 0; i < componenti.length; i++)
		{
			Component corrente = componenti[i];
			if (corrente instanceof JTextField)
			{
				JTextField textbox = (JTextField) corrente;
				textbox.setEditable(textbox.isEditable() ? false : true);
				textbox.setBackground(Color.white);

			}
			else if (corrente instanceof JComboBox)
			{
				JComboBox combobox = (JComboBox) corrente;
				combobox.setEnabled(combobox.isEnabled() ? false : true);

			}
		}
		descrizioneBC.setEditable(descrizioneBC.isEditable() ? false : true);
		pannelloTag.attivaDisattiva();
	}

	/**
	 * Questo metodo inizializza la toolbar della scheda di un bene culturale.
	 * 	
	 * @return javax.swing.JToolBar	- la toolbar.
	 */
	private JToolBar getToolbarSchedaBC()
	{
		if (toolbarSchedaBC == null)
		{
			toolbarSchedaBC = new JToolBar();
			toolbarSchedaBC.setFloatable(false);
			toolbarSchedaBC.add(getBtnModifica());
			toolbarSchedaBC.addSeparator();
			toolbarSchedaBC.add(getBtnSalva());
			toolbarSchedaBC.addSeparator();
			toolbarSchedaBC.add(getBtnAnnulla());
			toolbarSchedaBC.addSeparator();
			if (bc != null)
			{
				toolbarSchedaBC.add(getBtnModificaCommento());
				toolbarSchedaBC.addSeparator();
			}
		}
		return toolbarSchedaBC;
	}

	/**
	 * Questo metodo inizializza il pulsante per la modifica dei dati del bene
	 * culturale.
	 * 	
	 * @return javax.swing.JToggleButton - il pulsante per la modifica.
	 * 	
	 */
	private JToggleButton getBtnModifica()
	{
		if (btnModifica == null)
		{
			btnModifica = new JToggleButton();
			btnModifica.setText("Modifica Dati");
			btnModifica
					.setIcon(new ImageIcon(
							getClass()
									.getResource(
											Home.URL_IMAGES + "ModificaBC32.png")));
			btnModifica.setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
			btnModifica.addActionListener(new ActionListener()
			{

				public void actionPerformed(ActionEvent arg0)
				{
					mostraNascondiSuggerimenti();
					attivaDisattivaEdit();
					btnSalva.setVisible((btnModifica.isSelected() ? true
							: false));
					btnAnnulla.setVisible((btnModifica.isSelected() ? true
							: false));

				}

			});
		}
		return btnModifica;
	}

	/**
	 * This method initializes btnSalva	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getBtnSalva()
	{
		if (btnSalva == null)
		{
			btnSalva = new JButton();
			btnSalva.setText("Salva");
			btnSalva.setIcon(new ImageIcon(getClass().getResource(
					Home.URL_IMAGES + "salva.png")));
			btnSalva.setVisible(false);
			btnSalva.setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
			btnSalva.addActionListener(new ActionListener()
			{
				public void actionPerformed(ActionEvent pEvent)
				{
					if (bc == null)
					{
						bc = riversaDatiNelBean();
						try 
						{
							gestioneBC.inserisciBeneCulturale(bc);
						}
						catch (RemoteException ex)
						{
							ex.printStackTrace();
						}
					}
					else {
					//Costruzione della dialog per la conferma della modifica
					JPanel root = new JPanel(new BorderLayout());
					JLabel message = new JLabel(
							"Aggiornare il i dati del bene culturale "
									+ bc.getNome() + " con i"
									+ " dati della form?");
					message.setFont(new Font("Dialog", Font.BOLD, 14));
					JLabel avviso = new JLabel(
							"I dati precedenti non potranno essere pi� recuperati.",
							SwingConstants.CENTER);
					avviso
							.setIcon(new ImageIcon(
									getClass()
											.getResource(
													Home.URL_IMAGES + "warning16.png")));
					root.add(message, BorderLayout.NORTH);
					root.add(avviso, BorderLayout.SOUTH);
					String[] options = { "Modifica", "Annulla" };
					//La dialog viene visualizzata a schermo
					int scelta = JOptionPane
							.showInternalOptionDialog(
									jContentPane,
									root,
									"Conferma Modifica Dati Bene Culturale",
									JOptionPane.YES_NO_OPTION,
									JOptionPane.QUESTION_MESSAGE,
									new ImageIcon(
											getClass()
													.getResource(
															Home.URL_IMAGES + "ModificaBC48.png")),
									options, options[1]);
					//Se si � scelto di confermare la modifica
					if (scelta == JOptionPane.YES_OPTION)
					{
						bc = riversaDatiNelBean();
						caricaDatiForm();
						attivaDisattivaEdit();
						btnSalva.setVisible(false);
						btnAnnulla.setVisible(false);
						btnModifica.setSelected(false);
						mostraNascondiSuggerimenti();
						parent.updateTableModel(bc);
						JOptionPane
								.showInternalMessageDialog(
										jContentPane,
										"I dati del bene culturale sono stati aggiornati correttamente.",
										"Dati bene culturale modificati!",
										JOptionPane.OK_OPTION,
										new ImageIcon(
												getClass()
														.getResource(
																Home.URL_IMAGES + "ok32.png")));
					}
				}
				}
			});
		}
		return btnSalva;
	}

	/**
	 * Questo metodo inizializza il pulsante per l'azzeramento della form (nuovo bene
	 * culturale) o per ricaricare i dati del bene culturale (modifica dati).
	 * 	
	 * @return javax.swing.JButton	- il pulsante di cui sopra.
	 * 
	 */
	private JButton getBtnAnnulla()
	{
		if (btnAnnulla == null)
		{
			btnAnnulla = new JButton();
			btnAnnulla.setText("Annulla");
			btnAnnulla
					.setIcon(new ImageIcon(
							getClass()
									.getResource(
											Home.URL_IMAGES + "Annulla32.png")));
			btnAnnulla.setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
			btnAnnulla.setVisible(false);
			btnAnnulla.addActionListener(new ActionListener()
			{

				public void actionPerformed(ActionEvent arg0)
				{
					if (bc == null)
					{
						Component[] componenti = datiBC.getComponents();
						for (int i = 0; i < componenti.length; i++)
						{
							Component corrente = componenti[i];
							if (corrente instanceof JTextComponent)
							{
								((JTextComponent) corrente).setText("");
							}
							else if (corrente instanceof JComboBox)
							{
								JComboBox combobox = (JComboBox) corrente;
								combobox.setSelectedIndex(-1);
							}
						}
						pannelloTag.azzera();
						descrizioneBC.setText("");
					}
					else
					{
						caricaDatiForm();
					}

				}

			});
		}
		return btnAnnulla;
	}

	/**
	 * Questo metodo inizializza il pulsante per la modifica di un commento.
	 * 	
	 * @return javax.swing.JButton - il pulsante per la modifica di un commento.
	 */
	private JButton getBtnModificaCommento()
	{
		if (btnModificaCommento == null)
		{
			btnModificaCommento = new JButton();
			btnModificaCommento.setText("Modifica Commento");
			btnModificaCommento
					.setIcon(new ImageIcon(
							getClass()
									.getResource(
											Home.URL_IMAGES + "modificaCommento.png")));
			btnModificaCommento.setVisible(false);
			btnModificaCommento.setEnabled(false);
			btnModificaCommento.addActionListener(new ActionListener()
			{

				public void actionPerformed(ActionEvent pEvent)
				{
					int selectedRow = tableFeedback.getSelectedRow();
					String nuovoCommento = (String) JOptionPane.showInternalInputDialog(
							jContentPane,"Modifica del commento selezionato:",
									"Modifica Commento",JOptionPane.QUESTION_MESSAGE, 
							new ImageIcon(getClass().getResource(
									Home.URL_IMAGES + "modificaCommento.png")), 
								null, feedbackModel.getValueAt(selectedRow, 1));
					if (nuovoCommento != null)
					{
						feedbackModel.modificaCommento(nuovoCommento, selectedRow);
					}
				}
				
			});
		}
		return btnModificaCommento;
	}

	/**
	 * Questo metodo inizializza la form per i dati di un bene culturale.
	 * 	
	 * @return javax.swing.JPanel - la form per i dati.
	 * 
	 */
	private JPanel getDatiBCForm()
	{
		if (datiBC == null)
		{
			datiBC = new JPanel(null);
			datiBC.setBorder(new SoftBevelBorder(SoftBevelBorder.LOWERED));

			//Creazione Suggerimenti
			for (int i = 0; i < help.length; i++)
			{
				JLabel nuova = new JLabel();
				nuova.setIcon(new ImageIcon(getClass().getResource(
						Home.URL_IMAGES + "Info16.png")));
				nuova.setBounds(145, 8 + 30 * i, 24, 24);
				nuova.setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
				nuova.setToolTipText(help[i]);
				suggerimenti.add(nuova);
				datiBC.add(nuova);
				if (i == help.length - 1)
				{
					nuova.setBounds(400, 155, 24, 24);
				}

			}

			for (int i = 0; i < txts.length; i++)
			{
				JLabel nuova = new JLabel(txts[i], SwingConstants.RIGHT);
				nuova.setBounds(25, 10 + 30 * i, 120, 16);
				nuova.repaint();
				datiBC.add(nuova, null);
			}
			//Nome Bene Culturale
			nomeBC = new JTextField();
			nomeBC.setColumns(12);
			nomeBC.setDocument(new LimitedDocument(30));
			nomeBC.setBounds(185, 10, 136, 20);
			nomeBC.setName("Nome Bene Culturale");
			datiBC.add(nomeBC, null);

			//Indirizzo
			indirizzo2 = new JTextField();
			indirizzo2.setBounds(270, 40, 136, 20);
			indirizzo2.setDocument(new LimitedDocument(30));
			indirizzo1 = new JComboBox(INDIRIZZI);
			indirizzo1.setSelectedIndex(-1);
			indirizzo1.setBounds(185, 40, 60, 20);
			indirizzo2.setName("Indirizzo");
			datiBC.add(indirizzo2, null);
			datiBC.add(indirizzo1, null);
			
			//Citt�
			cittaBC = new JTextField();
			cittaBC.setColumns(12);
			cittaBC.setBounds(185, 70, 136, 20);
			cittaBC.setName("Citt�");
			cittaBC.setDocument(new OnlyCharactersDocument(25));
			datiBC.add(cittaBC);

			//Localita 
			localitaBC = new JTextField();
			localitaBC.setBounds(185, 100, 136, 20);
			localitaBC.setName("Localit�");
			localitaBC.setDocument(new OnlyCharactersDocument(25));
			datiBC.add(localitaBC, null);

			//CAP
			capBC = new JTextField();
			capBC.setColumns(8);
			capBC.setBounds(185, 130, 92, 20);
			capBC.setDocument(new NumericDocument(5));
			datiBC.add(capBC, null);
			

			//Posizione Geografica
			JLabel txtX = new JLabel("X");
			JLabel txtY = new JLabel("Y");
			JLabel txtZ = new JLabel("Z");
			Font nuovo = new Font("Dialog", Font.BOLD, 14);
			txtX.setFont(nuovo);
			txtY.setFont(nuovo);
			txtZ.setFont(nuovo);
			txtZ.setBounds(365, 190, 10, 22);
			txtY.setBounds(295, 190, 10, 22);
			txtX.setBounds(227, 190, 14, 20);
			posGeoX = new JTextField(12);
			posGeoX.setBounds(185, 190, 40, 20);
			posGeoY = new JTextField(12);
			posGeoY.setBounds(255, 190, 40, 20);
			posGeoZ = new JTextField(12);
			posGeoZ.setBounds(325, 190, 40, 20);
			posGeoX.setName("Coordinata X");
			posGeoY.setName("Coordinata Y");
			posGeoZ.setName("Coordinata Z");
			datiBC.add(txtX, null);
			datiBC.add(txtY, null);
			datiBC.add(txtZ, null);
			datiBC.add(posGeoX, null);
			datiBC.add(posGeoY, null);
			datiBC.add(posGeoZ, null);

			//Provincia
			provBC = new JComboBox(PROVINCE);
			provBC.setSelectedIndex(-1);
			provBC.setBounds(185, 160, 50, 20);
			datiBC.add(provBC, null);

			//Descrizione
			descrizioneBC = new JTextArea();
			descrizioneBC.setCursor(new Cursor(Cursor.TEXT_CURSOR));
			descrizioneBC.setWrapStyleWord(true);
			descrizioneBC.setLineWrap(true);
			jScrollPane = new JScrollPane(descrizioneBC);
			jScrollPane
					.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
			jScrollPane.setBounds(new Rectangle(185, 370, 395, 69));
			descrizioneBC.setName("Descrizione");
			datiBC.add(jScrollPane, null);

			//Telefono
			telefonoBC = new JTextField(12);
			telefonoBC.setBounds(185, 220, 136, 20);
			telefonoBC.setDocument(new NumericDocument(10));
			telefonoBC.setName("Telefono");
			datiBC.add(telefonoBC, null);

			//Orari
			oreAP = new JComboBox();
			oreAP.setBounds(185, 250, 40, 20);
			oreCH = new JComboBox();
			oreCH.setBounds(185, 280, 40, 20);
			for (int i = 0; i < 24; i++)
			{
				if (i < 10)
				{
					oreCH.addItem("0" + i);
					oreAP.addItem("0" + i);
				}
				else
				{
					oreAP.addItem(i);
					oreCH.addItem(i);
				}
			}
			minAP = new JComboBox();
			minAP.setBounds(241, 250, 40, 20);
			minAP.addItem("00");
			minAP.addItem("15");
			minAP.addItem("30");
			minAP.addItem("45");
			minCH = new JComboBox();
			minCH.setBounds(241, 280, 40, 20);
			minCH.addItem("00");
			minCH.addItem("15");
			minCH.addItem("30");
			minCH.addItem("45");
			minAP.setSelectedIndex(0);
			oreAP.setSelectedIndex(-1);
			minCH.setSelectedIndex(0);
			oreCH.setSelectedIndex(-1);
			nuovo = new Font("Dialog", Font.BOLD, 18);
			JLabel punto1 = new JLabel(":");
			punto1.setBounds(230, 245, 10, 24);
			punto1.setFont(nuovo);
			JLabel punto2 = new JLabel(":");
			punto2.setBounds(230, 275, 10, 24);
			punto2.setFont(nuovo);
			datiBC.add(oreAP, null);
			datiBC.add(minAP, null);
			datiBC.add(minCH, null);
			datiBC.add(oreCH, null);
			datiBC.add(punto1, null);
			datiBC.add(punto2, null);

			//Giorno di chiusura
			String[] giorni = { "Lunedì", "Martedì", "Mercoledì", "Giovedì",
					"Venerdì", "Sabato", "Domenica" };
			giornoChiusura = new JComboBox(giorni);
			giornoChiusura.setBounds(185, 310, 96, 20);
			giornoChiusura.setSelectedIndex(-1);
			datiBC.add(giornoChiusura, null);

			//Costo
			costoBC = new JTextField();
			costoBC.setColumns(8);
			costoBC.setBounds(185, 340, 40, 20);
			JLabel euro = new JLabel("Euro");
			euro.setBounds(230, 340, 30, 16);
			datiBC.add(costoBC, null);
			datiBC.add(euro, null);

			//PannelloTag
			pannelloTag = new TagPanel();
			pannelloTag.setBounds(405, 180, 180, 170);
			JLabel txtTag = new JLabel("Tag di ricerca");
			txtTag.setBounds(420, 150,140, 30);
			datiBC.add(txtTag,null);
			datiBC.add(pannelloTag, null);

		}
		return datiBC;
	}

	/**
	 * Questo metodo inizializza il pannello contenitore delle statistiche di un 
	 * bene culturale.
	 * 	
	 * @return javax.swing.JPanel - il pannello delle statistiche.
	 * 	
	 */
	private JPanel getStatistiche()
	{
		if (statistiche == null)
		{
			statistiche = new JPanel(new GridBagLayout());
			GridBagConstraints g = new GridBagConstraints();
			g.gridx = 0;
			g.gridy = 0;
			g.insets = new Insets(5, 5, 5, 5);
			g.anchor = GridBagConstraints.WEST;
			JLabel stat = new JLabel("Statistiche");
			Font nuovo = new Font("Dialog", Font.BOLD, 18);
			stat.setFont(nuovo);
			statistiche.add(stat, g);
			g.gridx = 1;
			txtNomeBene = new JLabel();
			txtNomeBene.setFont(nuovo);
			statistiche.add(txtNomeBene, g);
			g.gridx = 2;
			mediaVotoBC = new JLabel();
			statistiche.add(mediaVotoBC, g);
			g.gridwidth = 3;
			g.gridx = 0;
			g.anchor = GridBagConstraints.CENTER;
			g.gridy = 1;
			statistiche.add(getStatMeseCorrente(), g);
			g.gridy = 2;
			statistiche.add(getStatTotali(), g);
		}
		return statistiche;
	}

	/**
	 * Questo metodo inizializza il pannello per la visualizzazione dei feedback 
	 * ricevuti da un bene culturale.
	 * 	
	 * @return javax.swing.JPanel - il pannello dei feedback ricevuti.
	 * 
	 */
	private JPanel getFeedback()
	{
		if (feedback == null)
		{
			feedback = new JPanel();
			feedback.setLayout(new BorderLayout());
			feedbackModel = new FeedBackTableModel();
			tableFeedback = new JTable(feedbackModel);
			TableColumn aColumn = tableFeedback.getColumnModel().getColumn(0);
			//Voto
			aColumn.setPreferredWidth(80);
			aColumn.setCellRenderer(new MediaVotiRenderer());
			//Commento
			aColumn = tableFeedback.getColumnModel().getColumn(1);
			aColumn.setPreferredWidth(260);
			//Data di rilascio
			aColumn = tableFeedback.getColumnModel().getColumn(2);
			aColumn.setPreferredWidth(80);
			//Username
			aColumn = tableFeedback.getColumnModel().getColumn(3);
			aColumn.setPreferredWidth(80);
			tableFeedback.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
			tableFeedback.setColumnSelectionAllowed(false);
			ListSelectionModel selectionModel = tableFeedback.getSelectionModel();
			selectionModel
					.addListSelectionListener(new ListSelectionListener()
					{
						public void valueChanged(ListSelectionEvent event)
						{
							btnModificaCommento.setEnabled(
									(tableFeedback.getSelectedRow() != -1) 
										? true : false);
						}
					});
			scrollPaneFeedback = new JScrollPane(tableFeedback);
			scrollPaneFeedback
					.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
			feedback.add(scrollPaneFeedback, BorderLayout.CENTER);

		}
		return feedback;
	}

	/**
	 * Questo metodo inizializza il pannello delle statistiche per il mese corrente.
	 * 	
	 * @return javax.swing.JPanel - il pannello delle statistiche per il mese corrente.
	 * 
	 */
	private JPanel getStatMeseCorrente()
	{
		if (statMeseCorrente == null)
		{
			statMeseCorrente = new JPanel();
			statMeseCorrente.setLayout(new GridBagLayout());
			statMeseCorrente.setPreferredSize(new Dimension(500, 280));
			statMeseCorrente.setBorder(BorderFactory.createTitledBorder(
					BorderFactory.createLineBorder(new Color(51, 102, 255), 3),
					"Statistiche Mese Corrente",
					TitledBorder.DEFAULT_JUSTIFICATION,
					TitledBorder.DEFAULT_POSITION, new Font("Dialog",
							Font.BOLD, 12), new Color(0, 102, 204)));
			statMeseCorrente.setBackground(Color.white);
			statMeseC = new JLabel[6];
			GridBagConstraints g = new GridBagConstraints();
			g.gridx = 0;
			g.gridy = 0;
			g.gridwidth = 3;
			g.insets = new Insets(5, 5, 5, 8);
			g.anchor = GridBagConstraints.WEST;
			statMeseCorrente.add(new JLabel(
					"Dettaglio voti ricevuti nel mese corrente:"), g);
			g.anchor = GridBagConstraints.CENTER;
			g.gridwidth = 1;
			Font nuovo = new Font("Dialog", Font.BOLD, 16);
			for (int i = 5; i >= 1; i--)
			{

				int gridX = g.gridx;
				g.gridy++;
				JLabel aLabel = new JLabel(new ImageIcon(getClass()
						.getResource(
								Home.URL_IMAGES + "stella" + i
										+ ".gif")), JLabel.CENTER);
				statMeseCorrente.add(aLabel, g);
				g.gridx++;
				statMeseCorrente.add(new JLabel("=="), g);
				g.gridx++;
				g.anchor = GridBagConstraints.EAST;
				statMeseC[i - 1] = new JLabel();
				statMeseC[i - 1].setFont(nuovo);
				statMeseCorrente.add(statMeseC[i - 1], g);
				g.gridx = gridX;
				g.anchor = GridBagConstraints.CENTER;
			}
			g.gridy = 6;
			g.anchor = GridBagConstraints.WEST;
			g.gridwidth = 2;
			g.gridx = 0;
			statMeseCorrente.add(new JLabel(
					"Numero di Feedback rilasciati nel mese corrente:"), g);
			statMeseC[5] = new JLabel();
			statMeseC[5].setFont(new Font("Dialog", Font.BOLD, 18));
			g.gridx = 2;
			g.gridwidth = 1;
			statMeseCorrente.add(statMeseC[5], g);

		}
		return statMeseCorrente;
	}

	/**
	 * Questo metodo inizializza il pannello relativo alle statistiche totali
	 * del bene culturale.	
	 * 	
	 * @return javax.swing.JPanel - il pannello delle statistiche totali.
	 * 	
	 */
	private JPanel getStatTotali()
	{
		if (statTotali == null)
		{
			statTotali = new JPanel();
			statTotali.setLayout(new GridBagLayout());
			statTotali.setPreferredSize(new Dimension(500, 280));
			statTotali.setBorder(BorderFactory.createTitledBorder(BorderFactory
					.createLineBorder(new Color(51, 102, 255), 3),
					"Statistiche Totali", TitledBorder.DEFAULT_JUSTIFICATION,
					TitledBorder.DEFAULT_POSITION, new Font("Dialog",
							Font.BOLD, 12), new Color(0, 102, 204)));
			statTotali.setBackground(Color.white);
			statT = new JLabel[6];
			GridBagConstraints g = new GridBagConstraints();
			g.gridx = 0;
			g.gridy = 0;
			g.gridwidth = 3;
			g.insets = new Insets(5, 5, 5, 8);
			g.anchor = GridBagConstraints.WEST;
			statTotali.add(new JLabel(
					"Dettaglio voti ricevuti nel mese corrente:"), g);
			g.anchor = GridBagConstraints.CENTER;
			g.gridwidth = 1;
			Font nuovo = new Font("Dialog", Font.BOLD, 16);
			for (int i = 5; i >= 1; i--)
			{
				int gridX = g.gridx;
				g.gridy++;
				JLabel aLabel = new JLabel(new ImageIcon(getClass()
						.getResource(
								"/unisa/gps/etour/gui/images/stella" + i
										+ ".gif")), JLabel.CENTER);
				statTotali.add(aLabel, g);
				g.gridx++;
				statTotali.add(new JLabel("=="), g);
				g.gridx++;
				g.anchor = GridBagConstraints.EAST;
				statT[i - 1] = new JLabel();
				statT[i - 1].setFont(nuovo);
				statTotali.add(statT[i - 1], g);
				g.gridx = gridX;
				g.anchor = GridBagConstraints.CENTER;
			}
			g.gridy = 6;
			g.anchor = GridBagConstraints.WEST;
			g.gridwidth = 2;
			g.gridx = 0;
			statTotali.add(new JLabel(
					"Numero di Feedback rilasciati nel mese corrente:"), g);
			statT[5] = new JLabel();
			statT[5].setFont(new Font("Dialog", Font.BOLD, 18));
			g.gridx = 2;
			g.gridwidth = 1;
			statTotali.add(statT[5], g);
		}
		return statTotali;
	}
	
	private BeanBeneCulturale riversaDatiNelBean()
	{
		BeanBeneCulturale nuovo = new BeanBeneCulturale();
		nuovo.setNome(nomeBC.getText());
		nuovo.setDescrizione(descrizioneBC.getText());
		nuovo.setCap(capBC.getText());
		nuovo.setCostoBiglietto(Double.parseDouble(costoBC.getText()));
		nuovo.setGiornoChiusura((String)giornoChiusura.getSelectedItem());
		nuovo.setTelefono(telefonoBC.getText());
		nuovo.setCitta(cittaBC.getText());
		nuovo.setLocalita(localitaBC.getText());
		//Date(int year, int month, int date, int hrs, int min) 
		Date orarioAP = new Date(0,0,0,oreAP.getSelectedIndex(),
		minAP.getSelectedIndex());
		Date orarioCH = new Date(0,0,0,oreCH.getSelectedIndex(),
		minCH.getSelectedIndex());
		nuovo.setOrarioApertura(orarioAP);
		nuovo.setOrarioChiusura(orarioCH);
		nuovo.setProvincia((String)provBC.getSelectedItem());
		nuovo.setVia(((String)indirizzo1.getSelectedItem()) + " "+ indirizzo2.getText());
		Punto3D posizione = new Punto3D(
				Double.parseDouble(posGeoX.getText()),
				Double.parseDouble(posGeoY.getText()),
				Double.parseDouble(posGeoZ.getText()));
		nuovo.setPosizione(posizione);
		return nuovo;
	}
	
	private void caricaTags()
	{
		ArrayList<BeanTag> beanTags = null;
		try
		{
			beanTags = tag.ottieniTags();
			if (bc != null)
			{
				idTag = new ArrayList<Integer>();
				ArrayList<BeanTag> tagDaSelezionare = 
					gestioneBC.ottieniTagBeneCulturale(bc.getId());
				for (BeanTag b : tagDaSelezionare)
				{
					idTag.add(b.getId());
				}				
			}
		}
		// In caso di errore il pannello tag rimane vuoto.
		catch (RemoteException e)
		{
		}
		finally 
		{
			for (BeanTag b: beanTags)
			{
				pannelloTag.insertTag(b);
			}
			pannelloTag.setSelectedTags(idTag);
			pannelloTag.repaint();	
		}
	}
}
