/*
 * Turisti.java
 *
 * 1.0
 *
 * 26/05/2007
 *
 *  2007 eTour Project - Copyright by SE@SA Lab - DMI  University of Salerno
 */
package unisa.gps.etour.gui.operatoreagenzia;

import java.awt.BorderLayout;
import javax.swing.JPanel;
import javax.swing.JInternalFrame;
import java.awt.Dimension;

import javax.swing.JDesktopPane;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextPane;
import javax.swing.JToggleButton;
import javax.swing.JToolBar;
import javax.swing.JButton;
import javax.swing.ListSelectionModel;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import javax.swing.BorderFactory;
import javax.swing.JLabel;
import javax.swing.SwingConstants;
import java.awt.Color;
import java.awt.Insets;
import javax.swing.JTextField;
import javax.swing.JOptionPane;
import javax.swing.border.TitledBorder;
import javax.swing.event.InternalFrameAdapter;
import javax.swing.event.InternalFrameEvent;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import java.awt.Font;
import javax.swing.ImageIcon;
import unisa.gps.etour.util.Data;
import unisa.gps.etour.bean.BeanTurista;
import unisa.gps.etour.gui.DeskManager;
import unisa.gps.etour.gui.operatoreagenzia.tables.AttivazioneTuristaRenderer;
import unisa.gps.etour.gui.operatoreagenzia.tables.ScrollableTable;
import unisa.gps.etour.gui.operatoreagenzia.tables.TuristiTableModel;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.Date;

/**
 * Questa classe realizza l'interfaccia per la gestione dei turisti
 * da parte dell'Operatore Agenzia.
 * 
 * @version 1.0
 * @see javax.swing.JInternalFrame
 * @author Mario Gallo
 * 
 */
public class Turisti extends JInternalFrame
{

	private JPanel jContentPane = null;
	private JToolBar toolbarTuristi = null;
	private JButton btnModificaDati = null;
	private JScrollPane scrollTableTuristi = null;
	private JTable tableTuristi = null;
	private JPanel searchPanel = null;
	private JTextField cognomeTurista = null;
	private JPanel panelHelp = null;
	private JTextPane textGuida = null;
	private JPanel rightPanel = null;
	private JButton btnAttiva;
	private JButton btnVisualizzaScheda;
	private JButton btnElimina;
	private TuristiTableModel tableModel;
	private JToggleButton turistiDisattivati;
	private JToggleButton turistiAttivati;
	private JButton btnAzzera;
	private JButton btnRicerca;
	protected DeskManager desktopManager;
	protected JDesktopPane jDesktopPane;
	private ArrayList<schedaturista> figli;

	/**
	 * Questo  il costruttore di default.
	 * 
	 */
	public Turisti()
	{
		super("Turisti");
		setPreferredSize(new Dimension(700, 480));
		frameIcon = new ImageIcon(getClass().getResource(
				"/unisa/gps/etour/gui/operatoreagenzia/images/Turisti.png"));
		setDefaultCloseOperation(JInternalFrame.DO_NOTHING_ON_CLOSE);
		closable = true;
		resizable = true;
		iconable = true;
		maximizable = true;
		setContentPane(getJContentPane());
		figli = new ArrayList<schedaturista>();
		addInternalFrameListener(new InternalFrameAdapter()
		{
			public void internalFrameOpened(InternalFrameEvent pEvent)
			{
				jDesktopPane = pEvent.getInternalFrame().getDesktopPane();
				desktopManager = (DeskManager) jDesktopPane.getDesktopManager();
			}
			
			public void internalFrameClosing(InternalFrameEvent pEvent)
			{
				JPanel root = new JPanel(new BorderLayout());
				JLabel message = new JLabel(
						"Sei sicuro di voler chiudere la finestra di gestione dei turisti?");
				message.setFont(new Font("Dialog", Font.BOLD, 14));
				JLabel avviso = new JLabel(
						"N.B. Verranno chiuse tutte le finestre aperte da questa gestione.",
						SwingConstants.CENTER);
				avviso
						.setIcon(new ImageIcon(
								getClass()
										.getResource(
												"/unisa/gps/etour/gui/operatoreagenzia/images/warning16.png")));
				root.add(message, BorderLayout.NORTH);
				root.add(avviso, BorderLayout.SOUTH);
				String[] options = { "Chiudi", "Annulla" };
				int choice = JOptionPane.showInternalOptionDialog(jContentPane, root, "Conferma chiusura Turisti",
						JOptionPane.OK_CANCEL_OPTION, JOptionPane.QUESTION_MESSAGE, frameIcon, options, options[1]);  
				if (choice == JOptionPane.OK_OPTION)
				{
					for (int i = 0; i < figli.size(); i++)
					{
						figli.get(i).dispose();
					}
					pEvent.getInternalFrame().dispose();
				}
			}
		});
	}

	/**
	 * Questo metodo inizializza il content pane del frame interno.
	 * 
	 * @return javax.swing.JPanel - il content pane.
	 * 
	 */
	private JPanel getJContentPane()
	{
		if (jContentPane == null)
		{
			jContentPane = new JPanel();
			jContentPane.setLayout(new BorderLayout());
			jContentPane.add(getToolbarTuristi(), BorderLayout.NORTH);
			jContentPane.add(getRightPanel(), BorderLayout.EAST);
			jContentPane.add(getScrollTableTuristi(), BorderLayout.CENTER);
		}
		return jContentPane;
	}

	/**
	 * Questo metodo inizializza la toolbar relativa alla gestione dei
	 * turisti.	
	 * 	
	 * @return javax.swing.JToolBar - la toolbar.
	 * 	
	 */
	private JToolBar getToolbarTuristi()
	{
		if (toolbarTuristi == null)
		{
			toolbarTuristi = new JToolBar();
			toolbarTuristi.setLayout(null);
			toolbarTuristi.setPreferredSize(new Dimension(1, 50));
			toolbarTuristi.setOrientation(JToolBar.HORIZONTAL);
			toolbarTuristi.setFloatable(false);
			toolbarTuristi.add(getBtnAttiva());
			toolbarTuristi.add(getBtnModificaDati());
			toolbarTuristi.add(getBtnVisualizzaScheda());
			toolbarTuristi.add(getBtnElimina());

		}
		return toolbarTuristi;
	}

	/**
	 * Questo metodo inizializza il pulsante per l'attivazione 
	 * di un turista.
	 * 
	 * @return javax.swing.JButton - il pulsante.
	 * 
	 */
	private JButton getBtnAttiva()
	{
		if (btnAttiva == null)
		{
			btnAttiva = new JButton();
			btnAttiva.setText("<html>Attiva<br>Turista</html>");
			btnAttiva.setBounds(5, 5, 140, 40);
			btnAttiva.setEnabled(false);
			btnAttiva
					.setIcon(new ImageIcon(
							getClass()
									.getResource(
											"/unisa/gps/etour/gui/operatoreagenzia/images/AttivaTurista32.png")));

			btnAttiva.addActionListener(new ActionListener()
			{
				public void actionPerformed(ActionEvent arg0)
				{
					int selected = tableTuristi.getSelectedRow();
					String[] campi = { "Nome", "Cognome", "Data di Nascita",
							"Luogo di Nascita", "E-Mail", "Telefono",
							"Indirizzo", "CAP", "Provincia", "Citt" };
					int[] modelReference = { 1, 2, 5, 6, 3, 4, 7, 9, 10, 8 };
					JPanel datiTurista = new JPanel(new GridBagLayout());
					JPanel rootDialog = new JPanel(new GridBagLayout());
					datiTurista.setBorder(BorderFactory.createTitledBorder(
							BorderFactory.createLineBorder(new Color(51, 102,
									255), 2), "Dati Turista",
							TitledBorder.DEFAULT_JUSTIFICATION,
							TitledBorder.DEFAULT_POSITION, new Font("Dialog",
									Font.BOLD, 12), new Color(0, 102, 204)));
					GridBagConstraints g = new GridBagConstraints();
					g.insets = new Insets(5, 5, 5, 5);
					g.anchor = GridBagConstraints.WEST;
					g.gridx = 0;
					g.gridy = 0;
					for (int i = 0; i < campi.length; i++)
					{
						datiTurista.add(new JLabel(campi[i]), g);
						g.gridx++;
						if (i == 2) //Data di Nascita
						{
							Date dataNascita = (Date) tableModel.getValueAt(
									selected, 5);
							datiTurista.add(new JLabel(
									Data.toCompact(dataNascita)), g);

						}
						else
						{
							datiTurista.add(new JLabel(
								tableModel.getValueAt(
									selected, modelReference[i]).toString()), g);
						}
						g.gridx--;
						g.gridy++;
					}
					g.gridx = 0;
					g.gridy = 0;
					rootDialog.add(datiTurista, g);
					g.gridy++;
					JLabel txtAttiva = new JLabel();
					rootDialog.add(txtAttiva, g);
					String[] options = new String[2];
					ImageIcon iconDialog;
					options[1] = "Annulla";
					String title;
					boolean attivato = (Boolean) tableModel.getValueAt(
							selected, 0);
					if (attivato)  //Operazione di disattivazione
					{
						title = "Disattivare il turista "
								+ tableModel.getValueAt(selected, 1) + " "
								+ tableModel.getValueAt(selected, 2) + "?";
						options[0] = "Disattiva";
						txtAttiva
								.setText("Disattivare il turista selezionato?");
						iconDialog = new ImageIcon(
								getClass()
										.getResource(
												"/unisa/gps/etour/gui/operatoreagenzia/images/DisattivaTurista48.png"));
					}
					else			//Operazione di attivazione	
					{
						title = "Attivare il turista "
								+ tableModel.getValueAt(selected, 1) + " "
								+ tableModel.getValueAt(selected, 2) + "?";
						options[0] = "Attiva";
						txtAttiva.setText("Attivare il turista selezionato?");
						iconDialog = new ImageIcon(
								getClass()
										.getResource(
												"/unisa/gps/etour/gui/operatoreagenzia/images/AttivaTurista48.png"));
					}
					txtAttiva.setForeground(Color.red);
					int choice = JOptionPane.showInternalOptionDialog(
							jContentPane, rootDialog, title,
							JOptionPane.OK_CANCEL_OPTION,
							JOptionPane.QUESTION_MESSAGE, iconDialog, options,
							options[1]);
					if (choice == JOptionPane.OK_OPTION)
					{
						tableModel.setValueAt((attivato) ? false : true,
								selected, 0);
						tableModel.fireTableDataChanged();
					}
				}
			});
		}
		return btnAttiva;
	}

	/**
	 * Questo metodo inizializza il pulsante per la modifica dei 
	 * dati di un turista.
	 * 	
	 * @return javax.swing.JButton	- il pulsante.
	 */
	private JButton getBtnModificaDati()
	{
		if (btnModificaDati == null)
		{
			btnModificaDati = new JButton();
			btnModificaDati
					.setText("<html>Modifica&nbsp;Dati<br>Turista</html>");
			btnModificaDati.setBounds(155, 5, 140, 40);
			btnModificaDati.setEnabled(false);
			btnModificaDati
					.setIcon(new ImageIcon(
							getClass()
									.getResource(
											"/unisa/gps/etour/gui/operatoreagenzia/images/ModificaTurista32.png")));
			btnModificaDati.addActionListener(new ActionListener()
			{
				public void actionPerformed(ActionEvent arg0)
				{
					//TEST
					Date nuova = new Date();
					nuova.setMonth(12);
					nuova.setDate(30);
					nuova.setYear(82);
					Date nuova2 = new Date();
					nuova2.setMonth(4);
					nuova2.setDate(30);
					nuova2.setYear(107);
					BeanTurista TEST = new BeanTurista(0, "mlmlml", "Ottabio",
							"degli Michili", "Ottawa", "Frattamaggiore", "61616161",
							"84932", "V.le delle Mimose 33", "NA",
							"ottavio_ottawa@wawa.com", "micacae", nuova, nuova2, false);
					//TEST
					openScheda(TEST,true);
				}
			});
		}
		return btnModificaDati;
	}

	/**
	 * Questo metodo inizializza il pulsante per la visualizzazione della
	 * scheda di un turista.
	 * 
	 * @return javax.swing.JButton - il pulsante.
	 */
	private JButton getBtnVisualizzaScheda()
	{
		if (btnVisualizzaScheda == null)
		{
			btnVisualizzaScheda = new JButton();
			btnVisualizzaScheda
					.setText("<html>Visualizza<br>profilo&nbsp;Turista</html>");
			btnVisualizzaScheda.setBounds(305, 5, 140, 40);
			btnVisualizzaScheda
					.setIcon(new ImageIcon(
							getClass()
									.getResource(
											"/unisa/gps/etour/gui/operatoreagenzia/images/schedaturista.png")));
			btnVisualizzaScheda.setEnabled(false);
			btnVisualizzaScheda.addActionListener(new ActionListener()
			{

				public void actionPerformed(ActionEvent arg0)
				{
					//TEST
					Date nuova = new Date();
					nuova.setMonth(12);
					nuova.setDate(30);
					nuova.setYear(82);
					Date nuova2 = new Date();
					nuova2.setMonth(4);
					nuova2.setDate(30);
					nuova2.setYear(107);
					BeanTurista TEST = new BeanTurista(0, "mlmlml", "Ottabio",
							"degli Michili", "Ottawa", "Frattamaggiore", "61616161",
							"84932", "V.le delle Mimose 33", "NA",
							"ottavio_ottawa@wawa.com", "micacae", nuova, nuova2, false);
					//TEST
					openScheda(TEST,false);
				}
				
			});
		}
		return btnVisualizzaScheda;
	}

	/**
	 * 
	 * Questo metodo inizializza il pulsante per l'eliminazione di un turista.
	 *
	 * @return javax.swing.JButton - il pulsante.
	 */
	private JButton getBtnElimina()
	{
		if (btnElimina == null)
		{
			btnElimina = new JButton();
			btnElimina.setText("<html>Elimina<br>Turista</html>");
			btnElimina.setBounds(455, 5, 140, 40);
			btnElimina.setEnabled(false);
			btnElimina
					.setIcon(new ImageIcon(
							getClass()
									.getResource(
											"/unisa/gps/etour/gui/operatoreagenzia/images/EliminaTurista32.png")));
			btnElimina.addActionListener(new ActionListener()
			{

				public void actionPerformed(ActionEvent arg0)
				{
					int selectedRow = tableTuristi.getSelectedRow();
					if (selectedRow != -1)
					{
						String nome = (String) tableModel.getValueAt(
								selectedRow, 1)
								+ " "
								+ (String) tableModel
										.getValueAt(selectedRow, 2);
						JPanel root = new JPanel(new BorderLayout());
						JLabel message = new JLabel(
								"Sei sicuro di voler eliminare l'account del turista "
										+ nome + "?");
						message.setFont(new Font("Dialog", Font.BOLD, 14));
						JLabel avviso = new JLabel(
								"I dati dell'account e tutte le impostazioni personali"
										+ " non potranno essere pi recuperati.",
								SwingConstants.CENTER);
						avviso
								.setIcon(new ImageIcon(
										getClass()
												.getResource(
														"/unisa/gps/etour/gui/operatoreagenzia/images/warning16.png")));
						root.add(message, BorderLayout.NORTH);
						root.add(avviso, BorderLayout.SOUTH);
						String[] options = { "Elimina", "Annulla" };
						int choice = JOptionPane
								.showInternalOptionDialog(
										jContentPane,
										root,
										"Conferma Eliminazione",
										JOptionPane.YES_NO_OPTION,
										JOptionPane.QUESTION_MESSAGE,
										new ImageIcon(
												getClass()
														.getResource(
																"/unisa/gps/etour/gui/operatoreagenzia/images/EliminaTurista48.png")),
										options, options[1]);
						if (choice == JOptionPane.YES_OPTION)
						{
							tableModel.removeTurista(selectedRow);
							tableModel.fireTableDataChanged();
							JLabel confirm = new JLabel("L'account Turista di " + nome
									+ "  stato eliminato.");
							confirm.setFont(new Font("Dialog", Font.BOLD, 14));
							JOptionPane
									.showInternalMessageDialog(
											jContentPane,
											confirm,
											"Account Turista eliminato!",
											JOptionPane.OK_OPTION,
											new ImageIcon(
													getClass()
															.getResource(
																	"/unisa/gps/etour/gui/operatoreagenzia/images/ok32.png")));
						}
					}

				}

			});
		}
		return btnElimina;
	}

	/**
	 * Questo metodo inizializza lo scroll pane con la tabella dei turisti.	
	 * 	
	 * @return javax.swing.JScrollPane - lo scroll pane.
	 */
	private JScrollPane getScrollTableTuristi()
	{
		if (scrollTableTuristi == null)
		{
			tableModel = new TuristiTableModel();
			//TEST
			Date nuova = new Date();
			nuova.setMonth(12);
			nuova.setDate(30);
			nuova.setYear(82);
			Date nuova2 = new Date();
			nuova2.setMonth(4);
			nuova2.setDate(30);
			nuova2.setYear(107);
			BeanTurista nuovo = new BeanTurista(0, "mlmlml", "Ottabio",
					"degli Michili", "Ottawa", "Frattamaggiore", "61616161",
					"84932", "V.le delle Mimose 33", "NA",
					"ottavio_ottawa@wawa.com", "micacae", nuova, nuova2, false);
			BeanTurista nuovo2 = new BeanTurista(1, "mlmlml", "Ottabiolino",
					"degli Michili", "Ottawa", "Frattamaggiore", "61616161",
					"84932", "V.le delle Mimose 33", "NA",
					"ottavio_ottawa@wawa.com", "micacae", nuova, nuova2, true);
			BeanTurista nuovo3 = new BeanTurista(2, "mlmlml", "Ottavio",
					"Michili", "Ottawa", "Frattamaggiore", "61616161", "84932",
					"V.le Mimose 33", "NA", "ottavio_ottawa@wawa.com",
					"micacae", nuova, nuova2, false);
			tableModel.insertTurista(nuovo);
			tableModel.insertTurista(nuovo2);
			tableModel.insertTurista(nuovo3);
			for (int i = 0; i < 12; i++)
			{
				tableModel.insertTurista(nuovo);
			}
			// FINE TEST
			tableTuristi = new ScrollableTable(tableModel);
			tableTuristi.setAutoCreateColumnsFromModel(true);
			tableTuristi.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
			tableTuristi.setSelectionForeground(Color.RED);
			tableTuristi.setSelectionBackground(Color.white);
			tableTuristi.setColumnSelectionAllowed(false);
			tableTuristi.setShowVerticalLines(false);
			tableTuristi.setRowHeight(32);
			//Status
			tableTuristi.getColumnModel().getColumn(0).setPreferredWidth(40);
			tableTuristi.getColumnModel().getColumn(0).setCellRenderer(
					new AttivazioneTuristaRenderer());
			//Nome
			tableTuristi.getColumnModel().getColumn(1).setPreferredWidth(100);
			//Cognome
			tableTuristi.getColumnModel().getColumn(2).setPreferredWidth(100);
			//Email
			tableTuristi.getColumnModel().getColumn(3).setPreferredWidth(140);
			//Telefono
			tableTuristi.getColumnModel().getColumn(4).setPreferredWidth(80);
			//Data di Nascita
			tableTuristi.getColumnModel().getColumn(5).setPreferredWidth(100);
			//Citt di Nascita
			tableTuristi.getColumnModel().getColumn(6).setPreferredWidth(100);
			//Indirizzo
			tableTuristi.getColumnModel().getColumn(7).setPreferredWidth(100);
			//Residenza
			tableTuristi.getColumnModel().getColumn(8).setPreferredWidth(80);
			//CAP
			tableTuristi.getColumnModel().getColumn(9).setPreferredWidth(60);
			//Provincia
			tableTuristi.getColumnModel().getColumn(10).setPreferredWidth(30);
			//Data Registrazione
			tableTuristi.getColumnModel().getColumn(11).setPreferredWidth(90);

			ListSelectionModel selectionModel = tableTuristi
					.getSelectionModel();
			selectionModel.addListSelectionListener(new ListSelectionListener()
			{
				public void valueChanged(ListSelectionEvent event)
				{
					int selectedRow = tableTuristi.getSelectedRow();
					if (selectedRow != -1)
					{
						btnAttiva.setEnabled(true);
						btnModificaDati.setEnabled(true);
						btnVisualizzaScheda.setEnabled(true);
						btnElimina.setEnabled(true);
						if (!(Boolean) tableModel.getValueAt(selectedRow, 0))
						{
							btnAttiva.setText("<html>Attiva<br>Turista</html>");
							btnAttiva
									.setIcon(new ImageIcon(
											getClass()
													.getResource(
															"/unisa/gps/etour/gui/operatoreagenzia/images/AttivaTurista32.png")));
						}
						else
						{
							btnAttiva
									.setText("<html>Disattiva<br>Turista</html>");
							btnAttiva
									.setIcon(new ImageIcon(
											getClass()
													.getResource(
															"/unisa/gps/etour/gui/operatoreagenzia/images/DisattivaTurista32.png")));
						}
					}
					else
					{
						btnElimina.setEnabled(false);
						btnModificaDati.setEnabled(false);
						btnAttiva.setEnabled(false);
						btnVisualizzaScheda.setEnabled(false);
					}

				}
			});
			scrollTableTuristi = new JScrollPane();
			scrollTableTuristi.setViewportView(tableTuristi);
			scrollTableTuristi
					.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);
			scrollTableTuristi
					.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
		}
		return scrollTableTuristi;
	}

	/**
	 * Questo metodo inizializza il form per la ricerca dei turisti.	
	 * 	
	 * @return javax.swing.JPanel	
	 * 
	 */
	private JPanel getSearchPanel()
	{
		if (searchPanel == null)
		{
			searchPanel = new JPanel();
			searchPanel.setLayout(new GridBagLayout());
			searchPanel.setBorder(BorderFactory.createTitledBorder(
					BorderFactory.createLineBorder(new Color(51, 102, 255), 3),
					"Ricerca Turisti", TitledBorder.DEFAULT_JUSTIFICATION,
					TitledBorder.DEFAULT_POSITION, new Font("Dialog",
							Font.BOLD, 12), new Color(0, 102, 204)));
			GridBagConstraints g = new GridBagConstraints();
			//Creazione Pannello scelta turisti
			JPanel turisti = new JPanel(new GridBagLayout());
			turisti.setPreferredSize(new Dimension(200, 120));
			turisti.setBorder(BorderFactory.createLoweredBevelBorder());
			turistiAttivati = new JToggleButton("Turisti Attivati");
			turistiAttivati.setPreferredSize(new Dimension(165, 30));
			turistiDisattivati = new JToggleButton("Turisti Disattivati");
			turistiDisattivati.setPreferredSize(new Dimension(165, 30));
			turistiAttivati
					.setIcon(new ImageIcon(
							getClass()
									.getResource(
											"/unisa/gps/etour/gui/operatoreagenzia/images/schedaturista24.png")));
			ActionListener almenoUno = new ActionListener()
			{
				public void actionPerformed(ActionEvent pEvent)
				{
					if (pEvent.getSource() == turistiAttivati)
					{
						if (!turistiAttivati.isSelected()
								&& !turistiDisattivati.isSelected())
						{
							turistiDisattivati.setSelected(true);
						}
					}
					else
					{
						if (!turistiAttivati.isSelected()
								&& !turistiDisattivati.isSelected())
						{
							turistiAttivati.setSelected(true);
						}
					}
					
				}
			};
			turistiAttivati.addActionListener(almenoUno);
			turistiDisattivati.addActionListener(almenoUno);
			turistiAttivati.setSelected(true);
			turistiDisattivati.setSelected(true);
			turistiDisattivati
					.setIcon(new ImageIcon(
							getClass()
									.getResource(
											"/unisa/gps/etour/gui/operatoreagenzia/images/DisattivaTurista24.png")));
			g.insets = new Insets(8, 8, 8, 8);
			g.gridx = 0;
			g.gridy = 0;
			g.weighty = 0.2;
			g.fill = GridBagConstraints.HORIZONTAL;
			turisti.add(turistiAttivati, g);
			g.gridy++;
			turisti.add(turistiDisattivati, g);

			//Creazione pannello search
			g.gridx = 0;
			g.gridy = 0;
			g.gridwidth = 2;
			g.anchor = GridBagConstraints.CENTER;
			g.fill = GridBagConstraints.NONE;
			g.insets = new Insets(5, 5, 5, 5);
			searchPanel.add(new JLabel("Cognome Turista:"), g);
			cognomeTurista = new JTextField(12);
			g.gridy++;
			searchPanel.add(cognomeTurista, g);
			g.gridy++;
			searchPanel.add(new JLabel("Visualizza:"), g);
			g.gridy++;
			g.weighty = 0.2;
			g.insets = new Insets(5, 5, 20, 5);
			searchPanel.add(turisti, g);
			g.insets = new Insets(5,5,5,5);
			g.gridwidth = 1;
			g.gridy++;
			searchPanel.add(getBtnRicerca(), g);
			g.gridx = 1;
			searchPanel.add(getBtnAzzera(), g);
		}
		return searchPanel;
	}

	/**
	 * Questo metodo inizializza il pannello della guida in linea.	
	 * 	
	 * @return javax.swing.JPanel	
	 * 
	 */
	private JPanel getPanelHelp()
	{
		if (panelHelp == null)
		{
			panelHelp = new JPanel();
			panelHelp.setLayout(new BorderLayout());
			panelHelp.setBorder(BorderFactory.createTitledBorder(BorderFactory
					.createLineBorder(new Color(51, 102, 255), 3),
					"Guida in linea", TitledBorder.DEFAULT_JUSTIFICATION,
					TitledBorder.DEFAULT_POSITION, new Font("Dialog",
							Font.BOLD, 12), new Color(0, 102, 204)));
			panelHelp.setPreferredSize(new Dimension(200, 100));
			textGuida = new JTextPane();
			textGuida.setOpaque(false);
			textGuida.setContentType("text/html");
			textGuida.setEditable(false);
			textGuida.setOpaque(false);
			panelHelp.add(textGuida, BorderLayout.CENTER);

		}
		return panelHelp;
	}

	/**
	 * Questo metodo inizializza il pannello laterale dell'interfaccia.	
	 * 	
	 * @return javax.swing.JPanel	
	 * 
	 */
	private JPanel getRightPanel()
	{
		if (rightPanel == null)
		{
			rightPanel = new JPanel();
			rightPanel.setLayout(new GridBagLayout());
			GridBagConstraints g = new GridBagConstraints();
			g.gridx = 0;
			g.gridy = 1;
			g.fill = GridBagConstraints.BOTH;
			g.weighty = 0.3;
			rightPanel.add(getPanelHelp(), g);
			g.weighty = 0.7;
			g.gridx = 0;
			g.gridy = 0;
			rightPanel.add(getSearchPanel(), g);
		}
		return rightPanel;
	}

	/**
	 * Questo metodo inizializza il pulsante per la ricerca dei turisti.
	 * 	
	 * @return javax.swing.JButton
	 * 	
	 */
	private JButton getBtnRicerca()
	{
		if (btnRicerca == null)
		{
			btnRicerca = new JButton();
			btnRicerca.setText("Ricerca");
			btnRicerca.setPreferredSize(new Dimension(98, 26));
			btnRicerca
					.setIcon(new ImageIcon(
							getClass()
									.getResource(
											"/unisa/gps/etour/gui/operatoreagenzia/images/Ricerca16.png")));
		}
		return btnRicerca;
	}

	/**
	 * Questo metodo inizializza il pulsante per il reset del form di
	 * ricerca.	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getBtnAzzera()
	{
		if (btnAzzera == null)
		{
			btnAzzera = new JButton();
			btnAzzera.setText("Azzera");
			btnAzzera.setHorizontalTextPosition(SwingConstants.LEADING);
			btnAzzera.setPreferredSize(new Dimension(98, 26));
			btnAzzera
					.setIcon(new ImageIcon(
							getClass()
									.getResource(
											"/unisa/gps/etour/gui/operatoreagenzia/images/Azzera16.png")));
			btnAzzera.addActionListener(new ActionListener()
			{
				public void actionPerformed(ActionEvent pAcEvent)
				{
					cognomeTurista.setText("");
					turistiAttivati.setSelected(true);
					turistiDisattivati.setSelected(true);
				}
			});
		}
		return btnAzzera;
	}
	
	/**
	 * 
	 * Questo metodo apre una scheda relativa al bean turista fornito in input.
	 *
	 * @param pTurista BeanTurista - il bean del turista
	 * @param pModifica boolean <ul><li>true - se si modificano i dati.
	 * <li>false - se si visualizza la scheda.</ul>
	 * @return void
	 */
	private void openScheda(BeanTurista pTurista, boolean pModifica)
	{
		for (int i = 0; i < figli.size(); i++)
		{
			schedaturista corrente = figli.get(i);
			if (pTurista.getId() == corrente.getId()) 
			{
				desktopManager.activateFrame(corrente);
				return;
			}
		}
		schedaturista nuova = new schedaturista(this, pTurista, pModifica);
		jDesktopPane.add(nuova, Integer.MAX_VALUE);
		nuova.setVisible(true);
		figli.add(nuova);
	}
	
	/**
	 * 
	 * Chiude la scheda selezionata.
	 *
	 * @param pScheda schedaturista - la scheda da chiudere.
	 * @return void
	 */
	protected void closeScheda(schedaturista pScheda)
	{
		figli.remove(pScheda);
		pScheda.dispose();
	}
	
	/**
	 * 
	 * Aggiorna il modello della tabella dei turisti con il bean fornito in
	 * input.
	 *
	 * @param pTurista BeanTurista - il bean del turista.
	 * @return void
	 */
	protected void updateTableModel(BeanTurista pTurista)
	{
		tableModel.updateTurista(pTurista);
	}
}
